package com.myarch.antutil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.BuildException;
import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.filter.ElementFilter;

import com.myarch.dpbuddy.encryption.PasswordUtils;
import com.myarch.propmanagement.ExpressionEvaluator;

public class PropResolverForXML {
    private static Log logger=LogFactory.getLog( PropResolverForXML.class );

    private ExpressionEvaluator propExpressionEvaluator;

    public PropResolverForXML(ExpressionEvaluator propExpressionEvaluator) {
        this.propExpressionEvaluator=propExpressionEvaluator;
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    public void resolveVarsInXML( Document doc ) {
        resolveVarsInXML( doc, new HashMap() );
    }

    public void resolveVarsInXML( Document doc, Map<String, String> overridingProps ) {

        List<Element> unresolvedElements=new ArrayList<Element>();

        Iterator<Element> iter=doc.getDescendants( new ElementFilter() );
        while (iter.hasNext()) {
            Element elt=iter.next();
            String text=elt.getText();
            if (containsAntVars( text ) || overridingProps.get( elt.getName() ) != null) {
                unresolvedElements.add( elt );
            }

            // check for unresolved attributes
            List<Attribute> attrs=elt.getAttributes();
            for (Attribute attr : attrs) {
                if (containsAntVars( attr.getValue() ))
                    attr.setValue( replaceAntVars( attr.getValue(), elt ) );
            }
        }

        // now we can resolve elements
        for (Element elt : unresolvedElements) {
            String textToResolve=overridingProps.get( elt.getName() );
            if (textToResolve == null) {
                textToResolve=elt.getText();
            }
            else {
                // we're removing the props that we used up so we can figure out
                // if
                // there is any misspelled property
                overridingProps.remove( elt.getName() );
                logger.debug( "Overriding element " + elt.getName()
                    + " with the value provided in configTemplate: " + textToResolve );
            }
            // we still need to resolve new value, it may contain variables
            elt.setText( replaceAntVars( textToResolve, elt ) );
        }

        if (overridingProps.size() > 0) {
            String unresolvedKeys="";
            for (String key : overridingProps.keySet())
                unresolvedKeys+=" " + key;
            throw new BuildException( "You provided replacement properties that were not found in XML: "
                + unresolvedKeys );
        }
    }

    private String replaceAntVars( String unresolvedText, Element elt ) {
        if (!unresolvedText.contains( "${" ))
            return unresolvedText;

        String resolvedText=propExpressionEvaluator.evaluate( unresolvedText );

        if (resolvedText.contains( "${" ))
            throw new BuildException(
                            "XML element '" + elt.getName() + "' contains an unresolved property: "
                                + resolvedText + " Please make sure this property is defined." );

        logger.debug( "Resolving property: " + unresolvedText + "   Result: "
            + PasswordUtils.maskIfNeeded( elt.getName(), resolvedText ) );

        return resolvedText;
    }

    private boolean containsAntVars( String text ) {
        return text != null && text.trim().length() > 0 && text.contains( "${" );
    }

}
