package com.myarch.antutil;

import java.io.File;
import java.util.List;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;


public class TaskParameterValidator {
    
    private Project project;
    
    public TaskParameterValidator( Task task){
        this.project=task.getProject();
    }

    public TaskParameterValidator( Project project ){
        this.project=project;
    }
    
    private ValidationErrorContainer errors = new ValidationErrorContainer();

    public boolean validateRequiredProperty(String name) {
        return validateRequiredProperty(name, null);
    }
    
    public boolean validateRequiredProperty( String name, String desc ) {
        return validateRequired(project.getProperty(name), name, desc );
    }
    
    /**
     * @param value
     * @param name
     * @param desc
     * @return true if passed
     */
    
    public boolean validateRequired(String value, String name, String desc ) {
        boolean passed = true;
        String descMsg = "";
        if (desc!=null)
            descMsg = "("+desc+")";
        
        if (value == null || value.trim().length()==0 ) {
            errors.add("You must specify attribute \'"+name+"\' "+descMsg);
            passed=false;
        }
        else if ( value.contains("${")) {
            errors.add("Property or attribute \'"+name+"\' "+descMsg+" must be fully resolved");
            passed=false;
        }
        
        return passed;
    }

    public boolean validateRequired(Integer value, String name, String desc ) {
        boolean passed = true;
        String descMsg = "";
        if (desc!=null)
            descMsg = "("+desc+")";
        
        if (value == null ) {
            errors.add("You must specify attribute \'"+name+"\' "+descMsg);
            passed=false;
        }
        return passed;
    }

    
    public boolean validateRequired(File file, String name, String desc ) {
        boolean passed = true;
        String descMsg = "";
        if (desc!=null)
            descMsg = "("+desc+")";
        
        if (file == null ) {
            errors.add("You must specify attribute \'"+name+"\' "+descMsg);
            passed=false;
        }

        return passed;
    }
    
    
    public boolean validateExistingFile(File file, String name, String desc ) {
        boolean passed = true;
        String descMsg = "";
        if (desc!=null)
            descMsg = "("+desc+")";
        
        if (file == null  ) {
            errors.add("You must specify attribute \'"+name+"\' "+descMsg);
            passed=false;
        } 
        else if ( ! file.exists()) {
            errors.add(String.format("File '%s' %s does not exist", file.getAbsolutePath(), descMsg));
            passed=false;
            
        }
        
        return passed;
    }

    
    public boolean validateRequiredNested( @SuppressWarnings("rawtypes") List nestedList, String name  ){
        boolean passed = true;
        if (nestedList==null || nestedList.size()==0) {
            errors.add("You must specify at least one nested element \""+name+"\" ");
            passed=false;
        }
        return passed;
    }
    
    public ValidationErrorContainer getErrors(){
        return errors;
    }
    
    public void throwIfErrors() throws ValidationErrorContainer {
        errors.throwIfErrors();
    }

}
