package com.myarch.antutil;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.tools.ant.BuildException;

/**
 * This is a wrapper class for the list of errors. 
 * Provides methods to add errors. 
 * 
 * @author Alexander Ananiev
 */
public class ValidationErrorContainer extends BuildException {
    
    private static final long serialVersionUID = -1105060787488259519L;

    private List<String> errors = new ArrayList<String>();
    
    public ValidationErrorContainer(){
    	super("Validation error(s)");
    }
    
    public ValidationErrorContainer(String error){
        super("Validation error(s)");
        add(error);
    }
    
    
    public void add( String error ){
        errors.add( error );
    }
    
    public Collection<String> getErrors(){
    	return errors;   
    }
    
    public boolean isEmpty() {
        return errors.size()==0;
    }
    
    /**
     * Throws self if it has at least one error added
     * 
     * @throws ValidationErrorContainer
     */
    public void throwIfErrors() throws ValidationErrorContainer {
    	if (errors.size()>0)
            throw this;
    }
    
    public String getMessage(){
        return errorsToString();
    }
    
    
    /**
     * Returns String representation of this object.
     * It includes the list of all business errors.
     * @see java.lang.Object#toString()
     */
    public String toString(){
        
        String text=""; 
        if (errors.size()>0) {
    	    text=getMessage();
        }
        return text;
    }
  
    
    private String errorsToString(){
        
        StringBuilder errorText = new StringBuilder();
        for(String error:errors ) {
            if (errorText.length()>0)
                errorText.append("\n");
            errorText.append(error);
        }
        
        return errorText.toString();
    }
}
