package com.myarch.antutil;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Expand;
import org.apache.tools.ant.taskdefs.Zip;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.PatternSet;

import com.myarch.dpbuddy.DPBuddyException;

public class ZipHelper {

    private Log logger=LogFactory.getLog( getClass() );

    private Project antProject;

    /**
     * @param antProject
     */
    public ZipHelper(Project antProject) {
        super();
        this.antProject=antProject;
    }

    public void unzip( File srcZipFile, File targetDir ) {
        unzip( srcZipFile, null, targetDir );
    }

    public void unzip( File srcZipFile, List<? extends PatternSet> patternSets, File targetDir ) {
        Expand expandTask=createExpandTask( srcZipFile, targetDir );

        if (patternSets != null) {
            for (PatternSet patternSet : patternSets) {
                expandTask.addPatternset( patternSet );
            }
        }
        expandTask.execute();

    }

    private Expand createExpandTask( File zipFile, File targetDir ) {
        Expand expandTask=new Expand();
        expandTask.setTaskName( "unzip" );
        expandTask.setProject( antProject );
        expandTask.setSrc( zipFile );
        expandTask.setDest( targetDir );

        return expandTask;
    }

    private Zip createZipTask( File zipFile ) {
        Zip zipTask=new Zip();
        zipTask.setTaskName( "zip" );
        zipTask.setProject( antProject );
        zipTask.setDestFile( zipFile );

        return zipTask;
    }

    public void zip( File baseDir, List<? extends PatternSet> patternSets, File zipFile ) {

        Zip zipTask=createZipTask( zipFile );
        zipTask.setUpdate( true );
        zipTask.setRoundUp( false );

        for (PatternSet patternSet : patternSets) {
            FileSet fileSet=AntUtils.createFileSetFromPatternSet( antProject, baseDir, patternSet );
            // make sure we don't grab the zip file itself
            fileSet.appendExcludes( new String[] { "*.zip" } );
            zipTask.addFileset( fileSet );
        }

        zipTask.execute();
    }

    /**
     * Zip up the entire directory. Delete target zip if exists.
     * 
     * @param baseDir
     * @param zipFile
     */
    public void zip( File baseDir, File destZipFile ) {
        logger.debug( "Zipping up the content of " + baseDir.getAbsolutePath() + "\nDestination file: "
            + destZipFile.getAbsolutePath() );
        if (destZipFile.exists()) {
            try {
//                FileUtils.forceDelete( destZipFile );
                Files.delete(destZipFile.toPath() );
            }
            catch (IOException e) {
                throw new DPBuddyException( e );
            }
        }
        Zip zipTask=createZipTask( destZipFile );
        zipTask.setBasedir( baseDir );
        zipTask.execute();
    }

}
