package com.myarch.dpbuddy;

import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.xmlutil.JDomUtils;

public class BaseResponse implements DPResponse {

    
    protected Document doc;
   
    public BaseResponse( Document doc ){
        this.doc = doc;
    }
    
    public Document getXMLDocument() {
        return doc;
    }
    
    protected List<Element> getRequiredResponseElements(String eltName) {
        List<Element> elts = getResponseElements(eltName);

        if ( elts.size()==0)
            throw new DPBuddyException("Received invalid response from the device. " +
                    "The response has to contain at least one '%s' element. " +
                    "Please check the device logs. " +
                    "XML of the response:\n%s", eltName, toString());
        
        return elts;
    }

    protected List<Element> getResponseElements(String eltName) {
        List<Element> elts = JDomUtils.findDPElements(doc, "//"+eltName);
        
        return elts;
    }

    
    public Element getResponseRootElement( ) {
        Element body=JDomUtils.getSingleChild( doc.getRootElement() );
        return JDomUtils.getSingleChild(body); 
    }
    
    protected String logEventsToStr(Element logElt){
        List<Element> logEvents = logElt.getChildren("log-event");

        StringBuilder buf = new StringBuilder();
        for( Element logEvent : logEvents){
            buf.append("\t"+logEvent.getTextTrim()+"\n");
        }
        return buf.toString();
    }
    
    public String toString() {
        return JDomUtils.docToString(doc);
    }

    
}
