package com.myarch.dpbuddy;

import java.net.MalformedURLException;
import java.net.URL;

import com.myarch.dpbuddy.client.SecureHTTPClient;
import com.myarch.util.FilePathUtils;


public class DPConnection  {
    
    private String deviceName;
    
    private String url;
    private String domain;
    private String username;
    private String password;
    
    private boolean isTrustAllCerts=true;
    
    private SecureHTTPClient httpClient;
    private SecureHTTPClient httpClient2004;
    
    
    private boolean schemaValidation = false;
    
    public boolean isSchemaValidation() {
        return schemaValidation;
    }


    public void setSchemaValidation(boolean schemaValidation) {
        this.schemaValidation = schemaValidation;
    }


    public DPConnection(String url, String domain, String username, String password) {
        super();
        setUrl( url );
        setDomain(domain);
        this.username = username;
        this.password = password;
    }

    public DPConnection(String deviceName, String url, String domain, String username, String password) {
        this(url, domain, username, password);
        setDeviceName(deviceName);
    }


    public DPConnection(DPConnection anotherConnection) {
        this(anotherConnection.url, anotherConnection.domain, anotherConnection.username, anotherConnection.password);
        setDeviceName(anotherConnection.deviceName);
        setSchemaValidation(anotherConnection.schemaValidation);
        setTrustAllCerts( anotherConnection.isTrustAllCerts );        
    }

    
    public String getUrl() {
        return url;
    }
    
    public String getHost() {
        return FilePathUtils.getHost(getUrl());
    }
    
    
    
    public boolean isTrustAllCerts() {
        return isTrustAllCerts;
    }


    public void setTrustAllCerts( boolean isTrustAllCerts ) {
        this.isTrustAllCerts=isTrustAllCerts;
    }


    /**
     * Sets the URL of the SOMA service. If the url parameter contains no path, adds "/service/mgmt/current"
     * to ensure that the latest schema is used. 
     * Otherwise, DataPower defaults to 2004 schema 
     * @param url
     */
    public void setUrl( String url ){
        if (! url.contains( ":/" )) {
            url="https://"+url;
        }
        URL somaURL;
        try {
            somaURL = new URL(url);
        } catch (MalformedURLException e) {
            throw new DPBuddyException("Invalid URL: "+url); 
        }
        if (somaURL.getPort()<0) {
            try {
                somaURL=new URL( somaURL.getProtocol(), somaURL.getHost(), 5550, somaURL.getFile() );
                url=somaURL.toString();
            } catch (MalformedURLException e) { 
                throw new DPBuddyException("Invalid URL: "+url);
            }
        }
        if (somaURL.getFile().length()==0 || somaURL.getFile().equals("/"))
            url=url+"/service/mgmt/current";
        this.url = url;
        
        httpClient=null;
        httpClient2004=null;
    }

    
    public String getUsername() {
        return username;
    }

    
    
    public void setUsername( String username ) {
        this.username=username;
    }


    public void setPassword( String password ) {
        this.password=password;
    }


    public String getPassword() {
        return password;
    }
    
    public String getDomain() {
        return domain;
    }


    public void setDomain(String domain) {
        this.domain = domain;
        if (this.domain!=null)
            this.domain=this.domain.trim();
    }

    public boolean isDefaultDomain() {
        return domain.equalsIgnoreCase(Device.DEFAULT_DOMAIN_NAME);
    }
    
    public void setDeviceName(String deviceName) {
        this.deviceName = deviceName;
    }
    
    public String getDeviceName() {
        return deviceName;
    }


    // TODO: implement clonable
    public DPConnection clone() {
        return new DPConnection(this);
    }


    public SecureHTTPClient getHttpClient(boolean use2004URL) {
        SecureHTTPClient client;
        if (use2004URL)
            client=get2004Client();
        else
            client=getClient();
        return client;
    }

    private SecureHTTPClient get2004Client() {
        if (httpClient2004==null)
            httpClient2004=new SecureHTTPClient( convertTo2004URL(url), isTrustAllCerts );
        return httpClient2004; 
    }
    
    private SecureHTTPClient getClient() {
        if (httpClient==null)
            httpClient=new SecureHTTPClient( url, isTrustAllCerts );
        return httpClient; 
    }

    public void resetHttpClient() {
        httpClient=null;
        httpClient2004=null;
    }
    
    
    /**
     * Convert the default schema URL to the "old" 2004 URL. E.g., file upload
     * only works with the XML management URL that uses 2004 schema since it
     * allows for multiple files in the request.
     * 
     * @param current
     *            url
     * @return 2004 url
     */
    private static String convertTo2004URL( String url ) {
        URL currentSchemaURL;
        try {
            currentSchemaURL=new URL( url );
        }
        catch (MalformedURLException e) {
            throw new DPBuddyException( "Invalid URL: " + url );
        }
        String oldSchemaURL=currentSchemaURL.getProtocol() + "://" + currentSchemaURL.getHost() + ":"
            + currentSchemaURL.getPort() + "/service/mgmt/2004";
        return oldSchemaURL;

    }

    
    
}
