package com.myarch.dpbuddy;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.status.ObjectNotFoundException;
import com.myarch.dpbuddy.status.ObjectStatus;

public class DPObjectMatcher {

    private final Log logger=LogFactory.getLog( getClass() );
    // private final Log logger=LogFactory.getLogger(
    // ObjectMatchingUtils.class.getName() );

    public Set<ObjectStatus> matchObjectStatuses( Iterable<ObjectStatus> statuses,
                    Iterable<? extends DPObject> includeFilters, Iterable<? extends DPObject> excludeFilters,
                    boolean mustMatch ) {

        if (mustMatch) {
            validateAllIncludeFiltersMatch( statuses, includeFilters );
        }

        Set<ObjectStatus> matchedStatuses=new HashSet<ObjectStatus>();
        for (ObjectStatus status : statuses) {
            if (matchesOneOfFilters( status, includeFilters ) && !matchesOneOfFilters( status, excludeFilters )) {
                matchedStatuses.add( status );
            }
        }

        if (matchedStatuses.size() > 0) {
            logger.debug( "Following objects matched the provided filters:\n"
                + DPObject.objectListToString( matchedStatuses ) );
        }

        return matchedStatuses;
    }

    private void validateAllIncludeFiltersMatch( Iterable<ObjectStatus> statuses,
                    Iterable<? extends DPObject> filters ) {
        for (DPObject filter : filters) {
            if (!matchesOneOfObjects( filter, statuses )) {
                throw new ObjectNotFoundException(
                                "Object pattern '%s' did not match any existing object in the domain",
                                filter.toClassObjectStr() );
            }
        }
    }

    private boolean matchesOneOfObjects( DPObject filter, Iterable<ObjectStatus> statuses ) {
        for (ObjectStatus status : statuses) {
            if (DPObject.objectMatchesFilter( status, filter )) {
                return true;
            }
        }
        return false;
    }

    private boolean matchesOneOfFilters( ObjectStatus status, Iterable<? extends DPObject> filters ) {
        for (DPObject filter : filters) {
            if (DPObject.objectMatchesFilter( status, filter )) {
                return true;
            }
        }

        return false;
    }

}
