package com.myarch.dpbuddy;

import java.io.File;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;

public enum ExportImportFileType {
    ZIP, XML;

    public static ExportImportFileType fromString( String formatString ) {
        if (formatString == null)
            return null;

        formatString=formatString.toUpperCase();
        for (ExportImportFileType formatInstnance : ExportImportFileType.values()) {
            if (formatString.equals( formatInstnance.toString() )) {
                return formatInstnance;
            }
        }
        throw new DPBuddyException(
                        "You must provide ZIP or XML values for the file type. You provided: "
                            + formatString );
    }

    public static ExportImportFileType fromFile( String file ) {

        String ext=FilenameUtils.getExtension( file );
        if (ext == null) {
            throw new DPBuddyException( "You must specify file extension for the file " + file );
        }
        
        if (isXmlExtensionString( ext ))
            ext="XML";
        ExportImportFileType format;
        try {
            format=fromString( ext );
        }
        catch (DPBuddyException e) {
            throw new DPBuddyException(
                            "You must provide a file name with 'zip', 'xml' or 'xcfg' extensions. You provided: "
                                + file );
        }
        return format;
    }

    public static ExportImportFileType fromFile( File file ) {
        return fromFile( file.getAbsolutePath() );
    }

    public static boolean isZipExtension( File file ) {
        return isExtension( file, ExportImportFileType.ZIP );
    }
    
    private static boolean isXmlExtensionString(String ext) {
        return StringUtils.equalsIgnoreCase( ext, "xml") || StringUtils.equalsIgnoreCase( ext, "xcfg");
    }

    private static boolean isExtension( File file, ExportImportFileType fileType ) {
        String ext=FilenameUtils.getExtension( file.getAbsolutePath() );
        return ( ext != null && fileType.toString().equalsIgnoreCase( ext ) );
    }
    
    public static String defaultToZip(String fileName) {
        String fileNameWithExt=fileName;
        
        String ext=FilenameUtils.getExtension( fileName );
        if (StringUtils.isBlank( ext )) {
            fileNameWithExt=fileName+".zip";
        }
        
        return fileNameWithExt;
    }

}
