package com.myarch.dpbuddy;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.xmlutil.JDomUtils;

/**
 * Response containing "result" elements. This is the most common response from
 * the device. It is used for all actions and file uploads.
 */
public class ResultResponse extends BaseResponse {

    // private final static Log logger = LogFactory.getLog( ResultResponse.class
    // );

    private List<ResultResponseItem> results=new ArrayList<ResultResponseItem>();

    public ResultResponse(Document doc) {
        super( doc );
        results=extractResults( doc );
    }

    private List<ResultResponseItem> extractResults( Document doc ) {
        List<ResultResponseItem> results=new ArrayList<ResultResponseItem>();

        List<Element> resultElts=getRequiredResponseElements( "dp:result" );
        int i=0;
        for (Element resultElt : resultElts) {
            results.add( new ResultResponseItem( resultElt, i ) );
            i++;
        }

        return results;

    }

    public void validate() {
        for (ResultResponseItem resultItem : results) {
            if (!resultItem.isSuccess()) {
                String errMsg;
                if (resultItem.getLogEventsStr() != null && resultItem.getLogEventsStr().length() > 0) {
                    errMsg="Request failed with the following information:\n"
                        + resultItem.getLogEventsStr();
                }
                else {
                    errMsg="Request failed with the status '" + resultItem.getStatus()
                        + "'. Full response from the device:\n" + toString();
                }
                throw new DPBuddyException( errMsg );
            }
        }
    }

    public List<ResultResponseItem> getResultItems() {
        return results;
    }

    public List<ResultResponseItem> getErrorResultItems( String ignoreErrorPattern ) {

        List<ResultResponseItem> errors=new ArrayList<ResultResponseItem>();
        for (ResultResponseItem resultItem : results) {
            if (!resultItem.isSuccess()) {
                String errorMessage=resultItem.getLogEventsStr();

                if (ignoreErrorPattern == null || errorMessage == null
                    || !StringUtils.containsIgnoreCase( errorMessage, ignoreErrorPattern )) {
                    errors.add( resultItem );
                }
            }
        }
        return errors;
    }

    public List<ResultResponseItem> getErrorResultItems() {
        return getErrorResultItems( null );
    }
    
    
    // TODO: support multiple patterns
    public void validate( List<Element> actionElts, String ignoreErrorPattern ) {

        StringBuilder errorMsgBuf=new StringBuilder();
        List<ResultResponseItem> responseErrorItems=getErrorResultItems(ignoreErrorPattern);

        for (ResultResponseItem errorItem : responseErrorItems) {
            errorMsgBuf.append( String.format( "Action %s failed",
                            JDomUtils.elementWithClidrenToFunctionString(
                                            actionElts.get( errorItem.getIndex() ) ) ) );

            if (errorItem.getLogEventsStr() != null) {
                errorMsgBuf.append( " with the error: " + errorItem.getLogEventsStr() );
            }
            errorMsgBuf.append( "\n" );
        }

        if (errorMsgBuf.length() > 0) {
            throw new DPBuddyException( "Request failed:\n" + errorMsgBuf.toString() );
        }
    }

}
