package com.myarch.dpbuddy.action;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElementNS;
import org.apache.tools.ant.util.XMLFragment;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.xmlutil.XMLInput;

/**
 * Executes "do-action" request. "do-action" can have nested elements specified
 * as XML
 */
public class ActionAntTask extends BaseDPBuddyTask implements DynamicElementNS {

    private String actionName;

    private XMLFragment xmlFragment=new XMLFragment();

    @Option(name="-name", usage="Name of SOMA action as per the DataPower XML schema.")
    public void setName( String actionName ) {
        this.actionName=actionName;
    }

    public Object createDynamicElement( String uri, String name, String qName ) {
        xmlFragment.setProject( getProject() );
        return xmlFragment.createDynamicElement( uri, name, qName );
    }

    public void addText( String s ) {
        xmlFragment.setProject( getProject() );
        xmlFragment.addText( s );
    }

    public void executeDPTask() throws BuildException {

        if (!xmlFragment.getFragment().hasChildNodes()) {
            validator.validateRequired( actionName, "actionName",
                            "Name of the action - this is the name of the nested XML element inside do-action" );
        }
        validator.throwIfErrors();
        ActionCommand actionCommand=null;

        if (actionName != null) {
            actionCommand=new ActionCommand( actionName );
        }
        else if (xmlFragment.getFragment().hasChildNodes()) {
            actionCommand=new ActionCommand( new XMLInput( xmlFragment.getFragment() ) );
        }
        else {
            throw new BuildException( "You must specify either action name or nested XML" );
        }
        
        logger.info( String.format( "Executing action '%s' ...", actionCommand.getActionName() ) );
        
        try {
            actionCommand.execute( getDevice() );
        }
        catch (DPBuddyException e) {
            e.setRollback( true );
            throw e;
        }

        saveDomainConfigIfEnabled();
    }

    @Override
    protected Boolean isValidateIfNotSet() {
        return true;
    }

}
