package com.myarch.dpbuddy.action;

import java.util.List;

import org.jdom2.Element;

import com.myarch.dpbuddy.BaseRequest;
import com.myarch.dpbuddy.DPCommand;
import com.myarch.dpbuddy.DPRequest;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.ResultResponse;
import com.myarch.dpbuddy.xmlutil.XMLInput;

/**
 * Action command
 */
public class ActionCommand implements DPCommand<ResultResponse> {

    private boolean ignoreErrors=false;
    public final static String DO_ACTION_ELEMENT_NAME="do-action";

    protected BaseRequest request=new BaseRequest( DO_ACTION_ELEMENT_NAME );

    public ActionCommand() {
    }

    public ActionCommand(String actionName) {
        super();
        addAction( actionName );
    }

    public ActionCommand(XMLInput xml) {
        super();
        xml.clearAntlibNamespace();
        Element nestedXML=xml.getDocument().getRootElement().detach();

        getDoActionElement().addContent( nestedXML );
    }

    
    public Element addAction( String actionName ) {
        Element actionElement=new Element( actionName );
        getDoActionElement().addContent( actionElement );
        return actionElement;
    }

    public Element addAction( String actionName, String actionChildName, String actionChildText ) {

        Element actionChildElement=new Element( actionChildName );
        actionChildElement.addContent( actionChildText );
        Element actionElement=new Element( actionName );
        actionElement.addContent( actionChildElement );
        getDoActionElement().addContent( actionElement );

        return actionElement;
    }

    public void addActionChildElement( String actionChildName, String actionChildText ) {
        Element actionChildElement=new Element( actionChildName );
        actionChildElement.addContent( actionChildText );
        getActionElement().addContent( actionChildElement );
    }

    public String getActionName() {
        return request.getActionName( getDoActionElement() );
    }

    protected Element getDoActionElement() {
        return request.getRequestChild( DO_ACTION_ELEMENT_NAME );
    }

    public Element getActionElement() {
        List<Element> actionElements=getDoActionElement().getChildren();
        if (actionElements.size() > 1)
            throw new IllegalStateException( "Found multiple action elements under 'do-action'; "
                + "can't return the first one" );
        if (actionElements.size() == 0)
            throw new IllegalStateException( "Found no action elements under 'do-action', "
                + "at least one is required by the DataPower schema" );

        return actionElements.get( 0 );
    }

    protected List<Element> getActionElements() {
        return getDoActionElement().getChildren();
    }

    public void setIgnoreErrors( boolean ignoreErrors ) {
        this.ignoreErrors=ignoreErrors;
    }

    
    public boolean isIgnoreErrors() {
        return ignoreErrors;
    }

    public ResultResponse execute( Device device ) {

        ResultResponse dpResponse=device.executeRequestWithResultResponse( request );

        if (!ignoreErrors) {
            dpResponse.validate(getDoActionElement().getChildren(), null );
        }

        return dpResponse;
    }

    public DPRequest getRequest(){
        return request;
    }
    
    @Override
    public String toString() {
        return request.toString();
    }

}
