package com.myarch.dpbuddy.action;

import java.io.File;

import org.apache.commons.lang3.StringUtils;
import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.cli.BoolOptionHandler;
import com.myarch.dpbuddy.filemanagement.CopyFilesCommand;
import com.myarch.dpbuddy.inpututil.Input;
import com.myarch.dpbuddy.inpututil.InputFileWrapper;
import com.myarch.dpbuddy.inpututil.InputStringWrapper;

/**
 * Executes a remote command/script on the device
 */
public class ExecAntTask extends BaseDPBuddyTask {

    private File file;
    private boolean resolveVars=true;

    public final static String DEFAULT_SCRIPT_DIR="local:/cfg-scripts";
    private String toDir=DEFAULT_SCRIPT_DIR;

    public final static String DEFAULT_SCRIPT_NAME="commands.cfg";
    private String scriptName=DEFAULT_SCRIPT_NAME;

    private String command;

    @Option(name="-command", aliases="-cmd", usage="A single command to execute on the device")
    public void setCommand( String command ) {
        this.command=command;
    }

    @Option(name="-file", usage="File with the script to execute")
    public void setFile( File file ) {
        this.file=file;
    }

    @Option(
                    name="-toDir",
                    usage="The target directory for uploaded script files on the device. Defaults to local:/cfg-scripts.")
    public void setToDir( String toDir ) {
        this.toDir=toDir;
    }

    @Option(name="-resolveVars", handler=BoolOptionHandler.class,
                    usage="Resolve variables (${}) inside the script file.")
    public void setResolveVars( boolean resolveVars ) {
        this.resolveVars=resolveVars;
    }

    @Option(
                    name="-scriptName",
                    usage="The name of the script file on the device. If 'file' was provided, defaults to the name of the file. If 'command' was provided, defaults to 'commands.cfg'")
    public void setScriptName( String scriptName ) {
        this.scriptName=scriptName;
    }

    @Override
    public void executeDPTask() throws BuildException {
        if (StringUtils.isBlank( command ) && file == null)
            throw new DPBuddyException( "You must provide a file or a command" );

        if (StringUtils.isNotBlank( command ) && file != null)
            throw new DPBuddyException( "You can specify either a file or a command but not both" );

        CopyFilesCommand copyFilesCommand=new CopyFilesCommand();

        String scriptDirOnTheDevice=toDir;
        Input input=createInput();
        copyFilesCommand.addFile( input, scriptDirOnTheDevice );
        copyFilesCommand.execute( getDevice() );

        String filePathOnDevice=copyFilesCommand.getRemoteFilePath();
        ActionCommand actionCommand=new ActionCommand( "ExecConfig" );
        actionCommand.addActionChildElement( "URL", filePathOnDevice );

        logger.info( String.format( "Executing script '%s' ...", filePathOnDevice ) );

        try {
            actionCommand.execute( getDevice() );
        }
        catch (DPBuddyException e) {
            e.setRollback( true );
            throw e;
        }

        saveDomainConfigIfEnabled();

    }

    private Input createInput() {
        if (resolveVars || StringUtils.isNotBlank( command )) {
            InputStringWrapper input=null;
            if (file != null)
                input=new InputStringWrapper( file );
            else if (command != null)
                input=new InputStringWrapper( command, scriptName );

            input.resolveVars( getProject() );
            return input;
        }
        else {
            return new InputFileWrapper( file );
        }

    }

}
