package com.myarch.dpbuddy.ant;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.Project;

import com.myarch.antutil.AntUtils;
import com.myarch.dpbuddy.DeviceFactory;
import com.myarch.dpbuddy.DeviceFactoryImpl;
import com.myarch.propmanagement.PropRepository;
import com.myarch.propmanagement.ant.AntPropRepository;

public class ExtensionInitializer {

    private final Log logger = LogFactory.getLog(getClass());
    
    private Project project;
    
    
    /**
     * @param project
     */
    public ExtensionInitializer(Project project) {
        super();
        this.project = project;
    }

    
    
    public final static PropRepository DEFAULT_PROP_REPOSITORY=new AntPropRepository();
    public final static String PROP_REPOSITORY_PROP_NAME="dp.prop.repos.classname";
    
    public  PropRepository instantiatePropRepository() {
        String propReposClassName=project.getProperty(PROP_REPOSITORY_PROP_NAME);
        PropRepository propRepos=null;
        if (propReposClassName!=null) {

            logger.debug(String.format("Custom property repository was specified using property '%s'. Instantiating implementing class '%s'"
                    ,PROP_REPOSITORY_PROP_NAME, propReposClassName));

            propRepos=AntUtils.<PropRepository>instantiateClass(propReposClassName);
        }
        else
            propRepos=DEFAULT_PROP_REPOSITORY;
        propRepos.initialize(project);
        return propRepos;
    }

    public final static DeviceFactory DEFAULT_DEVICE_FACTORY=DeviceFactoryImpl.newInstance();
    public final static String DEVICE_FACTORY_PROP_NAME="dp.device.factory.classname";

    
    public  DeviceFactory instantiateDeviceFactory() {
        String deviceFactoryClassName=project.getProperty(DEVICE_FACTORY_PROP_NAME);
        DeviceFactory deviceFactory=null;
        if (deviceFactoryClassName!=null) {

            logger.debug(String.format("Custom device factory was specified using property '%s'. Instantiating implementing class '%s'"
                    ,DEVICE_FACTORY_PROP_NAME, deviceFactoryClassName));

            deviceFactory=AntUtils.<DeviceFactory>instantiateClass(deviceFactoryClassName);
        }
        else
            deviceFactory=DEFAULT_DEVICE_FACTORY;
        
        return deviceFactory;
    }

    
}
