        package com.myarch.dpbuddy.anysoma;


import java.io.File;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.BaseResponse;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.xmlutil.JDomUtils;
import com.myarch.dpbuddy.xmlutil.XMLInput;

/**
 * Executes a SOMA request file
 */
public class AnySOMARequestAntTask extends BaseDPBuddyTask{
    
    private File file;
   
    private boolean isPrintResponse=false;
    
    @Option(name="-file", required=true, usage="SOMA request file.")
    public void setFile( File file ) {
        this.file = file;
    }
    
    @Option(name="-printResponse", handler=ExplicitBooleanOptionHandler.class, usage="Print XML response from the device.")
    public void setPrintResponse( boolean isPrintResponse ) {
        this.isPrintResponse=isPrintResponse;
    }


    @Override
    public void executeDPTask() throws BuildException {
        validator.validateExistingFile(file, "file", 
                    "XML file containing DataPower XML management request");
        validator.throwIfErrors();
        
        XMLInput requestXML =  new XMLInput( file );
        requestXML.resolveVars(envPropSet.getPropExpressionEvaluator());
        
        AnySOMARequestCommand anySomaCmd = new AnySOMARequestCommand(requestXML);

        logger.info(String.format("Executing request '%s'", anySomaCmd.getRequestName()));
        BaseResponse response=null;
        try {
            response=anySomaCmd.execute( getDevice() );
        }
        catch (DPBuddyException e) {
            e.setRollback( true );
            throw e;
        }
          
        if (isPrintResponse)
            logger.info("Response from the device:\n"+JDomUtils.xmlFragmentToString( response.getResponseRootElement()));
        
        
        saveDomainConfigIfEnabled();
    }
    
    @Override
    protected Boolean isValidateIfNotSet() {
        return true;
    }

    
}
