package com.myarch.dpbuddy.audit;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.jdom2.Element;

import com.google.gson.JsonObject;
import com.myarch.util.JsonUtils;
/**
 * Represents a file entry in the audit event (i.e., an uploaded file).
 *
 */
public class AuditEventFile {
    private Date lastModified;
    private File localFile;
    private String path;

    public AuditEventFile(File localFile) {
        this.localFile=localFile;
        this.lastModified=new Date( localFile.lastModified() );
    }

    public AuditEventFile(String path) {
        this.path=path;
    }

    public AuditEventFile(File localFile, String path) {
        this( localFile );
        this.path=path;
    }

    public Element toXML() {
        // TODO: constants
        Element elt=new Element( "file" );
        if (localFile != null) {
            elt.setAttribute( "localPath", localFile.getAbsolutePath() );
            String timestampString=new SimpleDateFormat( AuditEvent.TIMESTAMP_FORMAT )
                            .format( lastModified );
            elt.setAttribute( "lastModified", timestampString );
        }

        if (path != null)
            elt.setAttribute( "path", path );

        return elt;
    }
    
    public JsonObject toJson() {
        JsonObject json=new JsonObject();
        if (localFile != null) {
            json.addProperty( "local_path", localFile.getAbsolutePath() );
            json.addProperty( "last_modified", JsonUtils.dateToIsoStr( lastModified ) );
        }

        if (path != null) {
            json.addProperty( "path", path );
        }
        json.addProperty( "name", getFileName() );
        
        return json;
    }
    
    public String getFileName(){
        String fileName=null;
        if (localFile != null) {
            fileName=localFile.getName();
        }
        else {
            fileName=new File(path).getName();
        }
        
        return fileName;
    }
}
