package com.myarch.dpbuddy.audit;

import java.io.File;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.filemanagement.CopyFilesCommand;
import com.myarch.dpbuddy.filemanagement.DownloadFileCommand;
import com.myarch.dpbuddy.filemanagement.FileResponse;
import com.myarch.eventmodel.Event;
import com.myarch.util.FilePathUtils;

public class AuditEventHandler {
    public final static String AUDIT_LOGGER_NAME="dpbuddy-audit";

    private Log logger=LogFactory.getLog( getClass() );
    private Logger auditLogger=LoggerFactory.getLogger( AUDIT_LOGGER_NAME );

    public final static String AUDIT_LOG_FILE_NAME_DEFAULT="dpbuddy-audit-log";

    public final String LOCAL_XML_AUDIT_LOG_FILE_DEFAULT=System.getProperty( "user.home" ) + "/.dpbuddy/log/"
        + AUDIT_LOG_FILE_NAME_DEFAULT + ".xml";

    public final String DP_AUDIT_LOG_FILE_DEFAULT="chkpoints://" + AUDIT_LOG_FILE_NAME_DEFAULT;

    private File localXmlAuditLogFile=new File( LOCAL_XML_AUDIT_LOG_FILE_DEFAULT );
    private String dpAuditLogFile=DP_AUDIT_LOG_FILE_DEFAULT;
    private String auditLogFileDomain;

    private boolean isXmlAuditOn=false;
    
    public void handleAuditEvent( AuditEvent auditEvent, Device device ) {
        
        logJson(auditEvent);
        
        // Legacy code:
        if (isXmlAuditOn &&localXmlAuditLogFile != null) {

            AuditLog localAuditLog=new AuditLog( localXmlAuditLogFile );

            localAuditLog.addEvent( auditEvent );

            localAuditLog.persist();
            if (dpAuditLogFile != null && device !=null)
                appendEventToRemoteLog( localAuditLog, auditEvent, device );
        }
    }

    private void logJson(AuditEvent auditEvent) {
        List<Event> events=auditEvent.asJsonEvents();
        for(Event event:events) {
            auditLogger.info(event.json().toString() );    
        }
    }
    
    
    public void setXmlAuditOn( boolean isXmlAuditOn ) {
        this.isXmlAuditOn=isXmlAuditOn;
    }



    private void appendEventToRemoteLog( AuditLog localAuditLog, AuditEvent auditEvent, Device device ) {

        Device auditFileDevice=device;
        if (auditLogFileDomain != null)
            auditFileDevice=device.switchDomain( auditLogFileDomain );

        logger.info( String.format( "Updating audit log file '%s' on the device in domain '%s' ...",
                        dpAuditLogFile, auditFileDevice.getCurrentDomain() ) );

        AuditLog dpAuditLog;

        DownloadFileCommand downloadCommand=new DownloadFileCommand();
        downloadCommand.setFromFile( dpAuditLogFile );
        FileResponse fileResponse=downloadCommand.execute( auditFileDevice );

        if (fileResponse.isSuccess()) {
            logger.debug( "The log file exists, updating it" );
            dpAuditLog=new AuditLog( fileResponse.getFileContentAsString() );
        }
        else {
            logger.info( "Didn't find the audit log file on the device, will create the new one" );
            dpAuditLog=new AuditLog();
        }
        dpAuditLog.addEvent( auditEvent );
        uploadAuditLogToDP( dpAuditLog, auditFileDevice );
    }

    private void uploadAuditLogToDP( AuditLog auditLog, Device device ) {
        CopyFilesCommand copyFilesCommand=new CopyFilesCommand();

        String targetFileName=FilePathUtils.getFileNameFromURL( dpAuditLogFile );
        String targetDir=dpAuditLogFile.substring( 0, dpAuditLogFile.length() - targetFileName.length() );

        copyFilesCommand.setQuiet( true );
        copyFilesCommand.addFile( auditLog.getXML(), targetDir, targetFileName );

        logger.debug( "Log will be saved in the domain: " + device.getCurrentDomain() );

        copyFilesCommand.execute( device );
    }

    public void setLocalXmlAuditLogFile( File localXmlAuditLogFile ) {
        this.localXmlAuditLogFile=localXmlAuditLogFile;
    }

    public String getDPAuditLogFile() {
        return dpAuditLogFile;
    }

    public void setDPAuditLogFile( String dpAuditLogFile ) {
        this.dpAuditLogFile=dpAuditLogFile;
    }

    public String getAuditLogFileDomain() {
        return auditLogFileDomain;
    }

    public void setAuditLogFileDomain( String auditLogFileDomain ) {
        this.auditLogFileDomain=auditLogFileDomain;
    }

}
