package com.myarch.dpbuddy.audit;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.xmlutil.XMLInput;
import com.myarch.dpbuddy.xmlutil.XMLParsingUtils;


// Only used for XML events
public class AuditLog {
    private Log logger=LogFactory.getLog(getClass());
    
    private File file;
    
    private Document auditLogDocument;
    
    public AuditLog(File xmlFile) {
        this.file=xmlFile;
        if (xmlFile.exists()) {
            auditLogDocument=XMLParsingUtils.parseFile(xmlFile);
        }
        else {
            logger.info(String.format("Audit file '%s' doesn't exist; it will be created", xmlFile.getAbsolutePath()));
            auditLogDocument=new Document(createRoot());
        }
    }

    public AuditLog(String auditLogXMLStr) {
        auditLogDocument=XMLParsingUtils.parseString(auditLogXMLStr);
    }

    /**
     * Creates empty audit log
     */
    public AuditLog() {
        auditLogDocument=new Document(createRoot());
    }
    
    public XMLInput getXML() {
        return new XMLInput(auditLogDocument);
    }
    
    
    private Element createRoot(){
        return new Element("events");
    }
    
    public void addEvent(AuditEvent event) {
        auditLogDocument.getRootElement().addContent(event.toXMLElement());
    }
    

    public Document selectEventsForDevice(String url, String domain) {
        Document filteredDoc=new Document(createRoot());
        for(Element eventElt:auditLogDocument.getRootElement().getChildren()) {
            AuditEvent event=AuditEvent.fromXMLElement(eventElt);
            if (event.urlEquals(url) && (domain==null || event.domainEquals(domain)))
                filteredDoc.getRootElement().addContent(eventElt.clone().detach());
        }
        
        return filteredDoc;
    }
    

    public void persist() {
        logger.debug(String.format("Updated audit file '%s'", file.getAbsolutePath()));
        XMLParsingUtils.persistXML(file, auditLogDocument);
    }
    
}
