package com.myarch.dpbuddy.backup;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.DPPattern;
import com.myarch.dpbuddy.ExportImportFileType;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;

public class BackupAntTask extends BaseDPBuddyTask {

    private String file;
    private File deploymentPolicyFile;
    private String deploymentPolicyName;
    private Boolean persisted;
    private Boolean appendTimestamp;
    private boolean failIfNoDomain=true;

    private String domainPatternsString;
    private List<Pattern> domainPatterns=new ArrayList<Pattern>();

    @Override
    public void executeDPTask() throws BuildException {

        validator.validateRequired( this.file, "file", "File to save the results of the backup to (ZIP)" );
        validator.throwIfErrors();

        BackupCommand backupCommand=new BackupCommand();
        String fileWithExt=ExportImportFileType.defaultToZip( file );

        backupCommand.setToFile( fileWithExt );

        if (domainPatternsString != null)
            backupCommand.addDomainPatterns( domainPatternsString );

        if (domainPatterns.size() > 0)
            backupCommand.addDomainPatterns( domainPatterns );

        setDeploymentPolicy( backupCommand, deploymentPolicyFile, deploymentPolicyName, null );

        if (persisted != null)
            backupCommand.setPersisted( persisted );

        if (appendTimestamp != null)
            backupCommand.setAppendTimestamp( appendTimestamp );
        backupCommand.setDomainMatchRequired( failIfNoDomain );

        executeRequest( backupCommand );
    }

    public void addConfiguredDomainPattern( DPPattern domainPattern ) {
        domainPatterns.add( domainPattern.getPattern() );
    }

    @Option(name="-domainPatterns", aliases="-domains",
                    usage="Comma-delimited list of regular expression patterns defining what domains to back up.")
    public void setDomainPatterns( String domainPatterns ) {
        this.domainPatternsString=domainPatterns;
    }

    @Option(name="-file", required=true, usage="Name of the zip file to save the backups to.")
    public void setFile( String file ) {
        this.file=file;
    }

    @Option(name="-appendTimestamp", handler=ExplicitBooleanOptionHandler.class,
                    usage="Automatically append timestamp to the file name.")
    public void setAppendTimestamp( Boolean appendTimestamp ) {
        this.appendTimestamp=appendTimestamp;
    }

    @Option(name="-failIfNoDomain", handler=ExplicitBooleanOptionHandler.class,
                    usage="Fail if a domain pattern didn't match any domain.")
    public void setFailIfNoDomain( boolean failIfNoDomain ) {
        this.failIfNoDomain=failIfNoDomain;
    }

    @Option(name="-persisted", handler=ExplicitBooleanOptionHandler.class,
                    usage="Backup only the persisted configuration.")
    public void setPersisted( Boolean persisted ) {
        this.persisted=persisted;
    }

    @Option(name="-deploymentPolicyFile", usage="Path to the deployment policy file on the local machine")
    public void setDeploymentPolicyFile( File deploymentPolicy ) {
        this.deploymentPolicyFile=deploymentPolicy;
    }

    @Option(name="-deploymentPolicyName", usage="Name of the deployment policy to include with the export/backup.")
    public void setDeploymentPolicyName( String deploymentPolicy ) {
        this.deploymentPolicyName=deploymentPolicy;
    }

    @Override
    protected boolean isDefaultDomainOverride() {
        return true;
    }

}
