package com.myarch.dpbuddy.backup;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.ant.BaseDPBuddyTask;

public class SecureBackupAntTask extends BaseDPBuddyTask {

    private String cert;
    private File toDir;

    private Boolean includeISCSI;
    private Boolean includeRAID;
    private String dpDir;

    private Boolean appendTimestamp;

    @Override
    public void executeDPTask() throws BuildException {

        validator.validateRequired( this.cert, "cert", "Name of the cert to use for secure backup" );
        validator.validateRequired( toDir, "to", "Local directory to download backup to" );

        validator.throwIfErrors();

        SecureBackupCommand sbackupCommand=new SecureBackupCommand( cert, toDir );

        if (includeISCSI != null)
            sbackupCommand.setIncludeISCSI( includeISCSI );

        if (includeRAID != null)
            sbackupCommand.setIncludeRAID( includeRAID );

        if (dpDir != null)
            sbackupCommand.setDPDir( dpDir );

        if (appendTimestamp != null)
            sbackupCommand.setAppendTimestamp( appendTimestamp );

        executeRequest( sbackupCommand );
    }

    @Option(
                    name="-cert",
                    required=true,
                    usage="Name of the crypto certificate containing the public key used in encrypting the secure backup.")
    public void setCert( String cert ) {
        this.cert=cert;
    }

    @Option(name="-toDir", required=true, usage="Local directory where to save the backup files to.")
    public void setToDir( File toDir ) {
        this.toDir=toDir;
    }

    @Option(name="-includeISCSI", handler=ExplicitBooleanOptionHandler.class, usage="Include iSCSI device in the backup.")
    public void setIncludeISCSI( Boolean includeISCSI ) {
        this.includeISCSI=includeISCSI;
    }

    @Option(name="-includeRAID", handler=ExplicitBooleanOptionHandler.class, usage="Include RAID device in the backup.")
    public void setIncludeRAID( Boolean includeRAID ) {
        this.includeRAID=includeRAID;
    }

    @Option(name="-dpDir", usage="DataPower directory for storing secure backup files.")
    public void setDPDir( String dpDir ) {
        this.dpDir=dpDir;
    }

    @Option(name="-appendTimestamp", handler=ExplicitBooleanOptionHandler.class,
                    usage="Automatically append timestamp to the directory name specified in 'toDir'.")
    public void setAppendTimestamp( Boolean appendTimestamp ) {
        this.appendTimestamp=appendTimestamp;
    }
    
    @Override
    protected boolean isDefaultDomainOverride(){
        return true;
    }
}
