package com.myarch.dpbuddy.backup;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Element;

import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.ResultResponse;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.dpbuddy.filemanagement.DownloadMultipleFilesCommand;
import com.myarch.dpbuddy.xmlutil.JDomUtils;
import com.myarch.util.FilePathUtils;

/**
 * Perform secure backup
 */
public class SecureBackupCommand extends ActionCommand {
    public final String DEFAULT_DP_DIR="temporary:/secure-backup";
    private final String DP_DIR_ELT_NAME="destination";

    private final Log logger=LogFactory.getLog( getClass() );

    private String dpDir=DEFAULT_DP_DIR;

    private File saveToDir;
    private boolean appendTimestamp=false;

    /*
     * TODO: support for optional attrs and dpdir appendTimestamp
     */

    public SecureBackupCommand(String cert, File saveToDir) {
        super( "SecureBackup" );
        addActionChildElement( "cert", cert );
        addActionChildElement( DP_DIR_ELT_NAME, DEFAULT_DP_DIR );
        this.saveToDir=saveToDir;
    }

    /**
     * The directory used to store the backup internally on the device
     * 
     * @param dir
     */
    public void setDPDir( String dpDir ) {
        this.dpDir=dpDir;
        Element sbackupActionElt=getActionElement();
        JDomUtils.addOrUpdateElement( sbackupActionElt, DP_DIR_ELT_NAME, dpDir );
    }

    public void setIncludeISCSI( boolean includeISCSI ) {
        Element sbackupActionElt=getActionElement();
        JDomUtils.addOrUpdateElement( sbackupActionElt, "include-iscsi",
                        Device.booleanToToggle( includeISCSI ) );
    }

    public void setIncludeRAID( boolean includeRAID ) {
        Element sbackupActionElt=getActionElement();
        JDomUtils.addOrUpdateElement( sbackupActionElt, "include-raid",
                        Device.booleanToToggle( includeRAID ) );
    }

    public void setAppendTimestamp( boolean appendTimestamp ) {
        this.appendTimestamp=appendTimestamp;
    }

    @Override
    public ResultResponse execute( Device device ) {

        logger.info( String
                        .format( "Performing secure backup of the device. This could take a few minutes ..." ) );
        Device defaultDomainDevice=device.switchDefaultDomain( );
        ResultResponse sbackupResponse=super.execute( defaultDomainDevice );

        File saveToDirActual=saveToDir;
        if (appendTimestamp) {
            saveToDirActual=new File( FilePathUtils.appendTimestamp( saveToDir.getAbsolutePath() ) );
        }

        logger.info( String
                        .format( "Completed creation of the secure backup on the device. Downloading secure backup files to '%s'",
                                        saveToDirActual.getAbsolutePath() ) );
        DownloadMultipleFilesCommand downloadCommand=new DownloadMultipleFilesCommand( dpDir+"/[^/]*",
                        saveToDirActual, true, true, false );
        downloadCommand.setMatchRequired( true );
        downloadCommand.setCleanToDir( true );
        
        downloadCommand.execute( defaultDomainDevice );

        return sbackupResponse;

    }

}
