package com.myarch.dpbuddy.backup;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.restart.RestartDPService;

public class SecureRestoreAntTask extends BaseDPBuddyTask {

    private String cred;
    private File backupLocation;
    private boolean waitForReboot=false;
    private int timeout=360;
    
    private String dpDir;

    @Override
    public void executeDPTask() throws BuildException {

        validator.validateRequired( this.cred, "cred",
                        "Name of the credential to use for secure restore" );
        validator.validateRequired( backupLocation, "dir",
                        "Local directory with the location of the secure backup files" );

        validator.throwIfErrors();

        SecureRestoreCommand restoreCommand=new SecureRestoreCommand( cred, backupLocation, waitForReboot, timeout );

        if (dpDir != null){
            restoreCommand.setDPDir( dpDir );
        }
        executeRequest( restoreCommand );
    }

    
    @Option(name="-cred", required=true,
                    usage="Name of the credential to use to decrypt the backup.")
    public void setCred( String cred ) {
        this.cred=cred;
    }

    @Option(name="-dir", required=true,
                    usage="Local directory with the location of the secure backup files.")
    public void setDir( File dir ) {
        this.backupLocation=dir;
    }

    @Option(name="-dpDir", usage="DataPower directory to upload secure backup files to. Defaults to "+SecureRestoreCommand.DEFAULT_DP_DIR)
    public void setDPDir( String dpDir ) {
        this.dpDir=dpDir;
    }

    @Option(name="-waitForReboot", handler=ExplicitBooleanOptionHandler.class, usage="Wait for the completion of reboot after restore.")
    public void setWaitForReboot( boolean waitForReboot ) {
        this.waitForReboot=waitForReboot;
    }

    @Option(name="-timeout",
                    usage="Time, in seconds, to wait for DataPower to become available after reboot triggered by restore. Defaults to "
                        + RestartDPService.DEFAULT_TIMEOUT + " seconds.")
    public void setTimeout( int timeout ) {
        this.timeout=timeout;
    }
    
}
