package com.myarch.dpbuddy.cache;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.tools.ant.BuildException;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.status.ObjectStatus;

/**
 * Flushes AAA cache
 * 
 */
public class FlushAAACacheAntTask extends BaseDPBuddyTask {

    public final static String AAA_POLICY_TYPE_NAME="AAAPolicy";

    private String policy=".*";

    public void setPolicy( String policy ) {
        this.policy=policy;
    }

    @Override
    public void executeDPTask() throws BuildException {

        Set<String> policies=getPolicyNames( policy );

        ActionCommand actionCommand=new ActionCommand();

        for (String policy : policies) {
            actionCommand.addAction( "FlushAAACache", "PolicyName", policy );
        }

        executeRequest( actionCommand );
    }

    private Set<String> getPolicyNames( String policyPattern ) {
        Device device=getDevice();
        logger.info( "Determining AAA policies to flush ..." );
        Set<ObjectStatus> policies=device.fetchObjects( new DPObject( AAA_POLICY_TYPE_NAME,
                        policyPattern ), true );
        if (policies.size() == 0)
            throw new DPBuddyException( "Was not able to find AAA policies matching the pattern '%s'",
                            policyPattern );

        Set<String> policyNames=new HashSet<String>();
        for (ObjectStatus policyObject : policies) {
            policyNames.add( policyObject.getName() );
        }

        logger.info( "Flushing the following AAA policies: " + StringUtils.join( policyNames, ", " ) );

        return policyNames;
    }

}
