package com.myarch.dpbuddy.cache;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.status.ObjectStatus;

/**
 * Flushes cache in xml manager. Cache type (document or stylesheet or both) is
 * determined by the name of the command.
 * 
 */
public class FlushXMLCacheAntTask extends BaseDPBuddyTask {

    public final static String XML_MANAGER_TYPE_NAME="XMLManager";
    public final static String ALL_XML_MANAGERS_PATTERN=".*";

    private String xmlManager=ALL_XML_MANAGERS_PATTERN;

    @Option(
                    name="-xmlManager",
                    usage="Regular expression matching the names of XML managers that should be flushed.")
    public void setXMLManager( String xmlManager ) {
        this.xmlManager=xmlManager;
    }

    @Override
    public void executeDPTask() throws BuildException {
        XMLCacheType cacheType=XMLCacheType.fromTaskName( getTaskName() );
        performFlush( getDevice(), cacheType, xmlManager );
    }

    public void performFlush( Device device, XMLCacheType cacheType, String xmlManagerPattern ) {

        Set<String> managers=getXMLManagerNames( device, xmlManagerPattern );

        ActionCommand actionCommand=new ActionCommand();

        for (String manager : managers) {
            addAction( actionCommand, cacheType, manager );
        }

        actionCommand.execute( device );
    }

    public void performFlush( Device device, XMLCacheType cacheType ) {
        performFlush( device, cacheType, ALL_XML_MANAGERS_PATTERN );
    }

    private Set<String> getXMLManagerNames( Device device, String xmlManagerPattern ) {
        logger.info( "Determining XML managers to flush ..." );
        Set<ObjectStatus> managers=device.fetchObjects( new DPObject( XML_MANAGER_TYPE_NAME,
                        xmlManagerPattern ) );
        if (managers.size() == 0)
            throw new DPBuddyException( "Was not able to find XML managers matching the pattern '%s'",
                            xmlManagerPattern );

        Set<String> managerNames=new HashSet<String>();
        for (ObjectStatus managerObject : managers) {
            managerNames.add( managerObject.getName() );
        }

        logger.info( "Flushing the following XML managers: " + StringUtils.join( managerNames, ", " ) );

        return managerNames;
    }

    private void addAction( ActionCommand actionCommand, XMLCacheType cacheType, String xmlManagerName ) {

        if (cacheType == XMLCacheType.ALL) {
            for (XMLCacheType cacheTypeInstance : XMLCacheType.values()) {
                if (cacheTypeInstance != XMLCacheType.ALL)
                    actionCommand.addAction( cacheTypeInstance.getActionName(), XML_MANAGER_TYPE_NAME,
                                    xmlManagerName );
            }
        }
        else {
            actionCommand.addAction( cacheType.getActionName(), XML_MANAGER_TYPE_NAME, xmlManagerName );
        }
    }

}
