package com.myarch.dpbuddy.checkpoint;

import java.util.Date;

import org.apache.commons.lang3.time.DateParser;
import org.apache.commons.lang3.time.FastDateFormat;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.xmlutil.JDomUtils;
import com.myarch.util.DateUtils;

public class Checkpoint {
    
    private static final DateParser CHECKPOINT_FORMAT_DATE_PARSER = FastDateFormat.getInstance("yyyy-MM-dd'T'HH:mm:ss zzz");
    
    private Date timestamp;
    private String name;
    
    
    /* CheckpointStatus response:
    <DomainCheckpointStatus >
        <Date>2017-03-20</Date>
        <Time>16:24:37 GMT</Time>
        <ChkName>test-ch1</ChkName>
    </DomainCheckpointStatus>  
    */    
    public static Checkpoint fromCheckpointStatusXml(Element checkpointStatusElt) {
        Checkpoint checkpoint = new Checkpoint();
        checkpoint.timestamp=parseDate(JDomUtils.getRequiredChildElementValue( checkpointStatusElt, "Date" ), 
                        JDomUtils.getRequiredChildElementValue( checkpointStatusElt, "Time" ));
        checkpoint.name=JDomUtils.getRequiredChildElementValue( checkpointStatusElt, "ChkName" );
        
        return checkpoint;
    }
    
    private static Date parseDate(String dateStr, String timeStr) {
        String dateTimeStr=dateStr+'T'+timeStr;
        Date dateTime=DateUtils.parseDate( dateTimeStr, CHECKPOINT_FORMAT_DATE_PARSER );
        if (dateTime==null) {
            throw new DPBuddyException( "Invalid checkpoint date, unable to parse it:", dateTimeStr);
        }
        return dateTime;
    }
    
    public Date timestamp(){
        return timestamp;
    }
    
    public String name(){
        return name;
    }

    @Override
    public String toString() {
        return "Checkpoint [timestamp=" + timestamp + ", name=" + name + "]";
    }

}
