package com.myarch.dpbuddy.checkpoint;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.cli.BoolOptionHandler;

/**
 * Executes "SaveCheckpoint" action request.
 * 
 */
public class CheckpointAntTask extends BaseDPBuddyTask {
    private final Log logger=LogFactory.getLog( getClass() );

    private String name;
    private Boolean timestamp;
    private int numberOfOldCheckpointToDelete=-1;

    @Option(name="-name", required=true, usage="Name of the checkpoint.")
    public void setName( String name ) {
        this.name=name;
    }

    @Option(name="-timestamp", aliases="-appendTimestamp", handler=BoolOptionHandler.class, usage="Append timestamp to the checkpoint name.")
    public void setTimestamp(Boolean timestamp ) {
        this.timestamp=timestamp;
    }
    
    @Option(name="-removeOld",usage="Number of old checkpoints to delete.")
    public void setRemoveOld(int numberOfOldCheckpointToDelete) {
        this.numberOfOldCheckpointToDelete=numberOfOldCheckpointToDelete;
    }

    @Override
    public void executeDPTask() throws BuildException {

        validator.validateRequired( name, "name", "checkpoint name" );
        validator.throwIfErrors();
        if (numberOfOldCheckpointToDelete>0) {
            logger.info( "Will try to remove up to "+numberOfOldCheckpointToDelete+" oldest checkpoints before creating the new one" );
            CheckpointService.deleteOldCheckpoints( getDevice(), numberOfOldCheckpointToDelete );
        }
        
        SaveCheckpointCommand actionRequest;
        if (timestamp != null)
            actionRequest=new SaveCheckpointCommand( name, timestamp );
        else
            actionRequest=new SaveCheckpointCommand( name );

        executeRequest( actionRequest );

        saveDomainConfigIfEnabled();
    }
    
}
