package com.myarch.dpbuddy.checkpoint;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Element;

import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.status.StatusCommand;
import com.myarch.dpbuddy.status.StatusResponse;

public class CheckpointService {
    
    
    /*
      <DomainCheckpointStatus xmlns:env="http://www.w3.org/2003/05/soap-envelope">
        <Date>2017-03-20</Date>
        <Time>16:24:37 GMT</Time>
        <ChkName>test-ch1</ChkName>
      </DomainCheckpointStatus>
      <DomainCheckpointStatus xmlns:env="http://www.w3.org/2003/05/soap-envelope">
        <Date>2017-06-18</Date>
        <Time>14:52:33 GMT</Time>
        <ChkName>test-ch2</ChkName>
      </DomainCheckpointStatus>
     */ 
    
    public static List<Checkpoint> fetchCheckpoints(Device device) {
        StatusCommand checkpointStatusCmd=new StatusCommand("DomainCheckpointStatus");
        StatusResponse checkpointStatusResponse=checkpointStatusCmd.execute( device );
        //checkpointStatus
        List<Element> checkpointStatusElts=checkpointStatusResponse.getStatusElements();
        List<Checkpoint> checkpoints=new ArrayList<>();
        // list of DomainCheckpointStatus elts
        for (Element elt:checkpointStatusElts) {
            Checkpoint checkpoint=Checkpoint.fromCheckpointStatusXml( elt );
            checkpoints.add( checkpoint );
        }
        return checkpoints;
    }
    
    public static void sortByTimestamp(List<Checkpoint> checkpoints) {
        Collections.sort(checkpoints, new CheckpointTimestampComparator());
    }
    
    private static class CheckpointTimestampComparator implements Comparator<Checkpoint> {

        @Override
        public int compare( Checkpoint o1, Checkpoint o2 ) {
            return o1.timestamp().compareTo( o2.timestamp() );
        }
        
    }
    
    public static void removeCheckpoint(Device device, String name) {
        RemoveCheckpointCommand command=new RemoveCheckpointCommand(name, null, true);
        command.execute( device );
    }
    
    public static void deleteOldCheckpoints(Device device, int numberOfOldCheckpointToDelete) {
        final Log logger=LogFactory.getLog( CheckpointService.class );
        List<Checkpoint> checkpoints = CheckpointService.fetchCheckpoints( device);
        if (checkpoints.size()>0) {
            CheckpointService.sortByTimestamp( checkpoints );
            for(int i=0;i<numberOfOldCheckpointToDelete && i<checkpoints.size();++i) {
                String chkName = checkpoints.get( i ).name();
                logger.info( "Removing old checkpoint '"+ chkName+"'");
                removeCheckpoint( device, checkpoints.get( i ).name() );
            }
        }
        
    }
 
    

}
