package com.myarch.dpbuddy.checkpoint;

import org.apache.commons.lang3.StringUtils;
import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.cli.BoolOptionHandler;

/**
 * Executes "SaveCheckpoint" action request.
 */
public class RemoveCheckpointAntTask extends BaseDPBuddyTask {

    private String name;
    private String namePattern;

    private boolean isMatchRequired=true;

    @Option(name="-name", usage="Name of the checkpoint to remove.")
    public void setName( String name ) {
        this.name=name;
    }

    @Option(
                    name="-namePattern",
                    usage="Regular expression pattern defining checkpoints to remove. Use '.*' to delete all checkpoints.")
    public void setNamePattern( String namePattern ) {
        this.namePattern=namePattern;
    }

    @Option(
                    name="-matchRequired",
                    handler=BoolOptionHandler.class,
                    usage="fail if no checkpoints matched the pattern specified in 'namePattern'. Defaults to 'true'.")
    public void setMatchRequired( boolean isMatchRequired ) {
        this.isMatchRequired=isMatchRequired;
    }

    @Override
    public void executeDPTask() throws BuildException {

        if (StringUtils.isBlank( name ) && StringUtils.isBlank( namePattern ))
            throw new DPBuddyException(
                            "You must provide either a checkpoint name or regular expression patterns to remove checkpoints" );

        RemoveCheckpointCommand actionRequest=new RemoveCheckpointCommand( name, namePattern,
                        isMatchRequired );
        executeRequest( actionRequest );

        saveDomainConfigIfEnabled();
    }

}
