package com.myarch.dpbuddy.checkpoint;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.ResultResponse;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.dpbuddy.status.StatusCommand;
import com.myarch.dpbuddy.status.StatusResponse;

public class RollbackCheckpointCommand extends ActionCommand {
    private final Log logger=LogFactory.getLog( getClass() );

    private String checkpointName;
    
    // rollback to the last checkpoint
    public RollbackCheckpointCommand() {
        super();
    }
    
    public RollbackCheckpointCommand( String checkpointName ) {
        super();
        this.checkpointName=checkpointName;
    }
    
    @Override
    public ResultResponse execute( Device device ) {
        String checkpointNameToUse=checkpointName;
        if (StringUtils.isBlank( checkpointNameToUse ) ) {
            logger.info( "Checkpoint name was not provided, will revert to the latest checkpoint");
            checkpointNameToUse=findLastCheckpoint(device);
        }
        String msgTemplate="Reverting configuration of the domain '%s' to the checkpoint '%s' ...";
        logger.info( String.format( msgTemplate,  device.getCurrentDomain(), checkpointNameToUse) );

        addAction("RollbackCheckpoint","ChkName", checkpointNameToUse);
        ResultResponse response=super.execute( device );
        
        return response;
    }
    
    private String findLastCheckpoint(Device device) {
        StatusCommand statusCmd=new StatusCommand("DomainCheckpointStatus");
        StatusResponse statusResponse=statusCmd.execute( device );
        List<String> existingCheckpoints=statusResponse.getStatusValues( "ChkName" );
        
        if (existingCheckpoints.size()==0)
            throw new CheckpointNotFoundException("Domain '%s' does not have any checkpoints", device.getCurrentDomain());
        
        return existingCheckpoints.get( existingCheckpoints.size() - 1);
    }


}
