package com.myarch.dpbuddy.checkpoint;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.ResultResponse;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.util.FilePathUtils;

public class SaveCheckpointCommand extends ActionCommand {

    private final Log logger=LogFactory.getLog( getClass() );

    private String checkpointName;
    
    private RemoveCheckpointCommand removeCheckpoint;

    public SaveCheckpointCommand(String checkpointName) {
        this( checkpointName, false );
    }

    public SaveCheckpointCommand(String suppliedCheckpointName, boolean appendTimestamp) {
        super();
        checkpointName=normalizeCheckpointName( suppliedCheckpointName );
        if (!checkpointName.equals( suppliedCheckpointName )) {
            logger.info( String.format( "Converted supplied checkpoint name '%s' to '%s'",
                            suppliedCheckpointName, checkpointName ) );
        }

        if (appendTimestamp)
            checkpointName=FilePathUtils.appendTimestamp( checkpointName );

        addAction( "SaveCheckpoint", "ChkName", checkpointName );

        removeCheckpoint=new RemoveCheckpointCommand( checkpointName, null, false );
        removeCheckpoint.setIgnoreErrors( true );
    }

    @Override
    public ResultResponse execute( Device device ) {
        logger.info(String.format("Creating checkpoint '%s' ...", checkpointName));
        if (removeCheckpoint != null)
            removeCheckpoint.execute( device );
        ResultResponse result=super.execute( device );
        return result;
    }

    public static String normalizeCheckpointName( String checkpointName ) {
        if (checkpointName.contains( "." ))
            checkpointName=checkpointName.replace( '.', '_' );
        return checkpointName;
    }

}
