package com.myarch.dpbuddy.cli;

import com.myarch.dpbuddy.action.*;
import com.myarch.dpbuddy.anysoma.AnySOMARequestAntTask;
import com.myarch.dpbuddy.backup.BackupAntTask;
import com.myarch.dpbuddy.backup.RestoreAntTask;
import com.myarch.dpbuddy.backup.SecureBackupAntTask;
import com.myarch.dpbuddy.backup.SecureRestoreAntTask;
import com.myarch.dpbuddy.cache.FlushMiscCacheAntTask;
import com.myarch.dpbuddy.cache.FlushXMLCacheAntTask;
import com.myarch.dpbuddy.checkpoint.CheckpointAntTask;
import com.myarch.dpbuddy.checkpoint.RemoveCheckpointAntTask;
import com.myarch.dpbuddy.checkpoint.RollbackCheckpointAntTask;
import com.myarch.dpbuddy.config.ConfigAntTask;
import com.myarch.dpbuddy.config.DelConfigAntTask;
import com.myarch.dpbuddy.config.UpdateConfigAntTask;
import com.myarch.dpbuddy.configreport.ConfigReportTask;
import com.myarch.dpbuddy.dpimport.CreateDomainAntTask;
import com.myarch.dpbuddy.dpimport.ImportAntTask;
import com.myarch.dpbuddy.encryption.DecryptAntTask;
import com.myarch.dpbuddy.encryption.EncryptAntTask;
import com.myarch.dpbuddy.encryption.PasswordMapAliasAntTask;
import com.myarch.dpbuddy.export.ExportAntTask;
import com.myarch.dpbuddy.filemanagement.ant.*;
import com.myarch.dpbuddy.firmware.FirmwareAntTask;
import com.myarch.dpbuddy.license.LicenseAntTask;
import com.myarch.dpbuddy.log.LogAntTask;
import com.myarch.dpbuddy.log.TailLogAntTask;
import com.myarch.dpbuddy.logtarget.*;
import com.myarch.dpbuddy.quiesce.QuiesceAntTask;
import com.myarch.dpbuddy.quiesce.QuiesceDomainAntTask;
import com.myarch.dpbuddy.restart.RestartDPAntTask;
import com.myarch.dpbuddy.status.*;
import com.myarch.hocon.ExportHoconTask;

public class DPBuddyMain {
    
    public static CliCommandDefContainer getCliCmdDefs(){
        CliCommandDefContainer cmdDefs=new CliCommandDefContainer();
        
        cmdDefs.add("status", new StatusAntTask());
        cmdDefs.add("serviceStatus", new ServiceStatusAntTask()); 
        cmdDefs.add("listServices", new ServiceStatusAntTask()); 
        
        cmdDefs.add("objectStatus", new ObjectStatusAntTask());
        cmdDefs.add("listObjects", new ObjectStatusAntTask());
        
        cmdDefs.add("import", new ImportAntTask()); 
        cmdDefs.add("export", new ExportAntTask()); 
        cmdDefs.add("delConfig", new DelConfigAntTask()); 
        cmdDefs.add("tailLog", new TailLogAntTask()); 
        cmdDefs.add("tail", new TailLogAntTask()); 
        cmdDefs.add("log", new LogAntTask()); 
        cmdDefs.add("setLogLevel", new SetLogLevelAntTask()); 
        cmdDefs.add("quiesceDomain", new QuiesceDomainAntTask()); 
        cmdDefs.add("unquiesceDomain", new QuiesceDomainAntTask()); 
        cmdDefs.add("quiesce", new QuiesceAntTask()); 
        cmdDefs.add("unquiesce", new QuiesceAntTask()); 
        cmdDefs.add("copy", new CopyFilesTask());
        cmdDefs.add("download", new DownloadMultipleFilesTask());
        cmdDefs.add("downloadFile", new DownloadFileTask());
        cmdDefs.add("delete", new DeleteMultipleFilesTask());
        cmdDefs.add("mkdir", new MkdirTask());
        cmdDefs.add("rmdir", new RmdirTask());
        cmdDefs.add("checkpoint", new CheckpointAntTask());
        cmdDefs.add("rollback", new RollbackCheckpointAntTask());
        cmdDefs.add("delCheckpoint", new RemoveCheckpointAntTask());
        cmdDefs.add("save", new SaveAntTask());
        cmdDefs.add("saveAll", new SaveAllAntTask());
        cmdDefs.add("action", new ActionAntTask());
        cmdDefs.add("somaRequest", new AnySOMARequestAntTask());
        cmdDefs.add("secureBackup", new SecureBackupAntTask());
        cmdDefs.add("secureRestore", new SecureRestoreAntTask());
        cmdDefs.add("backup", new BackupAntTask());
        cmdDefs.add("restore", new RestoreAntTask());
        cmdDefs.add("restartDomain", new MetadataDrivenActionAntTask());
        cmdDefs.add("resetDomain", new MetadataDrivenActionAntTask());
        cmdDefs.add("modifyConfig", new ConfigAntTask());
        cmdDefs.add("setConfig", new ConfigAntTask());
        
        cmdDefs.add("flushDocumentCache", new FlushXMLCacheAntTask());
        cmdDefs.add("flushStylesheetCache", new FlushXMLCacheAntTask());
        cmdDefs.add("flushXMLCache", new FlushXMLCacheAntTask());
        cmdDefs.add("flushMiscCache", new FlushMiscCacheAntTask());

        cmdDefs.add("assertStatus", new AssertStatusAntTask());
        cmdDefs.add("assertState", new AssertStateAntTask());
        cmdDefs.add("assertActiveService", new AssertActiveServiceAntTask());
        cmdDefs.add("assertOpenPorts", new AssertOpenPortsAntTask());
        
        cmdDefs.add("testConnection", new TestConnectionAntTask() );
        cmdDefs.add("connect", new TestConnectionAntTask() );
        cmdDefs.add("ping", new PingAntTask() );
        cmdDefs.add("testTcp", new TcpConnectionTestAntTask() );
        cmdDefs.add("exec", new ExecAntTask() );
        cmdDefs.add("createDomain", new CreateDomainAntTask() );
        cmdDefs.add("restartDP", new RestartDPAntTask() );
        cmdDefs.add("firmware", new FirmwareAntTask() );
        cmdDefs.add("remoteCopy", new RemoteCopyAntTask() );
        cmdDefs.add("encrypt", new EncryptAntTask() );
        cmdDefs.add("decrypt", new DecryptAntTask() );
        cmdDefs.add("passwordAlias", new PasswordMapAliasAntTask() );
        cmdDefs.add("license", new LicenseAntTask() );
        cmdDefs.add("exportConf", new ExportHoconTask() );
        cmdDefs.add("updateConfig", new UpdateConfigAntTask() );
        
        cmdDefs.add("updateLogTargetLevel", new UpdateLogTargetLevelTask() );
        // Alias
        cmdDefs.add("updateLogLevel", new UpdateLogTargetLevelTask() );
        cmdDefs.add("syslogTarget", new SyslogLogTargetAntTask() );
        cmdDefs.add("fileLogTarget", new FileLogTargetAntTask() );
        cmdDefs.add("soapLogTarget", new SoapLogTargetAntTask() );

        cmdDefs.add("configReport", new ConfigReportTask() );
        
        return cmdDefs;
    }
    
    public static void main(String [] args) {
        
        int returnCode=runDPBuddyCommand( args );
        System.exit(returnCode);
    }
    
    public static int runDPBuddyCommand(String [] args) {
        DPBuddyCliExecutor executor = new DPBuddyCliExecutor( getCliCmdDefs() );
        
        return executor.execTask( args );
    }
    

}
