package com.myarch.dpbuddy.client;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.net.URL;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.output.DOMOutputter;
import org.xml.sax.SAXException;

import com.myarch.dpbuddy.DPBuddyException;

public class DPSchemaValidator {

    private static Log logger = LogFactory.getLog(DPSchemaValidator.class);
    
    
    public void validateString( String xml ) {
        validate(new StreamSource( new StringReader(xml)));  
    }
    
    public void validate( Element jdomElement ) {
        Document doc = new Document();
        doc.addContent((Element)jdomElement.clone());
        DOMOutputter outputter = new DOMOutputter();
        DOMSource source;
        try {
            source = new DOMSource( outputter.output(doc));
        } catch (JDOMException e) {
            throw new DPBuddyException(e);
        }

        validate( source );  
    }

    
    private void validate(Source xml) {
        long startTime=System.nanoTime();
        
        SchemaFactory factory = 
            SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
        
        Source schemaLocation = locateDPSchema();

        Schema schema;
        try {
            schema = factory.newSchema(schemaLocation);
            Validator validator = schema.newValidator();
            validator.validate(xml);
        }
        catch (SAXException e) {
            throw new DPBuddyException(e);
        } 
        catch (IOException e) {
            throw new DPBuddyException(e);
        }
        
        long elapsedTime=System.nanoTime() - startTime;
        logger.debug( "Validated file in " + elapsedTime / 1000 / 1000 + " ms" );
    }
    
    public final static String DP_SCHEMA_VAR="DP_SCHEMA";
    public final static String DP_SCHEMA_PROP="dp.schema";
    public final static String DP_SCHEMA_FILE_NAME="xml-mgmt-ops.xsd";
    public final static String DP_SCHEMA_PACKAGE="dpschemas";

    public Source locateDPSchema() {
        Source schemaSource;
        // check if a there is a sys. property
        String schemaPath = System.getProperty(DP_SCHEMA_PROP);
        if (schemaPath==null) {
            // check if there is an env. var
            schemaPath = System.getenv(DP_SCHEMA_VAR); 
        }
        
        
        if (schemaPath==null) {
            // get it from the classpath
            String schemaClassPathLocation=DP_SCHEMA_PACKAGE+"/"+DP_SCHEMA_FILE_NAME;
            URL schemaURL=getClass().getClassLoader().getResource(schemaClassPathLocation);
            if (schemaURL==null) {
                throw new DPBuddyException(
                                "DataPower schema file was not found on the classpath at '%s'. " +
                                "You may have to define the property '%s' to point to the schema location", 
                                schemaClassPathLocation, DP_SCHEMA_PROP);
            }
            logger.debug("Using DataPower schema located at "+schemaURL);
            schemaSource=new StreamSource(schemaURL.toString());
        }
        else {        
            File schemaFile = new File(schemaPath);
            
            if (schemaFile.isDirectory())
                schemaFile=new File(schemaFile, DP_SCHEMA_FILE_NAME);
                
            if (!schemaFile.exists()) {
                throw new DPBuddyException("DataPower schema file was not found at '%s'", schemaFile.getAbsolutePath());
            }

            logger.debug("Using DataPower schema located at "+schemaFile.getAbsolutePath());

            schemaSource=new StreamSource(schemaFile);
        }
        
        return schemaSource;
    }

}
