package com.myarch.dpbuddy.config;

import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.xmlutil.JDomUtils;
import com.myarch.dpbuddy.xmlutil.XMLInput;

public class DPConfiguration {

    private Document doc;
    private Element configurationElt;

    public DPConfiguration() {
        doc=new Document();
        Element rootElt=new Element( "datapower-configuration" );
        doc.setRootElement( rootElt );
        configurationElt=new Element( "configuration" );
        rootElt.addContent( configurationElt );
    }

    public DPConfiguration(XMLInput xml) {
        this( xml.getDocument() );
    }

    public DPConfiguration(Document doc) {
        this.doc=doc;
        configurationElt=JDomUtils.getRequiredChildElement( doc.getRootElement(), "configuration" );
    }

    public List<Element> getObjectElements() {
        return configurationElt.getChildren();
    }
    
    public void addElement( Element clazzNameElt ) {
        configurationElt.addContent( clazzNameElt );
    }

    public Element getConfigurationElt() {
        return configurationElt;
    }

    public void addObject( DPObjectConfiguration objectConfig ) {
        configurationElt.addContent( objectConfig.getObjectElt() );
    }

    public void addObject( String clazz, String name) {
        addObject( new DPObjectConfiguration( clazz, name ));
    }

    
    public String getName() {
        Element configRoot=getSingleConfigRoot();

        return JDomUtils.getRequiredAttributeValue( configRoot, "name" );
    }
    
    public void setName( String newName ) {
        Element configRoot=getSingleConfigRoot();
        configRoot.setAttribute( "name", newName );
    }

    public void setElementValue( String eltName, String value ) {
        Element configRoot=getSingleConfigRoot();
        Element elementToUpdate=JDomUtils.getRequiredChildElement( configRoot, eltName );
        elementToUpdate.setText( value );
    }
    
    public void setOrAddElement( String eltName, String value ) {
        Element configRoot=getSingleConfigRoot();
        Element elementToUpdate=configRoot.getChild( eltName );
        if (elementToUpdate == null ){
            addChildElement( eltName, value );
        } else {
        elementToUpdate.setText( value );
        }
    }
    public void addChildElement( String eltName, String value ) {
        Element configRoot=getSingleConfigRoot();
        Element elementToUpdate=new Element(eltName);
        configRoot.addContent( elementToUpdate );
        elementToUpdate.setText( value );
    }

    public void addChildElement( Element elt ) {
        Element configRoot=getSingleConfigRoot();
        configRoot.addContent( elt );
    }

    private Element getSingleConfigRoot() {
        return JDomUtils.getSingleChild( configurationElt );
    }

    public Document getDocument() {
        return doc;
    }
    
    public XMLInput asXmlInput() {
        return new XMLInput( doc );
    }
    
    public String toXmlString() {
        return JDomUtils.docToString( doc );
    }

}
