package com.myarch.dpbuddy.config;

import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.ObjectListParser;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.audit.AuditEvent;
import com.myarch.dpbuddy.cli.BoolOptionHandler;

/**
 * Deletes objects on the device based on the provided patterns
 * 
 */
public class DelConfigAntTask extends BaseDPBuddyTask {

    private List<DPObject> objects=new ArrayList<DPObject>();
    private List<DPObject> excludeObjects=new ArrayList<>();
    
    private boolean isQuiet=false;
    private boolean isDryRun=false;

    private String namePattern;
    private String classPattern;
    
    private String domainPatternsString;

    @Option(name="-domainPatterns", aliases="-domains",
                    usage="Comma-delimited list of regular expression patterns defining what domains to apply the command to. Use '.*' for all domains except the default. Use '.*,default' to include the default.")
    public void setDomainPatterns( String domainPatterns ) {
        this.domainPatternsString=domainPatterns;
    }
    
    @Override
    protected String getDomainPatterns() {
        return domainPatternsString;
    }  
    
    @Option(name="-quiet", handler=BoolOptionHandler.class, usage="Do not fail if no matching object is found. ")
    public void setQuiet( boolean isQuiet ) {
        this.isQuiet=isQuiet;
    }

    @Option(name="-dryRun", handler=BoolOptionHandler.class,
                    usage="Do not delete anything, just print the matched objects.")
    public void setDryRun( boolean isDryRun ) {
        this.isDryRun=isDryRun;
    }

    @Option(name="-namePattern", usage="Regular expression defining what objects to delete")
    public void setNamePattern( String namePattern ) {
        this.namePattern=namePattern;
    }

    @Option(name="-classPattern", usage="Regular expression defining what classes (types) to delete.")
    public void setClassPattern( String classPattern ) {
        this.classPattern=classPattern;
    }

    @Option(name="-objects",
                    usage="Comma-delimited list of object patterns to delete in the format 'classPatter:namePattern, classPatter:namePattern'.")
    public void setObjects( String includeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( includeObjectsList );
        objects.addAll( objs );
    }

    @Option(name="-exclude",
                    usage="Comma-delimited list of object patterns to exclude in the format 'classPatter:namePattern, classPatter:namePattern'.")
    public void setExclude( String excludeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( excludeObjectsList );
        excludeObjects.addAll( objs );
    }
    
    
    public void addObject( DPObject object ) {
        objects.add( object );
    }

    @Override
    public void executeDPTask() throws BuildException {

        if (namePattern == null && classPattern == null && objects.isEmpty()) {
            throw new BuildException( "You must provide namePattern or classPattern or 'objects'" );
        }
        
        DelConfigCommand delConfigCommand=new DelConfigCommand( isQuiet );
        
        if (namePattern != null || classPattern != null) {
            delConfigCommand.addObjectPattern( new DPObject( classPattern, namePattern ) );
        }

        delConfigCommand.addObjectPatterns( objects );
        delConfigCommand.setExcludePatterns( excludeObjects );

        
        delConfigCommand.setDryRun( isDryRun );

        executeRequest( delConfigCommand );

        if (!isDryRun) {
            AuditEvent auditEvent=createAuditEvent( "delConfig" );
            auditEvent.setObjects( objects );
            if (auditEventHandler != null) {
                auditEventHandler.handleAuditEvent( auditEvent, getDevice() );
            }

            saveDomainConfigIfEnabled();
        }
    }

}
