package com.myarch.dpbuddy.configreport;

import java.util.*;

import org.apache.commons.lang3.StringUtils;

public class ConfObjReport {

    private String type;

    private Set<String> identifyingHeaders;

    private List<ReportRow> rows=new ArrayList<>();

    private Set<String> headers=new TreeSet<>();
    private Comparator<ReportRow> identifyingHeadersComparator=new IndentifyingHeadersComparator();

    public ConfObjReport(String type, List<String> identifyingHeaders) {
        this.type=type;
        this.identifyingHeaders=new LinkedHashSet<>( identifyingHeaders );
    }

    public void addRow( ReportRow row ) {
        row.flatten();
        rows.add( row );
        headers.addAll( row.getFlatHeaders() );
    }

    public Set<String> getHeaders() {
        return headers;
    }

    public Set<String> getDiffHeaders() {
        Set<String> contentHeaders=findHeadersForDiffColumns();
        return contentHeaders;
    }

    public Set<String> getIdentifyingHeaders() {
        return identifyingHeaders;
    }

    public Set<String> getAllHeadersOrEmptyIfNoDiff( boolean isDiffMode ) {
        Set<String> identifyingHeaders=getIdentifyingHeaders();

        Set<String> contentHeaders;
        if (isDiffMode) {
            contentHeaders=getDiffHeaders();
            if (contentHeaders.size() == 0) {
                return contentHeaders;
            }
        }
        else {
            contentHeaders=getHeaders();
        }

        Set<String> allHeaders=new LinkedHashSet<>();
        allHeaders.addAll( identifyingHeaders );
        allHeaders.addAll( contentHeaders );

        return allHeaders;

    }

    private Set<String> findHeadersForDiffColumns() {
        Set<String> diffHeaders=new LinkedHashSet<>();
        for (String header : headers) {
            if (!isIdentifyingHeader( header )) {
                Set<String> colVals=new HashSet<>();
                for (ReportRow row : rows) {

                    colVals.add( row.getVal( header ) );
                }
                if (colVals.size() > 1) {
                    diffHeaders.add( header );
                }
            }
        }
        return diffHeaders;
    }

    private boolean isIdentifyingHeader( String header ) {
        return identifyingHeaders.contains( header );
    }

    public String type() {
        return type;
    }

    public List<ReportRow> getRows() {
        Collections.sort( rows, identifyingHeadersComparator );
        return rows;
    }

    private class IndentifyingHeadersComparator implements Comparator<ReportRow> {

        @Override
        public int compare( ReportRow row1, ReportRow row2 ) {

            String identifyingStr1=concatByHeaders( row1, identifyingHeaders );
            String identifyingStr2=concatByHeaders( row2, identifyingHeaders );

            return identifyingStr1.compareTo( identifyingStr2 );
        }

        private String concatByHeaders( ReportRow row, Set<String> headers ) {
            return StringUtils.join( row.toStingsByFlatHeaders( headers ), "" );
        }

    }

}
