package com.myarch.dpbuddy.configreport;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.DPBuddyException;

public class CsvReportRenderer implements ReportRenderer {

    private Log logger=LogFactory.getLog( getClass() );

    private String delim=",";
    private String enclosingChar="\"";


    public CsvReportRenderer() {
    }

    public void render(ConfReport report, File outputDir, boolean isDiffMode) {
    
        logger.info( String.format( "Generating config report CSV files in '%s'", outputDir.getAbsolutePath() ) );
        if (isDiffMode) {
            logger.info( "Only columns with different values will be included" );
        }
        Collection<ConfObjReport> confObjReports=report.confObjReports();
        for (ConfObjReport confObjReport : confObjReports) {
            generateConfObjReport( confObjReport, outputDir, isDiffMode );
        }
    }

    private void generateConfObjReport( ConfObjReport confObjReport, File outputDir, boolean isDiffMode ) {
        
        Set<String> allHeaders=confObjReport.getAllHeadersOrEmptyIfNoDiff( isDiffMode );
        if (allHeaders.size() == 0) {
            logger.info( String.format( "No differences found for the type '%s', excluding from the report",
                            confObjReport.type() ) );
            return;
        }

        File reportFile=new File( outputDir, confObjReport.type() + ".csv" );
        logger.debug( String.format( "Generating report CSV file '%s'", reportFile.getName() ) );
        try (BufferedWriter writer=new BufferedWriter( new FileWriter( reportFile ) );) {
            List<ReportRow> rows=confObjReport.getRows();
            String headersStr=StringUtils.join( allHeaders, delim );
            writer.write( headersStr );
            writer.newLine();
            for (ReportRow row : rows) {
                writer.write( serializeRow( allHeaders, row ) );
                writer.newLine();
            }
        }
        catch (IOException ioe) {
            throw new DPBuddyException( ioe );
        }

    }

    private String serializeRow( Set<String> headers, ReportRow row ) {
        List<String> strs=row.toStingsByFlatHeaders( headers );
        StringBuilder buf=new StringBuilder();
        for (String val : strs) {
            appendVal( buf, val );
        }
        return buf.toString();
    }

    private void appendVal( StringBuilder buf, String val ) {
        String encodedVal="";
        if (val != null) {
            encodedVal=val;
            if (val.contains( delim )) {
                encodedVal=enclosingChar + val + enclosingChar;
            }
        }
        if (buf.length() > 0) {
            buf.append( delim );
        }
        buf.append( encodedVal );
    }

}
