package com.myarch.dpbuddy.configreport;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.config.DPConfiguration;

public class DPObjectsReportMaker {

    private Set<String> includeElements=new TreeSet<>();
    private Set<String> excludeElements=new TreeSet<>();

    public DPObjectsReportMaker(List<String> includeElements, List<String> excludeElements) {
        super();
        this.includeElements=listToLowercaseSet( includeElements );
        this.excludeElements=listToLowercaseSet( excludeElements );
    }

    private Set<String> listToLowercaseSet( List<String> names ) {
        Set<String> namesSet=new TreeSet<>();
        for (String name : names) {
            namesSet.add( name.toLowerCase() );
        }

        return namesSet;
    }

    // Populate the report for one response from export
    public void populateReports( ConfReport report, Document dpConfigXml, Device device ) {
        DPConfiguration dpXmlWrapper=new DPConfiguration( dpConfigXml );
        List<Element> dpoElts=dpXmlWrapper.getObjectElements();
        ReportRow deviceColumns=identifyingColumnsFromDevice( device );
        for (Element dpoElt : dpoElts) {
            DPObject dpo=new DPObject( dpoElt );
            ReportRow dpObjectRow=dpObjectToRow( dpo, dpoElt );
            dpObjectRow.merge( deviceColumns );
            report.addConfObjRow( dpo.getClazz(), dpObjectRow );
        }
    }

    // TODO: env and domain
    private ReportRow dpObjectToRow( DPObject dpo, Element dpoElt ) {
        ReportRow row=eltsToRow( dpoElt.getChildren() );
//        row.addValueColumn( "Class", dpo.getClazz() );
        row.addValueColumn( "Name", dpo.getName() );

        return row;
    }

    private ReportRow identifyingColumnsFromDevice( Device device ) {
        ReportRow row=new ReportRow();
        row.addValueColumn( "Device", device.getName() );
        row.addValueColumn( "Domain", device.getCurrentDomain() );

        return row;
    }

    private ReportRow eltsToRow( List<Element> elts ) {
        ReportRow row=new ReportRow();
        for (Element elt : elts) {
            if (isIncluded( elt )) {
                List<Element> eltChildren=elt.getChildren();
                if (eltChildren.size() > 0) {
                    ReportRow childRow=eltsToRow( eltChildren );
                    row.addRowColumn( elt.getName(), childRow );
                }
                else {
                    String val=elt.getTextNormalize();
                    row.addValueColumn( elt.getName(), val );
                }
            }
        }
        return row;
    }

    private boolean isIncluded( Element elt ) {
        String nameToMatch=elt.getName().toLowerCase();
        if (excludeElements.size() > 0 && excludeElements.contains( nameToMatch )) {
            return false;

        }
        if (includeElements.size() > 0) {
            if (includeElements.contains( nameToMatch )) {
                return true;
            }
            else {
                return false;
            }
        }
        return true;
    }

}
