package com.myarch.dpbuddy.configreport;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.WorkbookUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.myarch.dpbuddy.DPBuddyException;

public class ExcelReportRenderer implements ReportRenderer{

    private Log logger=LogFactory.getLog( getClass() );

    public ExcelReportRenderer() {
    }

    public void render( ConfReport report, File excelFile, boolean isDiffMode ) {
        logger.info( String.format( "Generating Excel config report '%s'", excelFile.getAbsolutePath() ) );
        if (isDiffMode) {
            logger.info( "Only columns with different values will be included" );
        }

        try (Workbook wb=new XSSFWorkbook(); FileOutputStream fileOut=new FileOutputStream( excelFile );) {

            Collection<ConfObjReport> confObjReports=report.confObjReports();

            for (ConfObjReport confObjReport : confObjReports) {
                generateConfObjReport( confObjReport, wb, isDiffMode );
            }

            wb.write( fileOut );
        }
        catch (IOException ioe) {
            throw new DPBuddyException( ioe );
        }
    }

    private void generateConfObjReport( ConfObjReport confObjReport, Workbook wb, boolean isDiffMode )
                    throws IOException {

        Set<String> allHeaders=confObjReport.getAllHeadersOrEmptyIfNoDiff( isDiffMode );
        if (allHeaders.size() == 0) {
            logger.info( String.format( "No differences found for the type '%s', excluding from the report",
                            confObjReport.type() ) );
            return;
        }

        String sheetName=WorkbookUtil.createSafeSheetName( confObjReport.type() );
        Sheet sheet=wb.createSheet( sheetName );

        createExcelRow( sheet, 0, allHeaders );

        List<ReportRow> rows=confObjReport.getRows();
        int i=1;
        for (ReportRow row : rows) {
            List<String> vals=row.toStingsByFlatHeaders( allHeaders );
            createExcelRow( sheet, i++, vals );
        }
    }

    private void createExcelRow( Sheet sheet, int rowI, Collection<String> vals ) {
        Row excelRow=sheet.createRow( rowI );
        int columnI=0;
        for (String val : vals) {
            Cell cell=excelRow.createCell( columnI++ );
            if (StringUtils.isNotBlank( val ) && StringUtils.isNumeric( val )) {
                cell.setCellValue( Integer.valueOf( val ) );
            }
            else {
                cell.setCellValue( val );
            }
        }
    }

}
