package com.myarch.dpbuddy.configreport;

import java.util.*;

import org.apache.commons.lang3.StringUtils;

public class ReportRow {

    private List<ReportColumn> identifyingColumns=new ArrayList<>();

    private List<ReportColumn> columns=new ArrayList<>();

    private Map<String, ReportColumn> headersAndColumns=new HashMap<>();
    private Map<String, ReportColumn> flatHeadersAndColumns=new HashMap<>();

    public ReportColumn getByFlatHeader( String header ) {
        return flatHeadersAndColumns.get( header );
    }

    public List<String> toStingsByFlatHeaders( Set<String> headers ) {
        List<String> strs=new ArrayList<>();
        for (String header : headers) {
            strs.add( getVal( header ) );
        }
        return strs;
    }

    public String getVal(String header) {
        ReportColumn col=flatHeadersAndColumns.get( header );
        String val="";
        if (col != null) {
            val=col.value().toString();
        }
        return val;
    }
    
    public void merge( ReportRow anotherRow ) {
        for (ReportColumn col : anotherRow.columns) {
            addOrOverrideColumn( col );
        }
    }

    private void addOrOverrideColumn( ReportColumn colToAdd ) {

        ReportColumn col=headersAndColumns.get( colToAdd.header() );
        headersAndColumns.put( colToAdd.header(), colToAdd );
        if (col == null) {
            columns.add( colToAdd );
        }
    }

    public void addIdentifyingColumn( String header, String val ) {
        identifyingColumns.add( ReportColumn.valCol( header, val ) );
    }

    public void addValueColumn( String header, Object value ) {
        ReportColumn col=headersAndColumns.get( header );
        if (col != null) {
            col.addValue( value );
        }
        else {
            col=ReportColumn.valCol( header, value );
            headersAndColumns.put( header, col );
            columns.add( col );
        }
    }

    public void addRowColumn( String header, ReportRow value ) {
        ReportColumn col=headersAndColumns.get( header );
        if (col != null) {
            col.addRow( value );
        }
        else {
            col=ReportColumn.rowCol( header, value );
            headersAndColumns.put( header, col );
            columns.add( col );
        }
    }

    public List<ReportColumn> getIdentifyingColumns() {
        return identifyingColumns;
    }

    public Set<String> getFlatHeaders() {
        return flatHeadersAndColumns.keySet();
    }

    public void flatten() {
        List<ReportColumn> columns=getFlattenColumns();
        for (ReportColumn column : columns) {
            flatHeadersAndColumns.put( column.header(), column );
        }
    }

    public List<ReportColumn> getFlattenColumns() {
        return flattenRow( null, this );
    }

    private List<ReportColumn> flattenRow( String path, ReportRow row ) {
        List<ReportColumn> flattenColumns=new ArrayList<>();
        List<ReportColumn> cols=row.columns;
        for (ReportColumn col : cols) {
            if (col.isValue()) {
                String headerPath=concatHeaderPath( path, col.header() );
                flattenColumns.add( ReportColumn.valCol( headerPath, col.value() ) );
            }
            else if (col.isRow()) {
                String headerPath=concatHeaderPath( path, col.header() );
                List<ReportColumn> colsFromNested=flattenRow( headerPath, col.row() );
                flattenColumns.addAll( colsFromNested );
            }
            else if (col.isRowList()) {
                String headerPath=concatHeaderPath( path, col.header() );
                int i=1;
                for (ReportRow rowFromList : col.rows()) {
                    headerPath=concatHeaderIndex( headerPath, i );

                    List<ReportColumn> colsFromNested=flattenRow( headerPath, rowFromList );
                    flattenColumns.addAll( colsFromNested );

                    ++i;
                }
            }
            // TODO: valuelist

        }
        return flattenColumns;
    }

    private String concatHeaderPath( String curPath, String colHeader ) {
        String name="";
        if (!StringUtils.isEmpty( curPath )) {
            name=curPath + "/";
        }

        name+=colHeader;
        return name;
    }

    private String concatHeaderIndex( String curPath, int i ) {
        String name="";
        if (!StringUtils.isEmpty( curPath )) {
            name=curPath + "/";
        }

        name+=i;
        return name;
    }

}
