package com.myarch.dpbuddy.dpimport;


import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.BaseResponse;

public class ImportResponse extends BaseResponse {

    @SuppressWarnings("unused")
    private Log logger = LogFactory.getLog(getClass());
    
    
    public ImportResponse(Document doc) {
        super(doc);
    }

    /**
     * Returns a list of imported object statuses 
     * @return list of imported object statuses
     */
    public List<ImportedObject> getObjects() {
        List<Element> resultElts = getResponseElements("cfg-result");
        List<ImportedObject> objs = new ArrayList<ImportedObject>();
        for( Element elt:resultElts) {
            objs.add(new ImportedObject(elt));
        }
        return objs;
    }

    /**
     * Returns a list of imported files
     * @return list of imported files
     */
    public List<ImportedFile> getFiles() {
        List<Element> resultElts = getResponseElements("file-result");
        List<ImportedFile> files = new ArrayList<ImportedFile>();
        for( Element elt:resultElts) {
            files.add(new ImportedFile(elt));
        }
        return files;
    }
    
    public List<ImportedFile> getFilesForStatus( ImportedFile.Status status) {
        List<ImportedFile> foundFiles=new ArrayList<ImportedFile>();
        List<ImportedFile> files=getFiles();
        for (ImportedFile obj:files){
            
            if ( obj.isInStatus(status) )
                foundFiles.add(obj);
        }
        return foundFiles;

    }

    
    /**
     * Returns a list of successfully imported objects
     * @return list of ImportedObject
     */
    public List<ImportedObject> getImportedObjects(){
        return getObjectsForStatus( ImportedObject.Status.SUCCESS);
    }
    
    public List<ImportedObject> getObjectsForStatus( ImportedObject.Status status) {
        List<ImportedObject> foundObjs=new ArrayList<ImportedObject>();
        List<ImportedObject> objs=getObjects();
        for (ImportedObject obj:objs){
            
            if ( obj.isInStatus(status) )
                foundObjs.add(obj);
        }
        return foundObjs;
    }
    
    
    public void validate() {
        
        // validate that import is present
        getRequiredResponseElements("dp:import");
        /*
        List<ImportedObject> failedObjs=getObjectsForStatus(ImportedObject.Status.FAILURE);
        if (failedObjs.size()>0)
            throw new ImportException(failedObjs);
            */
    }
    
    public void throwIfFailure() {
        List<ImportedObject> failedObjs=getFailedObjects();
        if (failedObjs.size()>0)
            throw new ImportException(failedObjs);
        
    }
    
    public List<ImportedObject> getFailedObjects() {
        return getObjectsForStatus(ImportedObject.Status.FAILURE);    
    }
    
}
