package com.myarch.dpbuddy.encryption;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.config.DPConfiguration;
import com.myarch.dpbuddy.dpimport.ImportCommand;
import com.myarch.dpbuddy.xmlutil.XMLInput;
import com.myarch.util.DecryptionUtils;

public class PasswordMapAliasAntTask extends BaseDPBuddyTask {

    private String aliasName;
    private String password;

    @Option(name="-name", required=true, usage="Name of the alias")
    public void setName( String aliasName ) {
        this.aliasName=aliasName;
    }

    @Option(name="-passwd", required=true, usage="Password. Could be encrypted.")
    public void setPasswd( String password ) {
        this.password=password;
    }

    @Override
    public void executeDPTask() throws BuildException {
        validator.validateRequired( aliasName, "aliasName", "Alias name" );
        validator.validateRequired( password, "passwd", "Password" );
        validator.throwIfErrors();

        DPConfiguration passwordAliasConfig=new DPConfiguration( );
        passwordAliasConfig.addObject("PasswordAlias", aliasName );
        String decrPassword=DecryptionUtils.decryptIfEncrypted( password );
        passwordAliasConfig.addChildElement( "Password", decrPassword );

        ImportCommand importCommand=new ImportCommand();
        XMLInput passwordAliasXmlInput=passwordAliasConfig.asXmlInput();
        // fake name
        passwordAliasXmlInput.setName( "password-alias.xml");
        importCommand.setFile( passwordAliasXmlInput );
        importCommand.execute( getDevice() );

        logger.info( String.format( "Successfully created password alias '%s'", aliasName ) );

        saveDomainConfigIfEnabled();
    }
}
