package com.myarch.dpbuddy.export;

import java.util.ArrayList;
import java.util.List;

import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.ObjectListParser;

public class ExportObject extends DPObject {

    public final static String ALL_CLASSES="all-classes";
    public final static String ALL_OBJECTS="all-objects";

    private List<DPObject> objectPatterns=new ArrayList<>();
    private List<DPObject> excludePatterns=new ArrayList<>();

    
    private Boolean refObjects=true;
    private Boolean refFiles=true;
    private Boolean includeDebug=false;

    private boolean allObjects=false;

    public ExportObject() {
    }

    public ExportObject(DPObject dpObject) {
        super( dpObject );
    }

    public ExportObject(String clazz, String name, Boolean refObjects, Boolean refFiles,
                    Boolean includeDebug) {

        this( clazz, name );

        this.refObjects=refObjects;
        this.refFiles=refFiles;
        this.includeDebug=includeDebug;
    }

    public ExportObject(String clazz, String name) {
        super( clazz, name );
    }
    
    public void setObjects( String includeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( includeObjectsList );
        objectPatterns.addAll( objs );
    }
    
    
    public void setExclude( String excludeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( excludeObjectsList );
        excludePatterns.addAll( objs );
    }

    public List<DPObject> getObjects() {
        List<DPObject> objs=new ArrayList<>(objectPatterns);
        if (!super.isEmpty()) {
            objs.add( this );
        }
        
        return objs;
    }

    public void setObjects( List<DPObject> objectPatterns ) {
        this.objectPatterns=objectPatterns;
    }

    public List<DPObject> getExclude() {
        return excludePatterns;
    }

    public void setExclude( List<DPObject> excludePatterns ) {
        this.excludePatterns=excludePatterns;
    }

    // TODO: clean it up--currently we don't need this, if class and name are
    // omitted, everything is going to be exported
    @SuppressWarnings("unused")
    private void setAllObjects( boolean allObjects ) {
        this.allObjects=allObjects;
    }

    public Boolean getRefObjects() {
        return refObjects;
    }

    public void setRefObjects( Boolean refObjects ) {
        this.refObjects=refObjects;
    }

    public Boolean getRefFiles() {
        return refFiles;
    }

    public void setRefFiles( Boolean refFiles ) {
        this.refFiles=refFiles;
    }

    public Boolean getIncludeDebug() {
        return includeDebug;
    }

    public void setIncludeDebug( Boolean includeDebug ) {
        this.includeDebug=includeDebug;
    }

    public boolean isAllObjects() {
        String includeAllPattern=".*";

        String clazz=getClazz();
        String object=getName();
        
        if (!objectPatterns.isEmpty()) {
            return false;
        }
                
        // TODO: improve by catching null in class and name
        return allObjects || clazz == null && object == null || clazz != null
            && clazz.equalsIgnoreCase( ALL_CLASSES ) && object != null
            && object.equalsIgnoreCase( ALL_OBJECTS ) || clazz != null
            && clazz.equals( includeAllPattern ) && object != null && object.equals( includeAllPattern );
    }

    public ExportObject copyAsAllObjectsExportObject() {
        ExportObject allObjExportObject=new ExportObject( ALL_CLASSES, ALL_OBJECTS );
        allObjExportObject.populateFlags( this );
        return allObjExportObject;
    }
    
    public static ExportObject createAllObjectsExportObject() {
        return new ExportObject( ALL_CLASSES, ALL_OBJECTS );
    }

    public void populateFlags( ExportObject anotherExportObject ) {
        setRefObjects( anotherExportObject.getRefObjects() );
        setRefFiles( anotherExportObject.getRefFiles() );
        setIncludeDebug( anotherExportObject.getIncludeDebug() );
    }
}
