package com.myarch.dpbuddy.filemanagement;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPCommand;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.util.StringPatternUtils;

public class DeleteMultipleFilesCommand implements DPCommand<List<DPFileOrDir>> {

    // TODO: append pattern, quiet
    private Log logger=LogFactory.getLog( getClass() );

    private String pattern=null;
    private boolean isDryRun=false;
    private boolean isMatchRequired=true;
    // suppress printing file names that will be deleted
    private boolean isQuiet=false;

    public DeleteMultipleFilesCommand() {
    }
    
    public DeleteMultipleFilesCommand(String pattern) {
        this.pattern=pattern;
    }
    
    public void appendPattern(String pattern) {
        if (this.pattern==null)
            this.pattern=pattern;
        // append only if it is not already there
        else if (!this.pattern.contains(pattern))
            this.pattern=StringPatternUtils.appendOrPattern(this.pattern, pattern);    
    }
    
    public void setPattern(String pattern) {
        this.pattern=pattern;
    }
    


    public boolean isDryRun() {
        return isDryRun;
    }

    public void setDryRun( boolean isDryRun ) {
        this.isDryRun=isDryRun;
    }

    public void setMatchRequired( boolean isMatchRequired ) {
        this.isMatchRequired=isMatchRequired;
    }
    
    public void setQuiet(boolean isQuiet) {
        this.isQuiet=isQuiet;
    }

    public List<DPFileOrDir> execute( Device device ) {
        if (pattern==null)
            throw new DPBuddyException( "Regexp pattern defining what files to delete is required, it was not provided" );
        if (isQuiet)
            logger.info( "Deleting files and directories matching the pattern '"+pattern+"'" );
        else
            logger.info( "Obtaining list of files and directories to delete for the pattern '"+pattern+"'" );

        List<DPFileOrDir> filesToDelete=device.fetchMatchingFileInfoSkipOverMatchingDirs( pattern );

        if (filesToDelete.size() > 0) {
            if (!isQuiet) {
                if (isDryRun)
                    logger.info( "Dry-run mode is set, the files won't actually be deleted. The following files and directories would be deleted:" );
                else
                    logger.info( "Deleting the following files and directories:" );
            }
            ActionCommand deleteActionCommand=new ActionCommand();

            for (DPFileOrDir fileInfo : filesToDelete) {
                if (!isQuiet)
                    logger.info( fileInfo.getPath() );

                if (fileInfo.isDirectory() && !fileInfo.isFileSystemRoot())
                    deleteActionCommand.addAction( "RemoveDir", "Dir", fileInfo.getPath() );
                else if (fileInfo.isFile())
                    deleteActionCommand.addAction( "DeleteFile", "File", fileInfo.getPath() );
            }

            if (!isDryRun ) {
                deleteActionCommand.execute( device );
            }
            
        }
        else {
            String noFilesMessage=String
                            .format( "Didn't find any files or directories to delete using pattern '%s'",
                                            pattern );
            if (isMatchRequired)
                throw new DPBuddyException( noFilesMessage );
            else
                logger.info( noFilesMessage );
        }
        
        return filesToDelete;
    }

}
