package com.myarch.dpbuddy.filemanagement;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.BaseRequest;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPCommand;
import com.myarch.dpbuddy.DPConst;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.xmlutil.FileSavingStreamParser;
import com.myarch.util.FilePathUtils;

/**
 * Downloads a file from a device ("get-file" SOMA operation).
 */
public class DownloadFileCommand extends BaseRequest implements DPCommand<FileResponse> {

    private Log logger=LogFactory.getLog( getClass() );

    private String dpFileName=null;
    private File toFile;

    private Element getFileElement;

    public DownloadFileCommand() {
        createGetFileElement();
    }

    public void setToFile( String file ) {
        this.toFile=new File( FilePathUtils.toCanonicalPath( file ) );
    }

    public void setFromFile( String dpFileName ) {

        this.dpFileName=Device.prependDefaultFileStore( dpFileName );
        getFileElement.setAttribute( "name", this.dpFileName );
    }

    private void createGetFileElement() {
        getFileElement=new Element( "get-file", DPConst.DP_NS );
        getRequestElement().addContent( getFileElement );
    }

    private void validate() {
        if (dpFileName == null) {
            throw new DPBuddyException( "'fromFile' is required" );
        }
    }

    public FileResponse execute( Device device ) {
        validate();
        Document responseDoc=null;
        if (toFile != null) {
            logger.info( String.format( "Downloading '%s' to '%s'", dpFileName, toFile.getAbsolutePath() ) );
            FileSavingStreamParser parser=new FileSavingStreamParser( toFile );
            responseDoc=device.executeRequest( parser, this );
            // account for zero byte files
            if (!toFile.exists()) {
                try {
                    FileUtils.touch( toFile );
                }
                catch (IOException e) {
                    throw new DPBuddyException( e );
                }
            }
        }
        else {
            logger.info( String.format( "Downloading '%s' ...", dpFileName ) );
            responseDoc=device.executeRequest( this );
        }
        FileResponse response=new FileResponse( responseDoc );

        return response;
    }
}
