package com.myarch.dpbuddy.filemanagement;

import java.io.IOException;
import java.nio.charset.Charset;
import java.util.List;

import org.apache.commons.io.output.StringBuilderWriter;
import org.apache.commons.io.output.WriterOutputStream;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.BaseResponse;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.xmlutil.XMLInput;
import com.myarch.util.EncoderUtils;

public class FileResponse extends BaseResponse {

    @SuppressWarnings("unused")
    private final Log logger=LogFactory.getLog( getClass() );

    public FileResponse(Document doc) {
        super( doc );
    }

    public void validate() {
        getRequiredFileElement();
    }
    
    public String getFileContentAsString() {
        StringBuilderWriter stringWriter=new StringBuilderWriter();
        WriterOutputStream wos=new WriterOutputStream( stringWriter, Charset.defaultCharset() );
        Element sourceFileElement=getRequiredFileElement();
        try {
            EncoderUtils.decode( sourceFileElement.getText(), wos );
            wos.close();
        }
        catch (IOException e) {
            throw new DPBuddyException( "Error reading file content from DataPower response", e );
        }

        return stringWriter.getBuilder().toString();
    }

    public XMLInput getFileAsXML() {
        return new XMLInput( getFileContentAsString() );
    }

    public boolean isSuccess() {
        return ( getFileElement() != null );

    }

    private Element getRequiredFileElement() {
        Element fileElement=getFileElement();
        if (fileElement == null)
            throw new DPBuddyException( "The requested file was not found on the device" );

        return fileElement;
    }

    private Element getFileElement() {
        List<Element> files=getResponseElements( "dp:file" );
        if (files.size() > 1)
            throw new DPBuddyException( "Detected multiple file entries in the response. "
                + "This is not allowed by the schema. XML of the response:\n" + this );
        return files.size() == 0 ? null : files.get( 0 );
    }

}
