package com.myarch.dpbuddy.filemanagement;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.FilenameUtils;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.BaseResponse;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.xmlutil.JDomUtils;

/**
 * Response from 'get-filestore' SOMA request
 */
public class FileStoreInfoResponse extends BaseResponse {
    // private final Log logger = LogFactory.getLog(getClass());

    public FileStoreInfoResponse(Document doc) {
        super( doc );
    }

    public DPFileOrDir getDirInfo( String dirPath ) {

        String normalizedDirPath=FilenameUtils.normalize( dirPath, true );
        if (normalizedDirPath.endsWith( ":/" ))
            normalizedDirPath=normalizedDirPath.substring( 0, normalizedDirPath.length() - 1 );

        String elementName=DPFileOrDir.DIR_ELEMENT_NAME;
        if (normalizedDirPath.endsWith( ":" ))
            elementName="location";

        List<Element> dirElts=JDomUtils.findDPElements( doc,
                        String.format( "//%s[@name='%s']", elementName, normalizedDirPath ) );
        if (dirElts.size() > 1)
            throw new IllegalStateException( "Found multiple directories with the same path "
                + normalizedDirPath );

        if (dirElts.isEmpty())
            return null;

        return new DPFileOrDir( dirElts.get( 0 ) );
    }

    public List<DPFileOrDir> findMatching( String patternStr ) {
        return findMatching( patternStr, false );
    }

    public List<DPFileOrDir> findMatching( String patternStr, boolean filesOnly ) {

        List<DPFileOrDir> matchedFiles=new ArrayList<DPFileOrDir>();
        matchFileElts( matchedFiles, preparePattern( patternStr ), getFileStoreElement().getChildren(),
                        false );

        if (filesOnly) {
            List<DPFileOrDir> matchedFilesOnly=new ArrayList<DPFileOrDir>();
            for (DPFileOrDir file : matchedFiles)
                if (!file.isDirectory())
                    matchedFilesOnly.add( file );

            return matchedFilesOnly;
        }
        else
            return matchedFiles;
    }

    public List<DPFileOrDir> findMatchingSkipOverMatchingDirs( String patternStr ) {

        List<DPFileOrDir> matchedFiles=new ArrayList<DPFileOrDir>();
        matchFileElts( matchedFiles, preparePattern( patternStr ), getFileStoreElement().getChildren(),
                        true);

        return matchedFiles;
    }

    private Pattern preparePattern( String patternStr ) {
        // normalize pattern
        patternStr=patternStr.replace( "//", "/" );
        return Pattern.compile( patternStr );
    }

    private void matchFileElts( List<DPFileOrDir> matchedFiles, Pattern pattern, List<Element> fileElts,
                    boolean skipOverMatchedDirs ) {
        
        for (Element fileElt : fileElts) {
            if (DPFileOrDir.isValidFileDirElement( fileElt )) {
                DPFileOrDir dpFile=new DPFileOrDir( fileElt);

                Matcher matcher=pattern.matcher( dpFile.getPath() );
                if (matcher.matches()) {
                    matchedFiles.add( dpFile );
                    if (dpFile.isDirectory() && ( !skipOverMatchedDirs || dpFile.isFileSystemRoot() ))
                        matchFileElts( matchedFiles, pattern, fileElt.getChildren(), skipOverMatchedDirs );
                }
                else if (dpFile.isDirectory()) {
                    matchFileElts( matchedFiles, pattern, fileElt.getChildren(), skipOverMatchedDirs );
                }

            }
        }

    }

    private Element getFileStoreElement() {
        List<Element> filestoreElts=getRequiredResponseElements( "dp:filestore" );
        if (filestoreElts.size() > 1)
            throw new DPBuddyException( "Encountered multiple 'filestore' elements" );

        return filestoreElts.get( 0 );
    }

}
