package com.myarch.dpbuddy.filemanagement;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.ResultResponse;
import com.myarch.dpbuddy.action.ActionCommand;

/**
 * Creates directory on a filesystem at the device.
 */
public class MkdirCommand extends ActionCommand {
    private Log logger=LogFactory.getLog( getClass() );

    private Set<String> currentDirs=new HashSet<>();

    public void addDir( String dir ) {
        currentDirs.add( Device.prependDefaultFileStore( dir ) );
    }


    private final static String NAME_EXISTS_ERROR="duplicate file name";

    @Override
    public ResultResponse execute( Device device ) {
        ResultResponse resultResponse=null;
        if (currentDirs.size() == 0)   {
            throw new DPBuddyException( "You must provide at least one directory to create" );
        }

        for (String targetDir : currentDirs) {
            if (!Device.isFileStoreRoot( targetDir )) {
                createCreateDirElement( targetDir );
            }
            else {
                logger.info( "Attempt to create filesystem root '" + targetDir + "' is ignored" );
            }
        }

        if (!getActionElements().isEmpty()) {

            resultResponse=device.executeRequestWithResultResponse( request );
            if (!isIgnoreErrors()) {
                resultResponse.validate( getDoActionElement().getChildren(), NAME_EXISTS_ERROR );
            }
        }
        return resultResponse;
    }
    
    private void createCreateDirElement( String targetDir ) {
        addAction( "CreateDir", "Dir", targetDir );
    }

}
