package com.myarch.dpbuddy.filemanagement;

import java.io.InputStream;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.ResultResponse;
import com.myarch.dpbuddy.action.ActionCommand;
import com.myarch.dpbuddy.dpimport.ImportCommand;
import com.myarch.dpbuddy.dpimport.ImportResponse;
import com.myarch.dpbuddy.inpututil.Input;
import com.myarch.dpbuddy.inpututil.InputStringWrapper;
import com.myarch.dpbuddy.xmlutil.XMLInput;
import com.myarch.util.FilePathUtils;

/**
 * Copies remote file to the device, used for firmware upgrades
 */
public class RemoteCopyCommand  {
    
    private Log logger = LogFactory.getLog(getClass());
    
    public final static String DEFAULT_SCRIPT_DIR="local:/cfg-scripts";
    private String scriptCopyToDir=DEFAULT_SCRIPT_DIR;

    public final static String DEFAULT_SCRIPT_NAME="copy-remote.cfg";
    private String scriptName=DEFAULT_SCRIPT_NAME;


    public ResultResponse execute(Device device, String fromUrl, String dpFile, boolean force, String xmlManager) {
        
        if (StringUtils.isBlank( fromUrl )) {
            throw new DPBuddyException(
                            "You must provide the url to download the file from (fromUrl option)" );
        }

        String xmlMgr=xmlManager;
        if (xmlMgr == null && StringUtils.startsWithIgnoreCase( fromUrl, "https:" )) {
            xmlMgr=DEFAULT_XML_MANAGER_NAME;
            logger.info( String.format(
                            "Importing configuration for XML Manager '%s' to support copying from https: ...",
                            xmlMgr ) );
            createXmlManagerForSsl( device );
        }
        
        logger.info( "Sending the script file with the copy command to DataPower ..." );

        CopyFilesCommand copyFilesCommand=new CopyFilesCommand();

        String scriptDirOnTheDevice=scriptCopyToDir;

        String copyCmd=constructCommand( fromUrl, dpFile, xmlMgr, force );
        Input input=createScriptInput( copyCmd );
        copyFilesCommand.addFile( input, scriptDirOnTheDevice );
        copyFilesCommand.execute( device );

        String filePathOnDevice=copyFilesCommand.getRemoteFilePath();
        ActionCommand actionCommand=new ActionCommand( "ExecConfig" );
        actionCommand.addActionChildElement( "URL", filePathOnDevice );

        logger.info( String.format( "Executing command '%s' ...", copyCmd ) );

        return actionCommand.execute( device );

    }

    private final static String XML_MANAGER_CONFIG_RESOURCE="/com/myarch/dpbuddy/dpconfig/xml-manager-for-remote-copy.xml";
    private final static String DEFAULT_XML_MANAGER_NAME="dpbuddy-copy";

    private ImportResponse createXmlManagerForSsl( Device device ) {
        String configResource=XML_MANAGER_CONFIG_RESOURCE;
        return importConfigResource( configResource, device );
    }

    // TODO: to import, resolve vars
    private ImportResponse importConfigResource( String configResource, Device device ) {
        InputStream configInputStream=getClass().getResourceAsStream( configResource );
        if (configInputStream == null)
            throw new DPBuddyException( "Internal DPBuddy exception: resource '%s' was not found",
                            configResource );
        XMLInput templateXml=new XMLInput( configInputStream );
        templateXml.setName( configResource );

        ImportCommand importCommand=new ImportCommand();
        importCommand.setFile( templateXml );
        return importCommand.execute( device );
    }

    private String constructCommand( String fromUrl, String dpFile, String xmlManager, boolean force ) {
        /*
         * copy -f https://delivery04.dhe.ibm.com/sar/CMA/WSA/06l8h/0/xi7521.scrypt4 image:///firmware752.crypt dpbuddy-copy" -domain default -env dr
         */
        String copyCmd="copy"; // +fromUrl;
        if (force) {
            copyCmd+=" -f";
        }
        copyCmd+=" " + fromUrl;
        copyCmd+=" " + constructTargetUrl( fromUrl, dpFile );

        copyCmd+=" " + xmlManager;

        return copyCmd;
    }


    private String constructTargetUrl( String fromUrl, String dpFile ) {
        return constructTargetUrl( fromUrl, Device.DEFAULT_FILESTORE, dpFile );
    }
    
    
    public static String constructTargetUrl( String fromUrl, String defaultFileStore, String dpFile ) {
        if (StringUtils.isBlank( dpFile) ) {
            dpFile="";
        }
        String dpUrl=Device.prependFileStore( defaultFileStore, dpFile );
        // if the file name was not provided
        if (Device.isFileStoreRoot( dpUrl )) {
            String dpFileName=FilePathUtils.getFileNameFromURL( fromUrl );
            dpUrl+=dpFileName;
        }

        return dpUrl;
    }

    private Input createScriptInput( String command ) {
        Input input=new InputStringWrapper( command, scriptName );
        return input;
    }

}
