package com.myarch.dpbuddy.filemanagement.ant;

import java.util.List;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.audit.AuditEvent;
import com.myarch.dpbuddy.audit.AuditEventFile;
import com.myarch.dpbuddy.filemanagement.DPFileOrDir;
import com.myarch.dpbuddy.filemanagement.DeleteMultipleFilesCommand;

/**
 * Deletes files and directories on DataPower based on regexp pattern.
 */
public class DeleteMultipleFilesTask extends BaseDPBuddyTask {

    private String pattern;
    private boolean isMatchRequired=true;
    private boolean isDryRun=false;
    
    @Option(name="-include", required=true, usage="Regular expression pattern defining what files to delete.")
    public void setInclude( String pattern ) {
        this.pattern=pattern;
    }
    @Option(name="-matchRequired", handler=ExplicitBooleanOptionHandler.class, usage="Fail if no files matched the include pattern.")
    public void setMatchRequired( boolean isMatchRequired ) {
        this.isMatchRequired=isMatchRequired;
    }
    
    @Option(name="-dryRun", handler=ExplicitBooleanOptionHandler.class, usage="Print the files that will be deleted but don't actually delete anything.")
    public void setDryRun( boolean isDryRun ) {
        this.isDryRun=isDryRun;
    }

    private String domainPatternsString;

    @Option(name="-domainPatterns", aliases="-domains",
                    usage="Comma-delimited list of regular expression patterns defining what domains to apply the command to. Use '.*' for all domains except the default. Use '.*,default' to include the default.")
    public void setDomainPatterns( String domainPatterns ) {
        this.domainPatternsString=domainPatterns;
    }
    
    @Override
    protected String getDomainPatterns() {
        return domainPatternsString;
    }      
    
    @Override
    public void executeDPTask() throws BuildException {

        validator.validateRequired( pattern, "pattern", "Regexp pattern defining what files/directories to delete" );
        validator.throwIfErrors();

        DeleteMultipleFilesCommand deleteCommand=new DeleteMultipleFilesCommand( pattern );
        deleteCommand.setDryRun( isDryRun );
        deleteCommand.setMatchRequired(isMatchRequired);
        
        List<DPFileOrDir> deletedFiles=deleteCommand.execute( getDevice() );
        
        if (!isDryRun)
            fireAuditEvent(deletedFiles);
    }
    
    private void fireAuditEvent(List<DPFileOrDir> deletedFiles) {
        AuditEvent auditEvent=createAuditEvent("delete");
        for( DPFileOrDir dpFile:deletedFiles){
            auditEvent.addFile( new AuditEventFile(dpFile.getPath()) );
        }
        if (auditEventHandler!=null && deletedFiles.size()>0) {
            auditEventHandler.handleAuditEvent(auditEvent, getDevice());
        }        
    }

}
