package com.myarch.dpbuddy.firmware;

import org.apache.commons.lang3.StringUtils;
import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;

public class FirmwareAntTask extends BaseDPBuddyTask {

    private int timeout=FirmwareCommand.DEFAULT_TIMEOUT;

    @Option(name="-timeout",
                    usage="Time, in seconds, to wait for DataPower to become available after applying the firmware image. -1 disables waiting. Defaults to "
                        + FirmwareCommand.DEFAULT_TIMEOUT + " seconds.")
    public void setTimeout( int timeout ) {
        this.timeout=timeout;
    }
    
    private String fromUrl;
    @Option(name="-fromUrl",
                    usage="URL of the firmware image. http and https protocols are supported. Can also be the path local to DataPower.")
    public void setFromUrl( String fromUrl ) {
        this.fromUrl=fromUrl;
    }
   
    private String xmlManager;

    @Option(name="-xmlManager",
                    usage="Name of the XML manager to use for copying from https URLs. If not provided, DPBuddy will create its own XML Manager and other required objects, named 'dpbuddy-copy'")
    public void setXmlManager( String xmlManager ) {
        this.xmlManager=xmlManager;
    }

    
    private String dpFile;
    
    @Option(name="-dpFile", usage="Absolute path to the image file on DataPower. If not provided, the name will be inferred from 'fromUrl'. The filestore defaults to 'image://'")
    public void setDpFile( String file ) {
        this.dpFile = file;
    }

    @Override
    public void executeDPTask() throws BuildException {
        if (StringUtils.isBlank( fromUrl ) && StringUtils.isBlank( dpFile )) {
            throw new DPBuddyException(
                            "You must provide the either 'fromUrl' to download the image from or 'dpFile' with the location of the file on DataPower" );
        }

        FirmwareCommand command=new FirmwareCommand();
        command.execute( getDevice(),  fromUrl, dpFile, timeout, xmlManager );
    }
    
    @Override
    protected boolean isDefaultDomainOverride(){
        return true;
    }

}
