package com.myarch.dpbuddy.log;

import org.apache.commons.lang3.StringUtils;

import com.myarch.dpbuddy.DPBuddyException;

public enum DPLogLevel {
    EMERG, ALERT, CRITIC, ERROR, WARN, NOTICE, INFO, DEBUG;
    
    public static DPLogLevel fromString( String levelStr ) {
        
        for (DPLogLevel logLevel : DPLogLevel.values()) {
            if (logLevel.toString().equalsIgnoreCase( levelStr ))
                return logLevel;
        }

        throw new DPBuddyException( "Log level must be one of these values: '%s'.\nYou provided: %s", StringUtils.join( DPLogLevel.values(), ", " ), levelStr);
    }

    public String getAbbreviatedString() {
        String abbreviation;
        if (isError())
            abbreviation="E";
        else if (isInfo())
            abbreviation="I";
        else
            abbreviation=toString().substring( 0,1 );
        return abbreviation;
    }
    
    public boolean isError() {
        return this==DPLogLevel.EMERG || this==DPLogLevel.ALERT || this==DPLogLevel.CRITIC || this==DPLogLevel.ERROR; 
    }
    
    public boolean isInfo() {
        return this==DPLogLevel.NOTICE || this==DPLogLevel.INFO;
    }
}
