package com.myarch.dpbuddy.log;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;

import com.google.gson.JsonObject;
import com.myarch.dpbuddy.xmlutil.XMLParsingUtils;

public class DPXmlLogMessageParser {
    private final Log logger=LogFactory.getLog( getClass() );
    /*
    <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
    <SOAP-ENV:Body>
    <log-entry serial='35'>
    <date>Sat Jan 28 2017</date>
    <time utc='1485649173587'>19:19:33</time>
    <date-time>2017-01-28T19:19:33</date-time>
    <type>auth</type>
    <class>rbm</class>
    <object>RBM-Settings</object>
    <level num='3'>error</level>
    <transaction-type>request</transaction-type>
    <transaction>302528</transaction>
    <gtid>302528</gtid>
    <client>127.0.0.1</client>
    <code>0x80800002</code>
    <file></file>
    <message>RBM: Authentication failed - Access Denied</message>
    </log-entry>
    </SOAP-ENV:Body>
    </SOAP-ENV:Envelope> 
     */

    
    public JsonObject parse( String logEntriesXmlStr ) {
        List<JsonObject> jsonObjs= parseMultiple(logEntriesXmlStr);
        if (jsonObjs.size()>1) {
            logger.warn( "Expected only one log entry, received "+jsonObjs.size()+" from DataPower");
        }
        
        JsonObject jsonObj=new JsonObject();
        if (jsonObjs.size()>0) {
            jsonObj=jsonObjs.get( 0 );
        }
        
        return jsonObj;

    }

    
    /**
     * 
     * @param dpLogMessage
     *            String containing XML message, potentially, could be multiple
     * @return
     */
    public List<JsonObject> parseMultiple( String logEntriesXmlStr ) {

        Document logEntriesDoc=XMLParsingUtils.parseString( logEntriesXmlStr );

        LogResponse logEntriesParser=new LogResponse( logEntriesDoc );
        List<LogEntry> logEntries=logEntriesParser.getLogEntries();
        List<JsonObject> logEntriesJson=new ArrayList<>();
        for (LogEntry logEntry : logEntries) {
            JsonObject logEntryJson=logEntryToJson( logEntry );
            logEntriesJson.add( logEntryJson );
        }

        return logEntriesJson;
    }

    private JsonObject logEntryToJson( LogEntry logEntry ) {
        JsonObject logEntryJson=logEntry.toJson();
        AdditionalLogMessageParsing.parse( logEntryJson, logEntry.getMessage());
        
        return logEntryJson;
    }

}
