package com.myarch.dpbuddy.log;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.DPPattern;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.util.FilePathUtils;

/**
 */
public class TailLogAntTask extends BaseDPBuddyTask {

    private int lines=48;
    private Boolean follow;
    private int followInterval;
    private Boolean failOnError;

    private String logTargetName;

    private String failPatternsString;
    private List<Pattern> failPatterns=new ArrayList<Pattern>();

    private String domainPatternsString;
    private List<Pattern> domainPatterns=new ArrayList<Pattern>();

    private String format;

    private File logFile;
    private Boolean appendTimestamp=false;

    private GetLogCommand getLog=new GetLogCommand();

    public void executeDPTask() throws BuildException {

        if (logTargetName != null)
            getLog.setLogName( logTargetName );

        if (follow != null)
            getLog.setFollow( follow );

        if (failOnError != null)
            getLog.setFailOnError( failOnError );
        if (followInterval > 0)
            getLog.setFollowInterval( followInterval );
        if (lines > 0)
            getLog.setNumberOfLines( lines );
        if (format != null)
            getLog.setLogFormat( format );

        if (failPatternsString != null)
            getLog.addFailPatterns( failPatternsString );

        if (failPatterns.size() > 0)
            getLog.addFailPatterns( failPatterns );

        if (domainPatternsString != null)
            getLog.addDomainPatterns( domainPatternsString );

        if (domainPatterns.size() > 0)
            getLog.addDomainPatterns( domainPatterns );

        if (logFile != null) {
            if (appendTimestamp != null && appendTimestamp) {
                String newFileName=FilePathUtils.appendTimestamp( logFile.getAbsolutePath() );
                logFile=new File( newFileName );
            }

            getLog.setLogFile( logFile );
        }

        executeRequest( getLog );
    }

    @Option(name="-lines", aliases="-l", usage="Number of the most recent log entries to display.")
    public void setLines( int lines ) {
        this.lines=lines;
    }

    @Option(name="-follow", aliases="-f", handler=ExplicitBooleanOptionHandler.class,
                    usage="Continuously query the device and tail the log.")
    public void setFollow( Boolean auto ) {
        this.follow=auto;
    }

    @Option(name="-failOnError", handler=ExplicitBooleanOptionHandler.class,
                    usage="Fail whenever an error-level log entry is encountered.")
    public void setFailOnError( Boolean failOnError ) {
        this.failOnError=failOnError;
    }

    @Option(name="-logTarget", usage="Name of the log target defined on the device.")
    public void setLogTarget( String logTargetName ) {
        this.logTargetName=logTargetName;
    }

    @Option(name="-followInterval", usage="Interval in milliseconds used for continuously querying the device.")
    public void setFollowInterval( int followInterval ) {
        this.followInterval=followInterval;
    }

    @Option(name="-domainPatterns", aliases="-domains",
                    usage="Comma-delimited list of regular expressions specifying domains to get logs from.")
    public void setDomainPatterns( String domainPatterns ) {
        this.domainPatternsString=domainPatterns;
    }

    public void addConfiguredDomainPattern( DPPattern domainPattern ) {
        domainPatterns.add( domainPattern.getPattern() );
    }

    @Option(name="-failPatterns",
                    usage="List of comma-delimited regular expression patterns. tailLog will raise an exception if it finds  one of the patterns in the log entry.")
    public void setFailPatterns( String failPatternsString ) {
        this.failPatternsString=failPatternsString;
    }

    public void addConfiguredFail( DPPattern fail ) {
        failPatterns.add( fail.getPattern() );
    }

    @Option(name="-format", usage="Format string.")
    public void setFormat( String format ) {
        this.format=format;
    }

    public void addConfiguredWhere( DPObjectLogFilter filter ) {
        getLog.addLogFilter( filter );
    }

    @Option(name="-logFile", usage="Local file to save the DataPower log file to.")
    public void setLogFile( File logFile ) {
        this.logFile=logFile;
    }

    /**
     * Append timestamp to the log file if it was specified
     * 
     * @param appendTimestamp
     */
    @Option(name="-appendTimestamp", handler=ExplicitBooleanOptionHandler.class,
                    usage="When logFile is set, append the timestamp to the local log file name")
    public void setAppendTimestamp( Boolean appendTimestamp ) {
        this.appendTimestamp=appendTimestamp;
    }

}
