package com.myarch.dpbuddy.logtarget;

import java.io.File;
import java.util.List;

import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.cli.BoolOptionHandler;
import com.myarch.dpbuddy.config.DPConfiguration;
import com.myarch.dpbuddy.dpimport.ImportCommand;
import com.myarch.dpbuddy.xmlutil.XMLInput;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;

public abstract class LogTargetBaseTask extends BaseDPBuddyTask {

    private String name;
    private File configFile;
    // syslog, zulu, Numeric
    private String timestampFormat="syslog";
    private boolean isFeedbackDetection=false;
    private boolean isDisabled=false;
    private String comments;

    @Option(name="-name", required=true, usage="Name of the log target")
    public void setName( String name ) {
        this.name=name;
    }

    @Option(name="-targetConfigFile",
                    usage="Target configuration to import in xml/xcfg file, may contain variables (${}).")
    public void setTargetConfigFile( File configFile ) {
        this.configFile=configFile;
    }

    protected String hoconStr;

    @Option(name="-levelInfo", required=true,
                    usage="List of category:level in HOCON format, e.g., all:notice, mgmt:info")
    public void setLevelInfo( String hoconStr ) {
        this.hoconStr=hoconStr;
    }

    @Option(name="-disabled", usage="Create target in the disabled state. Defaults to false.",
                    handler=BoolOptionHandler.class)
    public void setDisabled( boolean isDisabled ) {
        this.isDisabled=isDisabled;
    }

   
    @Option(name="-feedbackDetection", usage="Turn on feedback detection. Defaults to false.",
                    handler=BoolOptionHandler.class)
    public void setFeedbackDetection( boolean feedbackDetection ) {
        this.isFeedbackDetection=feedbackDetection;
    }

    @Option(name="-comments", usage="Comments")
    public void setComments( String comments ) {
        this.comments=comments;
    }

    @Option(name="-timestampFormat", usage="Timestamp format (syslog, zulu, Numeric)")
    public void setTimestampFormat( String timestampFormat ) {
        this.timestampFormat=timestampFormat;
    }

    protected void appendLogTargetEvents( DPConfiguration targetConfig, Config levelInfoConfig ) {
        List<LogTargetEvent> logTargetEvents=LogTargetEvent.fromConfig( levelInfoConfig );
        if (logTargetEvents.isEmpty()) {
            throw new DPBuddyException( "You must provide at least one category:level pair" );
        }
        for (LogTargetEvent logTargetEvent : logTargetEvents) {
            targetConfig.addChildElement( logTargetEvent.toXmlElement() );
        }
    }

    protected DPConfiguration prepTargetConfig( String templateResouce ) {
        // TODO: validate required for Ant
        Config levelInfoConfig=ConfigFactory.parseString( hoconStr );

        XMLInput templateXml=null;
        if (configFile == null) {
            templateXml=XMLInput.fromResource( templateResouce );
        }
        else {
            templateXml=new XMLInput( configFile );
            templateXml.setName( configFile.getAbsolutePath() );
        }

        templateXml.resolveVars( envPropSet.getPropExpressionEvaluator() );

        DPConfiguration dpConfigOfLogTarget=new DPConfiguration( templateXml );

        dpConfigOfLogTarget.setName( name );

        dpConfigOfLogTarget.setOrAddElement( "TimestampFormat", timestampFormat );

        String adminState=isDisabled ? "disabled" : "enabled";
        dpConfigOfLogTarget.setOrAddElement( "mAdminState", adminState );

        String feedbackDetection=isFeedbackDetection ? "on" : "off";
        dpConfigOfLogTarget.setOrAddElement( "FeedbackDetection", feedbackDetection );
        
        if (comments != null) {
            dpConfigOfLogTarget.setOrAddElement( "UserSummary", comments );
        }

        appendLogTargetEvents( dpConfigOfLogTarget, levelInfoConfig );

        return dpConfigOfLogTarget;

    }

    protected void executeImport( DPConfiguration dpConfigOfLogTarget ) {

        if (logger.isDebugEnabled()) {
            logger.debug( "Creating target using the following configuration:\n" + dpConfigOfLogTarget.toXmlString() );
        }

        ImportCommand importCommand=new ImportCommand();

        XMLInput xmlInput=dpConfigOfLogTarget.asXmlInput();

        String importFileName=configFile != null ? configFile.getAbsolutePath() : "log-target.xcfg";
        xmlInput.setName( importFileName );

        importCommand.setFile( xmlInput );

        importCommand.execute( getDevice() );

        saveDomainConfigIfEnabled();
    }

    protected String str( int num ) {
        return String.valueOf( num );
    }
}
