package com.myarch.dpbuddy.quiesce;

import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.ObjectListParser;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.myarch.dpbuddy.cli.BoolOptionHandler;

public class QuiesceAntTask extends BaseDPBuddyTask {

    private List<DPObject> objects=new ArrayList<>();
    private List<DPObject> excludeObjects=new ArrayList<>();

    private Integer timeout;
    private String namePattern;
    private String classPattern;
    private boolean ignoreDisabled=true;

    @Option(name="-timeout", usage="Time, in seconds, to wait for all objects to reach the desired operational state.")
    public void setTimeout( Integer timeout ) {
        this.timeout=timeout;
    }

    @Option(name="-namePattern", aliases="-name", usage="Regular expression defining object names to quiesce/unquiesce")
    public void setNamePattern( String namePattern ) {
        this.namePattern=namePattern;
    }

    @Option(name="-classPattern", aliases="-class",
                    usage="Regular expression defining what classes (types) to quiesce/unquiesce")
    public void setClassPattern( String classPattern ) {
        this.classPattern=classPattern;
    }

    @Option(name="-objects",
                    usage="Comma-delimited list of object patterns to quiesce/unquiesce in the format 'classPatter:namePattern, classPatter:namePattern'.")
    public void setObjects( String includeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( includeObjectsList );
        objects.addAll( objs );
    }

    @Option(name="-exclude",
                    usage="Comma-delimited list of object patterns to exclude in the format 'classPatter:namePattern, classPatter:namePattern'.")
    public void setExclude( String excludeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( excludeObjectsList );
        excludeObjects.addAll( objs );
    }

    @Option(name="-ignoreDisabled", usage="Do not quiesce/unquiesce disabled objects, defaults to true",
                    handler=BoolOptionHandler.class)
    public void setIgnoreDisabled( boolean ignoreDisabled ) {
        this.ignoreDisabled=ignoreDisabled;
    }

    public void addObject( DPObject object ) {
        objects.add( object );
    }

    @Override
    public void executeDPTask() throws BuildException {
        QuiesceAction quiesceAction=QuiesceAction.QUIESCE;
        if (getTaskName().toLowerCase().contains( QuiesceAction.UNQUIESCE.toString().toLowerCase() )) {
            quiesceAction=QuiesceAction.UNQUIESCE;
        }

        QuiesceUnquiesceObjectsCommand quiesceCommand=new QuiesceUnquiesceObjectsCommand( quiesceAction, timeout );
        quiesceCommand.setIgnoreDisabled( ignoreDisabled );
        quiesceCommand.addObjectPatterns( objects );
        if (namePattern != null || classPattern != null) {
            quiesceCommand.addObjectPattern( new DPObject( classPattern, namePattern ) );
        }

        executeRequest( quiesceCommand );
    }

}
