package com.myarch.dpbuddy.restart;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPCommand;
import com.myarch.dpbuddy.DPConnectionException;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.client.SecureHTTPClient;
import com.myarch.dpbuddy.status.StatusCommand;

public class DPPoller {
    public final static int CONNECTION_TIMEOUT=2000;
    
    private int pollInterval = 5000;
    
    // Add this number to the timeout supplied by user to account for running the status request itself
    // Milis
    private int fudgeTimeout = 3000;
    
    
    

    public void setPollInterval( int pollInterval ) {
        this.pollInterval=pollInterval;
    }
    
    // timeout in seconds
    public void pollUntilSuccess(Device device, int timeout) {
        pollUntilSuccess(device, StatusCommand.VERSION_STATUS_COMMAND, timeout);   
    }
    
    public void pollUntilSuccess(Device device, DPCommand<?> dpCommand, int timeout) {
        long startPollingTime=System.currentTimeMillis();
        long currentTime=startPollingTime;
        
        int realTimeout=timeout*1000+fudgeTimeout;
        reduceHttpTimeouts(device);
        while(true) {
            
            if (tryCommand(device, dpCommand)) {
                device.getDPConnection().resetHttpClient();
                return;
            }
            
            currentTime=System.currentTimeMillis();
            if (currentTime>startPollingTime+realTimeout){
                break;
            }
            delay();
            
        }
        // TODO: timeout exception
        throw new DPBuddyException("DataPower is still down after waiting for %d seconds", timeout);
        
    }
    
    private void reduceHttpTimeouts(Device device) {
        SecureHTTPClient httpClient=device.getDPConnection().getHttpClient( false );
        // TODO: constants
        httpClient.setTimeouts( CONNECTION_TIMEOUT, CONNECTION_TIMEOUT );
    }
    
    private void delay(){
        try {
            Thread.sleep(pollInterval);
        } catch (InterruptedException e) {
            // TODO ?
            e.printStackTrace();
        }
    }
    
    
    private boolean tryCommand(Device device, DPCommand<?> dpCommand) {
        try {
            dpCommand.execute( device );
        }
        catch(DPConnectionException | DPBuddyException e) {
            // TODO: logging
            return false;
        }
        
        return true;
        
    }
    

}
