package com.myarch.dpbuddy.status;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.ObjectListParser;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;

public class AssertActiveServiceAntTask extends BaseDPBuddyTask {

    private List<DPObject> objects=new ArrayList<DPObject>();
    private List<DPObject> excludeObjects=new ArrayList<>();

    @Option(name="-objects",
                    usage="Comma-delimited list of object patterns to check in the format 'class:name, class:name'.")
    public void setObjects( String includeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( includeObjectsList );
        objects.addAll( objs );
    }

    @Option(name="-exclude",
                    usage="Comma-delimited list of object patterns to exclude from the check in the format 'class:name, class:name'.")
    public void setExclude( String excludeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( excludeObjectsList );
        excludeObjects.addAll( objs );
    }
    
    public void addObject( DPObject object ) {
        objects.add( object );
    }

    @Override
    public void executeDPTask() throws BuildException {
        validator.validateRequiredNested( objects, "object" );
        validator.throwIfErrors();
        
        logger.info( "Querying the device to get the list of objects to check ..." );
        Set<ObjectStatus> matchingObjects=getDevice().fetchExistingObjects( objects, excludeObjects );

        ServiceStatusCommand serviceStatusCommand=new ServiceStatusCommand();
        ServiceStatusResponse serviceStatusResponse=serviceStatusCommand.execute( getDevice() );
        serviceStatusResponse.validateObjectsAreActiveServices( matchingObjects );

        logger.info( "All objects were found in the list of active services" );
    }
}
