package com.myarch.dpbuddy.status;


import java.util.Map;

import org.apache.tools.ant.BuildException;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;


public class AssertFreeSpaceAntTask extends BaseDPBuddyTask {
    
    /**
     * Minimally acceptable free space in MB
     */
    private Integer minFreeSpace;
    

    public void setMinFreeSpace(int minFreeSpace) {
        this.minFreeSpace=minFreeSpace;
    }

    public final static String FREE_SPACE_STATUS_NAME="FreeEncrypted";


    @Override
    public void executeDPTask() throws BuildException {
        
        validator.validateRequired(minFreeSpace, "minFreeSpace", "Minimally acceptable free disk space in MB");
        validator.throwIfErrors();
        
        StatusCommand statusRequest = new StatusCommand("FilesystemStatus");
        
        StatusResponse response =  statusRequest.execute(getDevice());
        
        Map<String, String> statusValMap=response.getSingleStatusValueMap();
        String freeSpaceStr=statusValMap.get(FREE_SPACE_STATUS_NAME);
        if (freeSpaceStr==null)
            throw new DPBuddyException("Did not find %s value in the returned status %s", 
                            FREE_SPACE_STATUS_NAME, response.toStatusString());

        int freeSpace=Integer.parseInt(freeSpaceStr);
        
        if (freeSpace<minFreeSpace)
            throw new DPBuddyAssertionException("Free disk space check failed: free space on the device is %d, this is less than the provided threshold %d", 
                            freeSpace, minFreeSpace);
        else 
            logger.info(String.format("Free disk space check passed: free space on the device is %d, more than the provided threshold %d",
                            freeSpace, minFreeSpace));
    }
    

    
}
