package com.myarch.dpbuddy.status;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;
import org.kohsuke.args4j.spi.ExplicitBooleanOptionHandler;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.ObjectListParser;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;

public class AssertStateAntTask extends BaseDPBuddyTask {

    public final static String IGNORE_OBJECTS_PROP_NAME="dp.status.ignore.objects";

    private List<DPObject> objects=new ArrayList<>();
    private List<DPObject> excludeObjects=new ArrayList<>();
    private OperationalState opStateToCheck=OperationalState.UP;
    private String namePattern;
    private String classPattern;

    private boolean isActiveService=false;
    
    private String domainPatternsString;

    @Option(name="-domainPatterns", aliases="-domains",
                    usage="Comma-delimited list of regular expression patterns defining what domains to query. Use '.*' for all domains except the default. Use '.*,default' to include the default.")
    public void setDomainPatterns( String domainPatterns ) {
        this.domainPatternsString=domainPatterns;
    }
    
    @Override
    protected String getDomainPatterns() {
        return domainPatternsString;
    }

    
    @Option(name="-activeService", handler=ExplicitBooleanOptionHandler.class, usage="Check active services")
    public void setActiveService( boolean isActiveService ) {
        this.isActiveService=isActiveService;
    }

    @Option(name="-state", usage="Operational state to check. Allowed values are 'up' or 'down'. ")
    public void setState( String opStateStr ) {
        try {
            opStateToCheck=OperationalState.fromString( opStateStr );
        }
        catch (DPBuddyException dpbe) {
            throw new BuildException( dpbe.getMessage() );
        }
    }

    @Option(name="-namePattern", aliases="-name", usage="Regular expression defining what object names to check.")
    public void setNamePattern( String namePattern ) {
        this.namePattern=namePattern;
    }

    @Option(name="-classPattern", aliases="-class", usage="Regular expression defining what classes (types) to check.")
    public void setClassPattern( String classPattern ) {
        this.classPattern=classPattern;
    }

    @Option(name="-objects",
                    usage="Comma-delimited list of object patterns to check in the format 'class:name, class:name'.")
    public void setObjects( String includeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( includeObjectsList );
        objects.addAll( objs );
    }

    @Option(name="-exclude",
                    usage="Comma-delimited list of object patterns to exclude from the check in the format 'class:name, class:name'.")
    public void setExclude( String excludeObjectsList ) {
        List<DPObject> objs=ObjectListParser.parsePatternList( excludeObjectsList );
        excludeObjects.addAll( objs );
    }

    public void addObject( DPObject object ) {
        objects.add( object );
    }

    @Override
    public void executeDPTask() throws BuildException {
        AssertOperationalStateCommand command=new AssertOperationalStateCommand( opStateToCheck );

        String globalIgnoreList=getProject().getProperty( IGNORE_OBJECTS_PROP_NAME );
        if (globalIgnoreList != null) {
            List<DPObject> objs=ObjectListParser.parsePatternList( globalIgnoreList );
            excludeObjects.addAll( objs );
        }

        if (namePattern != null || classPattern != null) {
            command.addObjectPattern( new DPObject( classPattern, namePattern ) );
        }
        
        command.addObjectPatterns( objects );
        command.setExcludePatterns( excludeObjects );
        OperationalStateResponse opStateResponse=command.execute( getDevice() );
        if (opStateToCheck == OperationalState.UP && isActiveService
            && opStateResponse.getObjectsInState().size() > 0) {
            logger.info( "Checking status of services" );
            checkActiveService( opStateResponse.getObjectsInState() );
        }
    }

    private void checkActiveService( Set<ObjectStatus> objectStatuses ) {
        ServiceStatusCommand serviceStatusCommand=new ServiceStatusCommand();
        ServiceStatusResponse serviceStatusResponse=serviceStatusCommand.execute( getDevice() );
        serviceStatusResponse.validateObjectsAreActiveServices( objectStatuses );
    }
}
