package com.myarch.dpbuddy.status;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPCommand;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.xmltransform.GroovyExpressionEvaluator;

public class AssertStatusCommand implements DPCommand<Boolean> {

    private final Log logger=LogFactory.getLog( getClass() );

    private String statusClassToCheck;
    private String expression;

    public AssertStatusCommand(String statusClassToCheck, String expression) {
        super();
        this.statusClassToCheck=statusClassToCheck;
        this.expression=expression;
    }

    public Boolean execute( Device device ) {

        StatusCommand statusCommand=new StatusCommand( statusClassToCheck );
        StatusResponse statusResponse=statusCommand.execute( device );
        printStatusReport( statusResponse );
        for (Element statusElement : statusResponse.getStatusElements()) {
            boolean passed=executeStatusAssertion( device, statusElement, expression );
            if (!passed) {
                throw new DPBuddyAssertionException(
                                "Assertion '%s' failed checking the status of the following object:\n%s",
                                expression, StatusResponse.statusElementToString( statusElement ) );
            }
        }
        logger.info( String.format( "Successfully validated status of '%s' using assertion '%s'",
                        StatusUtils.getDisplayClass( statusClassToCheck ), expression ) );
        return true;
    }

    private void printStatusReport( StatusResponse statusResponse ) {
        String statusString=statusResponse.toStatusString();
        if (StringUtils.isNotBlank( statusString )) {
            logger.info( statusString );
        }
    }

    private boolean executeStatusAssertion( Device device, Element statusElement,
                    String booleanExpression ) {

        Map<String, Object> props=new HashMap<String, Object>();

        for (Element statusChildElt : statusElement.getChildren()) {
            Object val=null;
            String valStr=statusChildElt.getTextTrim();
            val=valStr;
            if (StringUtils.isNumeric( valStr ))
                val=Integer.parseInt( valStr );

            props.put( statusChildElt.getName(), val );
        }

        GroovyExpressionEvaluator expressionEvaluator=new GroovyExpressionEvaluator();
        expressionEvaluator.setDpConnection( device.getDPConnection() );
        expressionEvaluator.setVariables( props );
        boolean result=expressionEvaluator.evaluateBoolean( expression );

        return result;
    }

}
