package com.myarch.dpbuddy.status;

import com.google.gson.JsonObject;
import com.myarch.dpbuddy.Device;
import com.myarch.dpbuddy.event.EventCreator;
import com.myarch.dpbuddy.event.EventUtils;
import com.myarch.eventmodel.Event;
import com.myarch.eventmodel.EventType;
import com.myarch.util.JsonUtils;

public final class JsonStatusEventCreator {

    private static JsonStatusEventCreator instance=new JsonStatusEventCreator();

    private JsonStatusEventCreator() {

    }

    public static JsonStatusEventCreator instance() {
        return instance;
    }

    // TODO: distinguish btw status and metric
    // TODO maybe to_underscore_case for metrics?, tags (DataPower)

    public final static String DP_OBJECT="dp";

    private EventCreator eventCreator=new EventCreator();

    public Event createStatusEvent( Device device, String domain, JsonObject addnlProps,
                    String statusClass, JsonObject statusJson ) {

        Event event=eventCreator.createDataPowerEvent( device, domain,
                        eventTypeFromStatusClass( statusClass ) );
        JsonUtils.mergeJson( addnlProps, event.json() );

        JsonObject dpJson=EventUtils.getDpJson( event );
        dpJson.addProperty( "status_type", statusClass );
        JsonUtils.mergeJson( statusJson, dpJson );

        event.setType( produceType( statusClass ) );

        setLatest( event );

        return event;
    }

    private void setLatest( Event event ) {
        event.json().addProperty( "latest", true );
    }

    private String produceType( String statusClass ) {
        return DP_OBJECT + "." + "status." + statusClass;
    }

    private EventType eventTypeFromStatusClass( String statusClass ) {
        // FIXME: dist. metrics and statuses
        return EventType.STATUS;
    }

    // TODO: level:error, root cause, stacktrace -- how to put stacktrace into
    // Json (do we need to?)
    public Event createErrorEvent( Device device, EventType eventType, Exception exception ) {
        Event event=eventCreator.createDataPowerEvent( device, device.getCurrentDomain(), eventType );
        // TODO: to Event?
        event.json().addProperty( "message", exception.getMessage() );
        event.json().addProperty( "error_type", exception.getClass().getName() );

        event.setType( DP_OBJECT + ".error" );

        return event;
    }

}
