package com.myarch.dpbuddy.status;

import java.util.ArrayList;
import java.util.List;

import org.jdom2.Element;

import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.log.LogEntry;

public class ObjectStatus extends DPObject{
    
/*
        <ObjectStatus xmlns:env="http://www.w3.org/2003/05/soap-envelope">
          <Class>ConfigDeploymentPolicy</Class>
          <OpState>up</OpState>
          <AdminState>enabled</AdminState>
          <Name>testServiceDeplPolicy</Name>
          <EventCode>0x00000000</EventCode>
          <ErrorCode />
          <ConfigState>saved</ConfigState>
        </ObjectStatus>
      
 */
    private String opState;
    private String configState;
    private String adminState;
    private String errorCode;
    
    private List<LogEntry> logEntries=new ArrayList<LogEntry>();
    
    public ObjectStatus(String clazz, String name) {
        super(clazz, name);
    }

    public ObjectStatus(DPObject dpObject) {
        super(dpObject);
    }

    
    public ObjectStatus(Element statusElement){
        super( statusElement.getChildText("Class"), statusElement.getChildText("Name") );
        opState=statusElement.getChildText("OpState");
        configState=statusElement.getChildText("ConfigState");
        adminState=statusElement.getChildText("AdminState");
        errorCode=statusElement.getChildText("ErrorCode");
    }
    
    public String getOpState() {
        return opState;
    }
    
    public String getAdminState() {
        return adminState;
    }
    
    public boolean isUp(){
        return OperationalState.UP.getOpStateString().equalsIgnoreCase(opState);
    }
        
    public boolean isInState(OperationalState opStateToCheck) {
        return (opStateToCheck.getOpStateString().equals(opState));
    }
    
    public boolean isEnabled() {
        return "enabled".equalsIgnoreCase( adminState );
    }
    
    public String toStatusString() {
        String s=super.toClassObjectStr();
        s+="--"+getOpState();
        
        return s;
    }

    public String getConfigState(){
        return configState;
    }
    
    public String getErrorCode(){
        return errorCode;
    }
    
    public boolean isDeleted(){
        return "deleted".equalsIgnoreCase(configState);
    }
    
    public boolean isSaved(){
        return "saved".equalsIgnoreCase( configState);
    }
    
    public List<LogEntry> getLogEntries() {
        return logEntries;
    }

    public void setLogEntries( List<LogEntry> logEntries ) {
        this.logEntries=logEntries;
    }

    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (!(obj instanceof ObjectStatus))
            return false;
        
        ObjectStatus otherDPObject=(ObjectStatus)obj;
        
        return super.equals(otherDPObject) && opState.equals(otherDPObject.opState) && configState.equals(otherDPObject.configState);
    }

    public int hashCode() { 
        int hash = 1;
        hash = hash * 31+super.hashCode(); 
        hash = hash * 31 
                + (opState == null ? 0 : opState.hashCode());
        hash = hash * 31 
                + (configState == null ? 0 : configState.hashCode());
        
        return hash;
      }

    
    public static String objectStatusListToString(Iterable<ObjectStatus> objectStatuses) {
        StringBuilder buf=new StringBuilder();
        for(ObjectStatus objectStatus: objectStatuses) {
            if (buf.length()>0)
                buf.append(", ");
            buf.append(objectStatus.toStatusString());
        }
        
        return buf.toString();
    }

    
    
    
    
}
