package com.myarch.dpbuddy.status;

import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;

import com.myarch.dpbuddy.DPCommand;
import com.myarch.dpbuddy.Device;
import com.myarch.util.StringPatternUtils;

public class ServiceStatusCommand implements DPCommand<ServiceStatusResponse> {

    private final Log logger=LogFactory.getLog( getClass() );

    public static final String SERVICE_STATUS_CLASS="ServicesStatus";

    private StatusRequest request=new StatusRequest( SERVICE_STATUS_CLASS );

    public ServiceStatusResponse execute( Device device ) {
        return execute( device, device.getCurrentDomain() );
    }

    public ServiceStatusResponse execute( Device device, String domain ) {
        logger.info( ServiceStatusReport.genHeader() );
        return executeNoHeaders( device, domain );
    }

    private ServiceStatusResponse executeNoHeaders( Device device, String domain ) {

        Document responseDoc=device.executeRequest( domain, request );
        ServiceStatusResponse dpResponse=new ServiceStatusResponse( responseDoc );

        String reportString=ServiceStatusReport.genRows( device.getName(), domain, dpResponse );
        if (StringUtils.isNotBlank( reportString )) {
            logger.info( reportString );
        }

        return dpResponse;
    }

    public void executeForDomainPatterns( Device device, String domainPatternsString ) {
        logger.info( ServiceStatusReport.genHeader() );

        Set<String> domains=device
                        .fetchRequiredDomains( StringPatternUtils.stringToPattnernList( domainPatternsString ) );
        for (String domain : domains) {
            executeNoHeaders( device, domain );
        }
    }
}
