package com.myarch.dpbuddy.status;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.BaseResponse;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPObject;
import com.myarch.util.StringPatternUtils;

public class ServiceStatusResponse extends BaseResponse {
    @SuppressWarnings("unused")
    private Log logger=LogFactory.getLog( getClass() );

    private StatusResponse statusResponse;

    public ServiceStatusResponse(Document doc) {
        super( doc );
        statusResponse=new StatusResponse( ServiceStatusCommand.SERVICE_STATUS_CLASS, doc );
    }

    public Set<DPObject> getObjectsWithoutServiceStatus( Iterable<? extends DPObject> serviceObjsToCheck ) {
        Set<DPObject> objectsWithoutStatus=new HashSet<DPObject>();
        for (DPObject obj : serviceObjsToCheck) {
            ServiceStatus serviceStatus=getServiceStatus( obj );
            if (serviceStatus == null)
                objectsWithoutStatus.add( obj );
        }

        return objectsWithoutStatus;
    }

    public ServiceStatus getServiceStatus( DPObject serviceObj ) {
        List<Element> elts=statusResponse.getStatusElements();
        for (Element elt : elts) {
            ServiceStatus serviceStatus=new ServiceStatus( elt );
            if (serviceStatus.equalsDPObject( serviceObj ))
                return serviceStatus;
        }

        return null;

    }

    /**
     * Returns list of all service statuses
     * 
     * @return list of service statuses
     */
    public Set<ServiceStatus> getServiceStatuses() {
        Set<ServiceStatus> serviceStatuses=new HashSet<ServiceStatus>();
        List<Element> elts=statusResponse.getStatusElements();

        for (Element elt : elts) {
            ServiceStatus objStatus=new ServiceStatus( elt );
            serviceStatuses.add( objStatus );
        }

        return serviceStatuses;
    }
    
    public void validatePorts( String portListStr) {
        validatePorts(StringPatternUtils.split( portListStr ));
    }

    public void validatePorts( Set<String> ports) {
        Set<ServiceStatus> serviceStatuses=getServiceStatuses();
        Set<String> notfoundPorts=new HashSet<String>();
        for( String port: ports) {
            if (!isPortUsed(serviceStatuses, port) ) {
                notfoundPorts.add( port );
            }
        }
        if (notfoundPorts.size()>0)
            throw new DPBuddyException(
                        "The following port(s) are not in use by the active services in this domain: %s",
                        StringUtils.join( notfoundPorts, ", " ) );
    }

    private boolean isPortUsed(Set<ServiceStatus> serviceStatuses, String port) {
        for (ServiceStatus serviceStatus:serviceStatuses) {
            if (serviceStatus.portEquals(port)) return true; 
         }
        
        return false;
    }
    
    
    public void validateObjectsAreActiveServices( Iterable<? extends DPObject> objectsToValidate ) {

        Set<DPObject> objectsWithoutActiveService=getObjectsWithoutServiceStatus( objectsToValidate );
        if (objectsWithoutActiveService.size() > 0) {
            throw new DPBuddyException(
                            "The following DataPower objects were not found in the list of active services: %s",
                            DPObject.objectListToString( objectsWithoutActiveService ) );
        }
    }
}
