package com.myarch.dpbuddy.status;

import java.util.HashSet;
import java.util.Set;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.Device;

public class StateChecker {
    
    private int pollStateInterval = 1500;
    
    // Add this number to the timeout supplied by user to account for running the status request itself
    // Milis
    private int fudgeTimeout = 3000;

    
    private Device device;
    
    public StateChecker(Device device){
        this.device=device;
    }
    
   
    public void pollStateUntilTimeout(DPObject dpObject, OperationalState targetOpState, int timeoutInMilis ){
        ObjectStatus obj=new ObjectStatus(dpObject);
        Set<ObjectStatus> objs=new HashSet<ObjectStatus>();
        objs.add(obj);
        pollStateUntilTimeout(objs, targetOpState, timeoutInMilis);
    }
    
    public void pollStateUntilTimeout(Set<ObjectStatus> objectsToCheck, OperationalState targetOpState, int timeoutInMilis){
        // note that we're using clock time, but it's ok for our purposes
        long startPollingTime=System.currentTimeMillis();
        long currentTime=startPollingTime;
        
        int realTimeout=timeoutInMilis+fudgeTimeout;
        
        while(true) {
            
            objectsToCheck=checkObjectState(objectsToCheck, targetOpState); 
            if (objectsToCheck.size()==0)
                return;
            
            currentTime=System.currentTimeMillis();
            if (currentTime>startPollingTime+realTimeout)
                break;
            
            try {
                Thread.sleep(pollStateInterval);
            } catch (InterruptedException e) {
                // TODO ?
                e.printStackTrace();
            }
        }
        
        throw new DPBuddyException("We expected all objects to be in the %s state. " +
                "However, the following objects are still not in this state after waiting for the specified timeout (%d seconds): %s"
                , targetOpState, timeoutInMilis/1000
                , ObjectStatus.objectStatusListToString(objectsToCheck));
    }

    /**
     * Returns list of object status that are not in the target state.
     * @param objectsToCheck
     * @param targetOpState
     * @return
     */
    private Set<ObjectStatus> checkObjectState(Iterable<? extends DPObject> objectsToCheck, 
                    OperationalState targetOpState) {

        Set<ObjectStatus> objectsInStatus=new HashSet<ObjectStatus>();
            
        Set<ObjectStatus> statuses=device.fetchExistingObjects(objectsToCheck);
        for(ObjectStatus status:statuses) {
            if (!status.isInState(targetOpState))
                objectsInStatus.add(status);   
        }
        
        return objectsInStatus;
    }
    

}
