package com.myarch.dpbuddy.xmltransform;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElementNS;
import org.apache.tools.ant.util.XMLFragment;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.xmlutil.JDomUtils;
import com.myarch.dpbuddy.xmlutil.XMLInput;
import com.myarch.hocon.HoconAntUtils;
import com.myarch.hocon.HoconUtils;
import com.myarch.propmanagement.ant.AntExpressionEvaluator;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigList;
import com.typesafe.config.ConfigObject;
import com.typesafe.config.ConfigValue;

public class AddUpdateTransformAction extends TransformAction implements DynamicElementNS {
    
    private boolean isRemoveBeforeAdd=false;
    
    public AddUpdateTransformAction(boolean isRemoveBeforeAdd){
        this.isRemoveBeforeAdd=isRemoveBeforeAdd;
    }
    
    public AddUpdateTransformAction(){
    }
    
    public void setRemoveBeforeAdd( boolean isRemoveBeforeAdd ) {
        this.isRemoveBeforeAdd=isRemoveBeforeAdd;
    }

    
    

    private List<XMLFragment> xmlFragments=new ArrayList<XMLFragment>();
    
    // nested XML
    public Object createDynamicElement(String uri, String name, String qName) {
        
        XMLFragment xmlFragment=new XMLFragment();
        // Bug in Ant -- it doesn't do it automatically
        xmlFragment.setProject(getProject());
        xmlFragments.add(xmlFragment);
        
        return xmlFragment.createDynamicElement(uri, name, qName);
    }

    
    private List<XMLInput> xmlsToAdd=new ArrayList<XMLInput>();
    
    private List<XMLInput> getXMLsToAdd(){
        
        List<XMLInput> xmls=new ArrayList<XMLInput>();
        xmls.addAll(xmlsToAdd);
        AntExpressionEvaluator propEvaluator=new AntExpressionEvaluator(getProject());
        for (XMLFragment xmlFragment:xmlFragments){
            XMLInput xmlInput=new XMLInput(xmlFragment.getFragment());
            xmlInput.clearAntlibNamespace();
            
            xmlInput.resolveVars( propEvaluator );
            
            xmls.add(xmlInput);
        }
        
        if (xmls.size()==0)
            throw new DPBuddyException("You must provide either nested XML fragment or XMLInput object");
        return xmls;
    }
    
    // this is used by unit tests but not by Ant
    public void addXMLToAdd(XMLInput xmlInput){
        xmlsToAdd.add(xmlInput);
    }
    
    
    protected void addXMLToElement(Element parentElement) {
        List<Element> elts=XMLInput.asDetachedElements( getXMLsToAdd() );
        parentElement.addContent(elts);
    }
    
    private String repeatPath;
    public void setRepeat(String repeatPath) {
        this.repeatPath=repeatPath;
    }
    
    public Document transform( Document doc ) {
        Document transformedDoc=doc.clone();
        List<Element> elements=matchAtLeastOneElementIfRequired( transformedDoc );

        for (Element elt : elements) {
            if (isRemoveBeforeAdd) {
                elt.removeContent();
            }
            
            if (repeatPath != null) {
                repeat( elt, repeatPath );
            }
            else {
                addXMLToElement( elt );
            }
            logTransform( "Updated element '%s'", JDomUtils.elementToString( elt ) );
        }

        return transformedDoc;
    }

    private ConfigValue getRepeaterConfig(String repeatPath) {
        Config config=HoconAntUtils.getConfig(getProject());
        if (! HoconUtils.isPathValidAndExists( config, repeatPath )) {
            throw new BuildException( "Repeater path '"+repeatPath+"' is not defined in the configuration" );
        }
        
        ConfigValue repeatFor=HoconAntUtils.getProjectPropertyProvider( getProject()).getWrappedConfigValue(repeatPath);
        
        
        return repeatFor;
        
    }
    
    // TODO: to the base
    private void repeat( Element elt, String repeatPath ) {
        ConfigValue repeatFor=getRepeaterConfig(repeatPath);
        
        if (repeatFor instanceof ConfigList) {
            ConfigList repeatForList=(ConfigList) repeatFor;
            int i=0;
            for (ConfigValue configValue : repeatForList) {
                addToElementWithConfigScope(elt, configValue, null, i+1);
                ++i;
            }

        }
        else if (repeatFor instanceof ConfigObject) {
            ConfigObject repeatForObj=(ConfigObject) repeatFor;
            Set<String> keys=repeatForObj.keySet();
            int i=0;
            for (String key : keys) {
                ConfigValue configValue=repeatForObj.get( key );
                addToElementWithConfigScope(elt, configValue, key, i+1);
                ++i;
            }

        }
        else {
            throw new DPBuddyException(
                            "Expected JSON/configuration object or array, the following value was provided instead: %s",
                            HoconUtils.configValueToString( repeatFor ) );
        }

    }
    
    private void addToElementWithConfigScope(Element elt, ConfigValue configValue, String key, Integer index){
        
        Config config=HoconUtils.configValueToConfig( configValue ) ;
        if (index!=null) {
            config=HoconUtils.addToConfig( config, "_index_", index );
        }
        if (key!=null) {
            config=HoconUtils.addToConfig( config, "_key_", key );
        }

        HoconAntUtils.pushConfig( getProject(), config );

        addXMLToElement( elt );

        HoconAntUtils.popConfig( getProject() );       
    }

    
}
