package com.myarch.dpbuddy.xmltransform;

import java.util.Map;

import org.apache.tools.ant.Project;

import com.myarch.antutil.AntUtils;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPConnection;

import groovy.lang.Binding;
import groovy.lang.GroovyShell;

public class GroovyExpressionEvaluator {

    private Project project;
    private DPConnection dpConnection;
    private Map<String, Object> vars;

    public GroovyExpressionEvaluator() {
    }

    public GroovyExpressionEvaluator(Project project) {
        this.project=project;
    }

    public GroovyExpressionEvaluator(Project project, DPConnection dpConnection) {
        this( project );
        this.dpConnection=dpConnection;
    }

    public void setProject( Project project ) {
        this.project=project;
    }

    public void setDpConnection( DPConnection dpConnection ) {
        this.dpConnection=dpConnection;
    }

    public void setVariables( Map<String, Object> vars ) {
        this.vars=vars;
    }

    public boolean evaluateBoolean( String expression ) {
        AntBinding binding=new AntBinding();
        if (project != null) {
            binding.setProject( project );
        }
        if (dpConnection != null)
            setDPConnectionVariables( binding, dpConnection );
        if (vars != null)
            setVariables( binding, vars );

        GroovyShell shell=new GroovyShell( binding );
        Object value=shell.evaluate( expression );

        if (!( value instanceof Boolean )) {
            throw new DPBuddyException(
                            "The expression '%s' is not a boolean expression, the expression's return type is %s",
                            expression, value != null ? value.getClass().getName() : "null" );
        }

        return (Boolean) value;
    }

    private void setDPConnectionVariables( Binding binding, DPConnection dpConnection ) {
        binding.setVariable( "dp", dpConnection );
    }

    private static Binding setVariables( Binding binding, Map<String, Object> vars ) {

        for (Map.Entry<String, Object> prop : vars.entrySet()) {
            binding.setVariable( prop.getKey(), prop.getValue() );
            binding.setVariable( AntUtils.toCamelCase( prop.getKey() ), prop.getValue() );
        }

        return binding;
    }

    public static class AntBinding extends Binding {
        private Project project;

        public final static String ANT_PROJECT_NAME="antProject";

        AntBinding() {
            super();
        }

        AntBinding(Project project) {
            this.project=project;
        }

        public void setProject( Project project ) {
            this.project=project;
        }

        @Override
        public Object getVariable( String name ) {
            Object val=getAntProperty( name );
            if (val == null) {
                val=getPredefined( name );
            }
            if (val == null) {
                val=super.getVariable( name );
            }
            return val;
        }

        @Override
        public boolean hasVariable( String name ) {
            if (getAntProperty( name ) != null) {
                return true;
            }

            return super.hasVariable( name );
        }

        private Object getAntProperty( String name ) {
            if (project == null) {
                return null;
            }
            Object val=project.getProperty( name );
            if (val == null) {
                val=project.getProperty( AntUtils.camelCaseToDotNotation( name ) );
            }
            return val;
        }

        private Object getPredefined( String name ) {
            if (ANT_PROJECT_NAME.equals( name )) {
                return project;
            }
            return null;
        }
    }

}
