package com.myarch.dpbuddy.xmltransform;

import java.util.ArrayList;
import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.DPObject;
import com.myarch.dpbuddy.xmlutil.JDomUtils;

public class IncludeExcludeDPObjectAction extends TransformAction {

    private DPObject filter=new DPObject();

    private boolean isExclude=false;
    
    public IncludeExcludeDPObjectAction(DPObject filter){
        this.filter=filter;
    }
    
    public IncludeExcludeDPObjectAction(){
        super();
    }
    
    public IncludeExcludeDPObjectAction(String classPattern, String namePattern){
        this.filter=new DPObject(classPattern, namePattern);
    }
    
    public void setClass(String clazz) {
        filter.setClass(clazz);
    }
    
    public void setName(String name) {
        filter.setName(name);
    }
    
    public void setExclude(boolean isExclude) {
        this.isExclude=isExclude;
    }

   // TODO: fix the hierarchy
    

    @Override
    public Document transform(Document doc) {
        
        Document transformedDoc=doc.clone();
        // TODO: handle isMatchRequired
        Element configElt=transformedDoc.getRootElement().getChild("configuration");
        if (configElt==null)
            throw new DPBuddyException("Did not find 'configuration' element in the provides XML file. " +
            		"dpInclude/dpExclude can only work with XML files in the DataPower configuration format");
        
        List<Element> objectElts=configElt.getChildren();
        List<Element> elementsToRemove=new ArrayList<Element>();
        int matchesCount=0;
        for(Element objectElt:objectElts) {
            boolean matched=objectMatches(objectElt);
            if (matched)
                ++matchesCount;
            
            if (isExclude && matched || !isExclude && !matched) {
                elementsToRemove.add(objectElt);
                // remove
            }
            
        }

        if ( (elementsToRemove.size()==0 && isExclude || (!isExclude && matchesCount==0) )&& isMatchRequired()) 
            throw new DPBuddyException("Action %s did not find any matching elements", toDisplayString());

        for(Element eltToRemove:elementsToRemove) {
            logTransform("Removing element '%s' because of inclusion/exclusion rules",
                            JDomUtils.elementToString(eltToRemove));
            eltToRemove.detach();
        }

        
        return transformedDoc;
    }
    
    private boolean objectMatches(Element objectElt) {
        DPObject dpObject=new DPObject(objectElt);
        return DPObject.objectMatchesFilter(dpObject, filter);
    }
    
    public String toDisplayString(){
        String actionName=isExclude?"dpExclude":"dpInclude";
        String displayString=String.format("%s('%s')", actionName, filter.toString());
        return displayString;
    }


}
