package com.myarch.dpbuddy.xmltransform;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.xmlutil.JDomUtils;

public class ReplaceTextTransformAction extends TransformAction {

    private String replaceWith;
    private String textToReplace;

    public void setReplaceWith( String value ) {
        replaceWith=value;
    }

    public void setTextToReplace( String textToReplace ) {
        this.textToReplace=textToReplace;
    }

    public Document transform( Document doc ) {

        if (replaceWith == null)
            throw new DPBuddyException( "You have to specify 'value' attribute" );

        if (textToReplace == null)
            throw new DPBuddyException( "You have to specify 'valueToReplace' attribute" );

        Document transformedDoc=doc.clone();
        boolean atLeastOneElementOrAttributeMatched=false;
        // this didn't work because of the concurrent modification exception:
        // IteratorIterable<Element> elts=transformedDoc.getDescendants( new
        // ElementFilter() );
        List<Element> elements=matchElements( transformedDoc, "//*" );
        for (Element elt : elements) {
            String eltText=elt.getTextTrim();
            if (StringUtils.isNotBlank( eltText ) && eltText.contains( textToReplace )) {
                String newEltText=eltText.replace( textToReplace, replaceWith );
                logTransform( "Replaced text '%s' contained within element '%s' with '%s'", eltText,
                                JDomUtils.elementToString( elt ),
                                maskIfNeeded( elt.getName(), newEltText ) );

                elt.setText( newEltText );
                atLeastOneElementOrAttributeMatched=true;
            }
            for (Attribute attr : elt.getAttributes()) {
                String attrVal=attr.getValue();
                if (attrVal.contains( textToReplace )) {
                    String newAttrVal=attrVal.replace( textToReplace, replaceWith );
                    logTransform( "Replaced the value '%s' of attribute '%s' with '%s'", attrVal,
                                    JDomUtils.attributeToString( attr ),
                                    maskIfNeeded( attr.getName(), newAttrVal ) );
                    attr.setValue( newAttrVal );
                    atLeastOneElementOrAttributeMatched=true;
                }
            }
        }

        if (!atLeastOneElementOrAttributeMatched) {
            String msg=String.format(
                            "Did not find any elements or attributes containing the value '%s' for action ReplaceText",
                            textToReplace );
            if (isMatchRequired()) {
                throw new DPBuddyException( msg );
            }
            else {
                logTransform( msg );
            }

        }

        return transformedDoc;
    }

    @Override
    public String toDisplayString() {
        String displayString=String.format( "%s('%s'->'%s')", getTransformActionName(), textToReplace,
                        replaceWith );
        return displayString;
    }
}
