package com.myarch.dpbuddy.xmltransform;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.xmltransform.GroovyExpressionEvaluator.AntBinding;
import com.myarch.dpbuddy.xmlutil.JDomUtils;

import groovy.lang.GroovyShell;

public class SetTextTransformAction extends TransformAction {
    @SuppressWarnings("unused")
    private Log logger=LogFactory.getLog( getClass() );

    private String newValue;
    private String expression;

    public void setValue( String value ) {
        newValue=value;
    }

    public void setExpression( String expression ) {
        this.expression=expression;
    }

    public Document transform( Document doc ) {

        if (newValue == null && expression == null)
            throw new DPBuddyException( "You have to specify 'value' or 'expression' attributes" );

        Document transformedDoc=doc.clone();

        List<Element> elements=matchElements( transformedDoc );
        boolean atLeastOneElementOrAttributeMatched=false;
        for (Element elt : elements) {
            String valueToSet=produceNewValue( transformedDoc, elt, null );
            logTransform( "Setting the text value of element '%s' to '%s'",
                            JDomUtils.elementToString( elt ),
                            maskIfNeeded( elt.getName(), valueToSet ) );
            elt.setText( valueToSet );
            atLeastOneElementOrAttributeMatched=true;
        }

        List<Attribute> matchedAttributes=matchAttributes( transformedDoc );
        for (Attribute attr : matchedAttributes) {
            String valueToSet=produceNewValue( transformedDoc, null, attr );
            logTransform( "Setting the value of attribute '%s' to '%s'",
                            JDomUtils.attributeToString( attr ),
                            maskIfNeeded( attr.getName(), newValue ) );
            attr.setValue( valueToSet );
            atLeastOneElementOrAttributeMatched=true;
        }

        if (!atLeastOneElementOrAttributeMatched) {
            String msg=String.format(
                            "Did not find any elements or attributes matching xpath expression '%s' for action SetText",
                            getXPath() );
            if (isMatchRequired()) {
                throw new DPBuddyException( msg );
            }
            else {
                logTransform( msg );
            }
        }

        return transformedDoc;
    }

    private String produceNewValue( Document doc, Element elt, Attribute attr ) {
        String newVal=newValue;
        if (expression != null) {
            newVal=executeGroovyExpression( doc, elt, attr );
        }
        return newVal;
    }

    private String executeGroovyExpression( Document doc, Element elt, Attribute attr ) {
        AntBinding binding=new AntBinding();
        if (getProject() != null){
            binding.setProject( getProject() );
        }
        binding.setVariable( "document", doc );
        if (elt != null) {
            binding.setVariable( "element", elt );
            binding.setVariable( "currentValue", elt.getText() );
        }
        if (attr != null) {
            binding.setVariable( "attribute", attr );
            binding.setVariable( "currentValue", attr.getValue() );
        }

        if (newValue != null)
            binding.setVariable( "newValue", newValue );

        GroovyShell shell=new GroovyShell( binding );
        Object value=shell.evaluate( expression );

        return value.toString();
    }

}
