package com.myarch.dpbuddy.xmltransform;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.types.DataType;
import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Namespace;
import org.jdom2.filter.Filters;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.encryption.PasswordUtils;
import com.myarch.dpbuddy.xmlutil.JDomUtils;

// note: we have to extend from DataType so we can have access to ant Project (needed by UpdateAdd action)
public abstract class TransformAction extends DataType {
    private Log logger=LogFactory.getLog( getClass() );

    private String xpath;

    private String ifExpression;

    private List<Namespace> namespaces=new ArrayList<Namespace>();

    private boolean isMatchRequired=true;
    // private boolean isVerbose;
    // private TransformActionContainer container;

    private TransformLogger transformLogger;

    public TransformAction() {
        transformLogger=new TransformLogger( getProject(), logger );
    }

    public TransformAction(String xpath, String value) {
        this();
        this.xpath=xpath;
    }

    public String getIf() {
        return ifExpression;
    }

    protected void logTransform( String msg, Object... args ) {
        transformLogger.log( msg, args );
    }

    private boolean isVerboseSet=false;

    public void setVerbose( boolean isVerbose ) {
        isVerboseSet=true;
        transformLogger.setVerbose( isVerbose );
        transformLogger.setProject( getProject() );
    }

    boolean isVerboseSet() {
        return isVerboseSet;
    }

    public String getXPath() {
        return xpath;
    }

    public void setXPath( String xpath ) {
        this.xpath=xpath;
    }

    public void setNamespaces( List<Namespace> namespaces ) {
        this.namespaces=namespaces;
    }

    public void addNamespace( Namespace namespace ) {
        namespaces.add( namespace );
    }

    public boolean isMatchRequired() {
        return isMatchRequired;
    }

    public void setMatchRequired( boolean isMatchRequired ) {
        this.isMatchRequired=isMatchRequired;
    }

    public String getIfExpression() {
        return ifExpression;
    }

    public void setIf( String ifExpression ) {
        this.ifExpression=ifExpression;
    }

    protected List<Element> matchAtLeastOneElementIfRequired( Document doc ) {
        List<Element> matchedElements=matchElements( doc );
        if (matchedElements.size() == 0 && isMatchRequired)
            throw new DPBuddyException(
                            "Did not find any elements matching xpath expression '%s' for action '%s'",
                            xpath, getClass().getSimpleName() );

        return matchedElements;
    }

    protected List<Element> matchElements( Document doc ) {
        return matchElements( doc, this.xpath );
    }

    protected List<Element> matchElements( Document doc, String xpath ) {
        XPathExpression<Element> xpathElements=XPathFactory.instance().compile( xpath, Filters.element(),
                        new HashMap<String, Object>(), namespaces );
        List<Element> matchedElements=xpathElements.evaluate( doc );
        if (matchedElements.size() > 0)
            logger.debug( String.format( "XPath '%s' matched the following elements: '%s'", xpath,
                            JDomUtils.elementsToString( matchedElements ) ) );
        else
            logTransform( String.format( "XPath '%s' did not match any elements", xpath ) );
        return matchedElements;
    }

    protected List<Attribute> matchAttributes( Document doc ) {
        XPathExpression<Attribute> xpathAttrs=XPathFactory.instance().compile( xpath,
                        Filters.attribute(), new HashMap<String, Object>(), namespaces );
        List<Attribute> matchedAttributes=xpathAttrs.evaluate( doc );
        if (matchedAttributes.size() > 0)
            logger.debug( String.format( "XPath '%s' matched the following attributes: '%s'", xpath,
                            JDomUtils.attributesToString( matchedAttributes ) ) );

        return matchedAttributes;
    }

    public String toDisplayString() {
        String displayString=String.format( "%s('%s')", getTransformActionName(), getXPath() );
        return displayString;
    }

    public String getTransformActionName() {

        String className=getClass().getSimpleName();
        String actionName=className;
        int suffixPos=className.indexOf( "TransformAction" );
        if (suffixPos > 0) {
            actionName=className.substring( 0, suffixPos );
        }
        return actionName;
    }

    protected String maskIfNeeded( String attrEltName, String val ) {
        return PasswordUtils.maskIfNeeded( attrEltName, val );
    }

    public abstract Document transform( Document doc );
}
