package com.myarch.dpbuddy.xmlutil;

import java.io.IOException;

import org.apache.commons.lang3.StringUtils;
import org.jdom2.Attribute;
import org.jdom2.Element;
import org.jdom2.output.support.AbstractXMLOutputProcessor;
import org.jdom2.output.support.FormatStack;
import org.jdom2.util.NamespaceStack;

import com.myarch.dpbuddy.encryption.PasswordUtils;

public class CutBase64XMLOutputProcessor extends AbstractXMLOutputProcessor {
    public final static String BASE64_ENCODED_REPLACEMENT_TEXT="Base64-encoded text ...";

    @Override
    public void printElement( java.io.Writer out, FormatStack fstack, NamespaceStack nstack, Element element )
                    throws IOException {
        if (isBase64ContainingElement( element )) {
            String eltS=elementToString( element, BASE64_ENCODED_REPLACEMENT_TEXT );
            out.write( eltS );
        }
        else if (!JDomUtils.hasChildElements( element ) && PasswordUtils.isPassword( element.getName() )) {
            String eltS=elementToString( element, "*****" );
            out.write( eltS );
        }
        else
            super.printElement( out, fstack, nstack, element );
    }

    private boolean isBase64ContainingElement( Element elt ) {
        String eltName=elt.getName();
        return "set-file".equals( eltName ) || "input-file".equals( eltName )
            || isResponseChild( elt ) && "file".equals( eltName );
    }

    private boolean isResponseChild( Element elt ) {
        return elt.getParentElement() != null && elt.getParentElement().getName().equals( "response" );
    }

    private String elementToString( Element elt, String newText ) {
        StringBuilder buf=new StringBuilder( "<" );
        buf.append( elementNameToString( elt ) );
        for (Attribute attr : elt.getAttributes()) {
            buf.append( ' ' + attr.getName() + '=' );
            buf.append( '"' + attr.getValue() + '"' );
        }

        buf.append( '>' );
        if (newText == null)
            newText=elt.getText();
        if (StringUtils.isNotBlank( newText ))
            buf.append( newText );
        buf.append( "</" );
        buf.append( elementNameToString( elt ) );
        buf.append( '>' );
        return buf.toString();
    }

    private String elementNameToString( Element elt ) {
        StringBuilder buf=new StringBuilder();
        if (StringUtils.isNotBlank( elt.getNamespacePrefix() )) {
            buf.append( elt.getNamespacePrefix() + ":" );
        }
        buf.append( elt.getName() );
        return buf.toString();
    }

}
