package com.myarch.dpbuddy.xmlutil;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import javax.xml.stream.XMLStreamException;

import org.apache.commons.codec.binary.Base64OutputStream;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.stax2.XMLStreamReader2;
import org.codehaus.stax2.util.StreamReader2Delegate;

import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.util.FilePathUtils;

public class FileSavingStreamParser extends StreamReader2Delegate {

    private final Log logger=LogFactory.getLog( getClass() );

    private OutputStream outputStream;
    private File fileOrDir;

    public FileSavingStreamParser(XMLStreamReader2 streamReader, OutputStream outputStream) {
        super( streamReader );
        this.outputStream=outputStream;
    }

    public FileSavingStreamParser(OutputStream outputStream) {
        super( null );
        this.outputStream=outputStream;
    }

    public FileSavingStreamParser(File fileOrDir) {
        super( null );
        this.fileOrDir=fileOrDir;
    }

    private boolean isFileElement=false;
    private String fileNameFromAttr;

    public String getFileName() {
        return fileNameFromAttr;
    }
    
    @Override
    public int getEventType() {
        checkIfFileElement();
        return super.getEventType();
    }

    private void checkIfFileElement() {
        if (super.getEventType() == START_ELEMENT && isFileElementName()) {
            isFileElement=true;
            fileNameFromAttr=getAttributeValue(null, "name");
        }
        else if (super.getEventType() == END_ELEMENT && isFileElementName())
            isFileElement=false;
    }

    private boolean isFileElementName() {
        return "file".equalsIgnoreCase( getLocalName() );

    }

    @Override
    public int next() throws XMLStreamException {
        checkIfFileElement();
        return super.next();
    }

    @Override
    public String getText() {
        String text;
        if (isFileElement) {
            text="Encoded Text";
            saveEncodedText();
        }
        else
            text=super.getText();

        return text;
    }

    
    private void saveEncodedText() {
        if (fileOrDir != null) {
            try {
                
                File fileToSaveTo=prepareForSaving(fileOrDir);
                outputStream=new FileOutputStream( fileToSaveTo );
                logger.debug( "Downloading the file from the device and saving it to " + fileOrDir );
            }
            catch (FileNotFoundException e) {
                throw new DPBuddyException( "Error creating file '%s' ", e, fileOrDir.getAbsolutePath() );
            }
        }
        Base64OutputStream base64os=new Base64OutputStream( outputStream, false );
        OutputStreamWriter osw=new OutputStreamWriter( base64os );

        try {
            getText( osw, false );
        }
        catch (IOException e) {
            throw new DPBuddyException( e );

        }
        catch (XMLStreamException e) {
            throw new DPBuddyException( e );
        }
        finally {

            try {
                osw.close();
                base64os.close();
                if (fileOrDir != null)
                    outputStream.close();
            }
            catch (IOException e) {
                throw new DPBuddyException( e );
            }
        }
    }
    
    private File prepareForSaving( File destinationFileOrDir ) {
        File destinationFile=destinationFileOrDir;

        if (destinationFileOrDir.isDirectory()) {
            String sourceFileURL=fileNameFromAttr;
            if (sourceFileURL == null) {
                throw new DPBuddyException( "You provided directory '%s' to save the file to "
                    + "but the file name was not provided on the response from the device",
                                destinationFileOrDir.toString() );
            }
            String fileName=FilePathUtils.removePrefix( sourceFileURL );
            destinationFile=new File( destinationFileOrDir, fileName );
        }

        File dir=destinationFile.getParentFile();
        if (dir != null)
            dir.mkdirs();
        
        return destinationFile;
    }

}
