package com.myarch.dpbuddy.xmlutil;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Namespace;
import org.jdom2.filter.ElementFilter;
import org.jdom2.util.IteratorIterable;
import org.w3c.dom.DocumentFragment;

import com.myarch.antutil.PropResolverForXML;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.inpututil.Input;
import com.myarch.dpbuddy.xmltransform.TransformActionContainer;
import com.myarch.propmanagement.ExpressionEvaluator;

public class XMLInput implements Input{
    
    
    private Document doc;
    
    private File file;
    
    public XMLInput( String xmlStr ){
        doc=XMLParsingUtils.parseString(xmlStr);    
    }

    public XMLInput(File xmlFile){
        if ( !xmlFile.exists() )
            throw new DPBuddyException("XML file '%s' does not exist", xmlFile.getAbsolutePath());
        file=xmlFile;
        doc=XMLParsingUtils.parseFile(xmlFile);    
    }
    
        
    public XMLInput( InputStream inputStream ){
        if ( inputStream==null )
            throw new DPBuddyException("Provided input stream is null, most likely the file behind this input stream doesn't exist");
        
        doc=XMLParsingUtils.parse(inputStream);
    }
    
    public XMLInput( DocumentFragment dom ){
        doc=XMLParsingUtils.parseDOM(dom);
    }

    public XMLInput( Document doc ){
        this.doc=doc;
    }

    public static XMLInput fromResource(String resource) {
        InputStream is=XMLInput.class.getResourceAsStream( resource );
        if (is == null)
            throw new DPBuddyException( "Internal DPBuddy exception: resource '%s' was not found", resource );
        
        XMLInput templateXml=new XMLInput( is );
        templateXml.setName( resource );

        return templateXml;
    }
    
    
    public Document getDocument() {
        return doc;
    }
    
    
    /**
     * Resolve variables in XML specified using ${ } notation against 
     * properties defined in the provided Ant project
     * @param antProject Ant project object
     */
    public void resolveVars( ExpressionEvaluator evaluator ) {
        PropResolverForXML resolver = new PropResolverForXML(evaluator);
        resolver.resolveVarsInXML(doc);
    }
    
    public void transform(TransformActionContainer transformActionContainer) {
        doc=transformActionContainer.transform(doc);
    }

    public InputStream getInputStream() {
        return new ByteArrayInputStream(JDomUtils.docToString(doc).getBytes());
    }
    
    public String getXMLAsString() {
        return JDomUtils.docToString(doc);
    }

    @Override
    public String toString() {
        return getXMLAsString();
    }
    
    private String name=NA_NAME;
    public String getName() {
        if (file!=null)
            return file.getAbsolutePath();
        return name;
    }
    
    
    public void setName( String name ) {
        this.name=name;
    }

    public Element getDetachedElement(){
        return getDocument().clone().getRootElement().detach(); 
    }
    
    public static List<Element> asDetachedElements(List<XMLInput> xmlInputs) {
        List<Element> elts=new ArrayList<Element>();
        for (XMLInput xmlInput:xmlInputs) {
            elts.add(xmlInput.getDetachedElement());
        }
        return elts;
    }
    
    public void clearAntlibNamespace(){
        JDomUtils.clearNamespace(doc.getRootElement(), "antlib");
    }
    
    public void removeIntroducedNamespaces() {
        IteratorIterable<Element> elts=doc.getRootElement().getDescendants( new ElementFilter() );
        for (Element elt : elts) {
            removeIntroducedNamespaces(elt);
        }
        removeIntroducedNamespaces( doc.getRootElement() );
    }
    
    private void removeIntroducedNamespaces(Element elt) {
        List<Namespace> namespaces=elt.getNamespacesIntroduced();
        for(Namespace namespace:namespaces) {
            elt.removeNamespaceDeclaration( namespace );
        }
    }
    
}
