package com.myarch.dpbuddy.xmlutil.streaming;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import org.apache.commons.io.FileUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CombiningInputStream extends InputStream {
    private Log logger = LogFactory.getLog(getClass());
    
    private Iterable<InputStream> inputStreams;
    private Iterator<InputStream> inputStreamsIter;
    private InputStream currentInputStream;

    public CombiningInputStream(Iterable<InputStream> inputStreams) {
        this.inputStreams=inputStreams;
        inputStreamsIter=inputStreams.iterator();
        if (inputStreamsIter.hasNext()) {
            currentInputStream=inputStreamsIter.next();
        }
    }

    private long byteCounter=0;
    private long LOGGING_SIZE=10*1024*1024; // 10MB
    
    @Override
    public int read() throws IOException {
        int c;
        
        if (currentInputStream == null) {
            return -1;
        }

        c=currentInputStream.read();
        if (c == -1) {

            while (inputStreamsIter.hasNext()) {
                currentInputStream=inputStreamsIter.next();
                c=currentInputStream.read();
                if (c != -1)
                    break;
            }
        }

        if (c != -1) {
            ++byteCounter;
            if (byteCounter%LOGGING_SIZE==0) {
                logger.info("Uploaded "+FileUtils.byteCountToDisplaySize( byteCounter ));
            }
        }
        else {
            if (byteCounter>LOGGING_SIZE) {
                logger.info( "Completed uploading "+FileUtils.byteCountToDisplaySize( byteCounter ));
            }
            byteCounter=0;
        }
        return c;
    }

    @Override
    public void close() throws IOException {
        for(InputStream is:inputStreams) {
            is.close();
        }
    }
    
}
