package com.myarch.dpbuddy.xmlutil.streaming;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

import org.jdom2.Element;
import org.jdom2.output.support.FormatStack;
import org.jdom2.util.NamespaceStack;

import com.myarch.dpbuddy.xmlutil.XMLOutputProcessorHelper;

public class ExternalContentChunkingOutputProcessor extends XMLOutputProcessorHelper {

    private ElementMapper<?> elementMapper;
    List<Object> chunks=new ArrayList<>();

    public ExternalContentChunkingOutputProcessor(ElementMapper<?> elementMapper) {
        this.elementMapper=elementMapper;
    }

    public List<Object> getChunks() {
        return chunks;
    }

    private int startChunkPos=-1;

    @Override
    protected void printElement( java.io.Writer out, FormatStack fstack, NamespaceStack nstack,
                    Element elt ) throws IOException {
        
        Object externalContent=elementMapper.mapElement( elt );

        if (externalContent != null) {
            printOpenTag( out, fstack, nstack, elt );
            processChunk(out);
            chunks.add( externalContent );

            printCloseTag( out, fstack, nstack, elt );
        }
        else {
            super.printElement( out, fstack, nstack, elt );
        }
    }

    public void complete(Writer out) {
        processChunk(out);
    }

    private void processChunk(Writer out) {
        // assuming string writer as the output
        String chunk=out.toString();
        if (startChunkPos >= 0) {
            chunk=chunk.substring( startChunkPos );
        }

        startChunkPos=out.toString().length();
        chunks.add( chunk );
        
    }

}
