package com.myarch.dpbuddy.xmlutil.streaming;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.output.StringBuilderWriter;
import org.jdom2.Document;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import com.myarch.dpbuddy.DPBuddyException;

public class ExternalContentXmlChunker {

    public static List<InputStream> chunk( Document xmlDoc,
                    ElementMapper<InputStream> eltToInputStream ) {
        Format format=Format.getCompactFormat();
        format.setOmitDeclaration( true );

        ExternalContentChunkingOutputProcessor chunker=new ExternalContentChunkingOutputProcessor(
                        eltToInputStream );
        XMLOutputter outputter=new XMLOutputter( format, chunker );

        StringBuilderWriter writer=new StringBuilderWriter();

        try {
            outputter.output( xmlDoc.getRootElement(), writer );
        }
        catch (IOException e) {
            throw new DPBuddyException( e );
        }

        chunker.complete( writer );

        return chunksToInputStreams( chunker.getChunks() );
    }

    private static List<InputStream> chunksToInputStreams( Iterable<Object> chunks ) {
        List<InputStream> inputStreams=new ArrayList<>();
        for (Object chunk : chunks) {
            if (chunk instanceof InputStream) {
                inputStreams.add( (InputStream) chunk );
            }
            else {
                ByteArrayInputStream bais=new ByteArrayInputStream( chunk.toString().getBytes() );
                inputStreams.add( bais );
            }
        }

        return inputStreams;
    }

}
