package com.myarch.environment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.PropertyHelper;
import org.apache.tools.ant.PropertyHelper.PropertyEvaluator;

public class PrefixedPropertyProvider implements PropertyProvider, PropertyEvaluator {

    final Log logger=LogFactory.getLog( getClass() );

    public final static String DEFAULT_SEPARATOR=".";
    public final static String ANY_SELECTOR_NAME="any";

    // legacy, not used
    private final static String PROPERTY_PROVIDER_TRACE_PROP_NAME="dpbuddy.trace.properties";

    private String prefixSeparator=DEFAULT_SEPARATOR;

    private Map<String, String> selectors=new HashMap<String, String>();

    public PrefixedPropertyProvider() {
        super();
    }

    @SuppressWarnings("unused")
    private void setSelectors( Map<String, String> newSelectors ) {
        selectors.clear();
        selectors.putAll( newSelectors );
        removeEmpty();
    }

    
    public void setSelector(String prefix) {
        selectors.clear();
        selectors.put( ANY_SELECTOR_NAME, prefix );
    }
    
    public String getSelector() {
        return selectors.get( ANY_SELECTOR_NAME );
    }
    
    private void removeEmpty() {
        List<String> emptySelectors=new ArrayList<String>();
        for (Map.Entry<String, String> entry : selectors.entrySet()) {
            if (StringUtils.isBlank( entry.getValue() ))
                emptySelectors.add( entry.getKey() );
        }

        for (String emptySelectorKey : emptySelectors) {
            selectors.remove( emptySelectorKey );
        }
    }

    public Object evaluate( String propertyName, PropertyHelper propertyHelper ) {
        Object value=null;
        
        if (PROPERTY_PROVIDER_TRACE_PROP_NAME.equalsIgnoreCase( propertyName ))
            return null;

        String selectorName=getSelectorFromPropertyName( propertyName );

        if (selectorName != null) {
            String propertyNameToSearch=propertyName;
            value=getPropertyValue( propertyHelper, selectorName, propertyNameToSearch );
        }

        if (value == null && selectors.containsKey( ANY_SELECTOR_NAME )) {
            value=getPropertyValue( propertyHelper, ANY_SELECTOR_NAME, propertyName );
        }
        // even if we return null here, the global property provider will kick
        // in
        return value;
    }

    private Object getPropertyValue( PropertyHelper propertyHelper, String selectorName,
                    String propertyName ) {
        String propValue=null;
        
        // create hierarchy of selectors
        String currentPrefix=(String) selectors.get( selectorName );

        while (StringUtils.isNotBlank( currentPrefix ) && propValue == null) {

            @SuppressWarnings("unchecked")
            Map<String, Object> propMap=(Map<String, Object>) propertyHelper.getProject().getReference(
                            currentPrefix );
            if (propMap != null) {
                propValue=propMap.get( propertyName ).toString();
            }
            else {
                String prefixedPropertyName=currentPrefix + prefixSeparator + propertyName;
                propValue=(String) propertyHelper.getProperties().get( prefixedPropertyName );
            }
            String displayedPropValue=propValue;
            if (propertyName.contains( "password" ))
                displayedPropValue="***";
            if (propValue != null)
                logger.debug( String.format( "Resolved property '%s'. Prefix: '%s' Value: '%s'",
                                propertyName, currentPrefix, displayedPropValue ) );

            currentPrefix=cutLastPrefix( currentPrefix );
        }

        return propValue;
    }

    
    
    private String cutLastPrefix( String prefixesString ) {
        int lastPrefixPos=prefixesString.lastIndexOf( '.' );
        if (lastPrefixPos <= 0)
            return null;
        else
            return prefixesString.substring( 0, lastPrefixPos );
    }

    private String getSelectorFromPropertyName( String name ) {
        String prefix=StringUtils.substringBefore( name, prefixSeparator );

        if (selectors.containsKey( prefix ))
            return prefix;
        else
            return null;
    }

}
