package com.myarch.eventmodel;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;

import com.google.gson.JsonObject;
import com.myarch.util.JsonUtils;

/*
 * Event example:
 *dpbuddy status -class FilesystemStatus -json
{"FreeEncrypted":"12830","TotalEncrypted":"14896","FreeTemporary":"1009","TotalTemporary":"1024","FreeInternal
":"1023","TotalInternal":"1024"}

payload_name:
dp:{status_type: domain:, url: FileSystemStatus:{ 
 */

public class Event {
    
    // log_file
    // log_host or collector_host
    
    public static final String HOST_NAME="host";
    
    private JsonObject json=new JsonObject();
    private EventType eventType;
    
    public Event(String techType, EventType eventType) {
        json.addProperty( "tech", techType );
        json.addProperty("event_type", eventType.toString());
        this.eventType=eventType;
    }
     
    public Event(String techType, EventType eventType, String url) {
        this(techType, eventType);
        setUrlAndHost(url);
    }
    
    public EventType eventType() {
        return this.eventType;
    }
    
    public void setHost(String host){
        json.addProperty( HOST_NAME, host ); 
    }
    
    
    public String host() {
        return JsonUtils.getString( json(), HOST_NAME );
    }
    
    public void setUrlAndHost(String urlS) {
        URL url;
        try {
            url=new URL(urlS);
        }
        catch (MalformedURLException e) {
            throw new EventException(e);
        }
        json.addProperty( "url", url.toString() );
        json.addProperty( HOST_NAME, url.getHost() );
    }
    
    public void setEnvironment (String environment) {
        json.addProperty( "environment", environment );
    }

    public void setNode (String node) {
        json.addProperty( "node", node );
    }
    
    
    public void setType (String type) {
        //TODO: Reconcile with ES _type
        json.addProperty( "type", type);
    }
    
    public JsonObject json() {
        return json;
    }
    
    public String get(String name) {
        return JsonUtils.getString( json, name );
    }
    
    public void setTimestamp(Date date) {
        json.addProperty( "@timestamp", JsonUtils.dateToIsoStr( date ));
    }
    
    public void set(String name, String val) {
        json.addProperty( name, val);
    }
    
    
    public void setBatchStart(boolean start) {
        // TODO: implement proper batch start
        json.addProperty( "set_to_latest", true);
    }
}
