package com.myarch.hocon;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.commons.io.FileUtils;
import org.apache.tools.ant.BuildException;
import org.kohsuke.args4j.Option;

import com.myarch.dpbuddy.ant.BaseDPBuddyTask;
import com.typesafe.config.Config;

public class ExportHoconTask extends BaseDPBuddyTask {

    @Option(name="-template",
                    usage="Template for each variable (line). Must contain two '%s' (Java format). The first '%s' is the name of the variable, the second '%s' is the value from the config file.")
    private String variableTemplateUnix="export %s=\"%s\"";

    private String pathDelim="_";
    @Option(name="-outFile", aliases="-file", required=true,
                    usage="Output file containing shell variables generated based on the current DPBuddy configuration.")
    private File outputFile;

    @Override
    public void executeDPTask() {

        HoconExporter exporter=new HoconExporter();
        String template=determineTemplate();
        Config config=HoconAntUtils.getConfig( getProject() );
        String exported=exporter.export( config, template, pathDelim );

        logger.info( String.format( "Exported current configuration to '%s'", outputFile.getAbsolutePath() ) );
        logger.debug( "Output of export:\n" + exported );
        try {
            FileUtils.write( outputFile, exported, Charset.defaultCharset() );
        }
        catch (IOException e) {
            throw new BuildException( e );
        }
    }

    private String determineTemplate() {
        return variableTemplateUnix;
    }

}
