package com.myarch.hocon;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.PropertyHelper;

import com.myarch.dpbuddy.DPBuddyException;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;

public class HoconAntUtils {

    private final static String HOCON_PROP_PROVIDER_REF_NAME="com.myarch.hocon.HoconAntPropertyProvider";

    public static void addConfig( Project project, File file ) {
        addConfig( project, ConfigFactory.parseFileAnySyntax( file ) );
    }

    public static void addConfig( Project project, String hoconStr ) {
        addConfig( project, ConfigFactory.parseString( hoconStr ) );
    }

    public static void setEnvironmentPrefix( Project project, String prefix ) {
        try {
            getProjectPropertyProvider( project ).setEnvironmentPrefix( prefix );
        }
        catch (Exception e) {
            if (e instanceof BuildException) {
                throw (BuildException) e;
            }
            else {
                throw new BuildException( e.getMessage(), new DPBuddyException( e ) );
            }
        }
    }

    public static void setScope( Project project, String scopePath ) {
        getProjectPropertyProvider( project ).setScope( scopePath );
    }

    public static void pushConfig( Project project, Config config ) {
        getProjectPropertyProvider( project ).pushConfig( config );
    }

    public static void popConfig( Project project ) {
        getProjectPropertyProvider( project ).popConfig();
    }

    public static Config getConfig( Project project ) {
        return getProjectPropertyProvider( project ).getCurrentConfig();
    }

    public static void addConfig( Project project, Config config ) {
        getProjectPropertyProvider( project ).addConfig( config );
    }

    public static HoconAntPropertyProvider getProjectPropertyProvider( Project project ) {
        Object propProviderObj=project.getReference( HOCON_PROP_PROVIDER_REF_NAME );
        HoconAntPropertyProvider propProvider;
        if (propProviderObj != null) {
            propProvider=(HoconAntPropertyProvider) propProviderObj;
        }
        else {
            propProvider=new HoconAntPropertyProvider();
            project.addReference( HOCON_PROP_PROVIDER_REF_NAME, propProvider );
        }
        PropertyHelper propertyHelper=PropertyHelper.getPropertyHelper( project );
        // Note that Ant makes sure that only one instance of a given
        // propertyProvider class exists
        // If the delegate was added before, Ant will replace it (using object
        // equality)
        propertyHelper.add( propProvider );

        return propProvider;

    }

    public static void copyToProject( Project fromProject, Project toProject ) {
        HoconAntPropertyProvider provider=getProjectPropertyProvider( fromProject );
        toProject.addReference( HOCON_PROP_PROVIDER_REF_NAME, provider );
        PropertyHelper propertyHelper=PropertyHelper.getPropertyHelper( toProject );
        propertyHelper.add( provider );
    }

}
