package com.myarch.hocon;

import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.BuildException;

import com.typesafe.config.Config;
import com.typesafe.config.ConfigObject;
import com.typesafe.config.ConfigValue;
import com.typesafe.config.ConfigValueType;

public class HoconEnvironmentUtils {

    @SuppressWarnings("unused")
    private static final Log logger=LogFactory.getLog( HoconEnvironmentUtils.class );
  

    public final static String ENV_CASE_KEYWORD="_env";
    public final static String DEFAULT_KEYWORD="_default";


    
    public static Object getUnwrappedConfigValue( Config config, String path, String environmentPrefix ) {
        Object val=null;
        ConfigValue configValue=getWrappedConfigValue( config, path, environmentPrefix );
        if (configValue!=null) {
            val=configValue.unwrapped();
        }
        
        return val;
    }
    
    public static ConfigValue getWrappedConfigValue( Config config, String path, String environmentPrefix ) {

        ConfigValue configValue=config.getValue( path );

        if (configValue.valueType() == ConfigValueType.OBJECT) {
            Config configObject=config.getObject( path ).toConfig();
            if (configObject.hasPath( ENV_CASE_KEYWORD )
                && configObject.getBoolean( ENV_CASE_KEYWORD )) {
                configValue=getValueForEnv( configObject, environmentPrefix );
                if (configValue == null) {
                    String msg;
                    if (StringUtils.isBlank( environmentPrefix )) {

                        msg=String.format(
                                        "Environment configuration property '%s' was requested, but the environment ('dp.env') was not provided. You can also provide '_default' value to resolve this",
                                        path );
                    }
                    else {
                        msg=String.format(
                                        "Environment configuration property '%s' doesn't have a value defined for the environment '%s'. You can also provide '_default' value to resolve this",
                                        path, environmentPrefix );
                    }

                    throw new BuildException( msg );
                }
            }
        }
        // TODO: support for array syntax a.1, a.2

        return configValue;
    }

    private static ConfigValue getValueForEnv( Config config, String environmentPrefix ) {

        ConfigValue val=null;
        if (StringUtils.isNotBlank( environmentPrefix ) && config.hasPath( environmentPrefix )) {
            val=config.getValue( environmentPrefix );
        }
        else if (config.hasPath( DEFAULT_KEYWORD )) {
            val=config.getValue( DEFAULT_KEYWORD );
        }

        return val;

    }

    public static HoconUtils.ConfObjWithKey findEnvironmentByHost(Config config, String host) {
        Set<Entry<String, ConfigValue>> rootEntries=config.root().entrySet();
        
        for(Entry<String, ConfigValue> entry:rootEntries) {
            ConfigValue configVal=entry.getValue();
            if (configVal!=null && configVal.valueType()==ConfigValueType.OBJECT ) {
               ConfigObject configObj=(ConfigObject) configVal;
               if (matchesHost(configObj, host)) {
                   return new HoconUtils.ConfObjWithKey(entry.getKey(), configObj);
               }
               
            }
        }
        
        return null;
    }
    
    private static boolean matchesHost(ConfigObject envConf, String hostToMatch) {
        
        hostToMatch=StringUtils.trim( hostToMatch );
        String host=HoconUtils.get(envConf, "host" );
        if (host != null) {
            return host.equalsIgnoreCase(hostToMatch);
        }
        host=HoconUtils.get(envConf, "dpHost" );
        if (host != null) {
            return host.equalsIgnoreCase(hostToMatch);
        }
        String url=HoconUtils.get(envConf, "dpUrl" );
        if (url != null) {
            return StringUtils.containsIgnoreCase( url, hostToMatch );
        }
        
        return false;
    }
    
    
    

}
