package com.myarch.hocon;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import com.myarch.antutil.AntCommonsLogger;
import com.myarch.dpbuddy.DPBuddyException;
import com.myarch.dpbuddy.ant.BaseDPBuddyTask;

public class HoconFileAntTask extends Task {

    public HoconFileAntTask() {
        AntCommonsLogger.init( BaseDPBuddyTask.ROOT_LOGGER );
    }

    private File file;
    private boolean isRequired=true;

    public void setFile( File file ) {
        this.file=file;
    }

    public void setRequired( boolean required ) {
        this.isRequired=required;
    }

    @Override
    public void execute() throws BuildException {
        BaseDPBuddyTask.setUpLogging( getProject() );
        try {

            Log logger=LogFactory.getLog( getClass() );

            if (file == null) {
                throw new BuildException(
                                "You must provide 'file' attribute, which is a path to the config file" );
            }

            if (file.exists()) {

                try {
                    logger.debug( "Loading configuration file from " + file.getAbsolutePath() );
                    HoconAntUtils.addConfig( getProject(), file );
                }
                catch (Exception e) {
                    if (e instanceof BuildException) {
                        throw (BuildException) e;
                    }
                    else {
                        // we're wrapping in DPBuddyException to force Ant not
                        // to dump the error stack for certain exceptions in
                        // non-verbose mode
                        throw new BuildException( e.getMessage(), new DPBuddyException( e ) );
                    }
                }
            }
            else if (isRequired) {
                throw new BuildException( "Config file " + file.getAbsolutePath() + " does not exist" );
            }
            else {
                logger.debug( String.format( "Config file %s was not found", file.getAbsolutePath() ) );
            }
        }
        finally {
            AntCommonsLogger.deregister();
        }
    }

}
