package com.myarch.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.util.StringUtils;
import org.jasypt.encryption.pbe.StandardPBEStringEncryptor;
import org.jasypt.exceptions.EncryptionOperationNotPossibleException;

public class DecryptionUtils {
    
    private static Log logger=LogFactory.getLog( DecryptionUtils.class );
    
    public static String decrypt( String encrStr, String masterPassword ) {
        StandardPBEStringEncryptor textEncryptor=new StandardPBEStringEncryptor();
        textEncryptor.setPassword( masterPassword );
        return textEncryptor.decrypt( encrStr );
    }

    public static String decryptIfEncrypted( String encrStr ) {
        String decrStr=encrStr;
        if (isEncypted( encrStr )) {
            String masterPassword=obtainMasterPassword();
            if (masterPassword == null) {
                throw new BuildException(
                                String.format( "Property '%s' appears to be encrypted, but the master password for decryption was not provided",
                                                encrStr ) );
            }

            decrStr=decryptIfEncrypted( encrStr, masterPassword );
        }

        return decrStr;
    }

    public static String decryptIfEncrypted( String encrStr, String masterPassword ) {
        String decrStr=encrStr;
        if (isEncypted( encrStr )) {
            try {
                decrStr=decrypt( stripWrapping( encrStr ), masterPassword );
            }
            catch (EncryptionOperationNotPossibleException eonp) {
                throw new BuildException( String.format(
                                "Unable to decrypt value '%s'."
                                    + " Make sure that the decryption password matches the encryption password",
                                encrStr ), eonp );
            }
        }

        return decrStr;
    }

    public static boolean isEncypted( String encrStr ) {
        return encrStr != null && encrStr.startsWith( ENCR_WRAP_PREFIX )
            && encrStr.endsWith( ENCR_WRAP_SUFFIX );
    }

    public final static String ENCR_WRAP_PREFIX="ENC{";
    public final static String ENCR_WRAP_SUFFIX="}";

    private static String stripWrapping( String encrStr ) {
        String unwrappedStr=StringUtils.removePrefix( encrStr, ENCR_WRAP_PREFIX );
        unwrappedStr=StringUtils.removeSuffix( unwrappedStr, ENCR_WRAP_SUFFIX );

        return unwrappedStr;
    }

    public final static String DEFAULT_MASTER_PASSWORD_PROP_NAME="dp.master.password";
    public final static String DEFAULT_MASTER_PASSWORD_ENV_NAME="DP_MASTER_PASSWORD";

    public static String obtainMasterPassword() {
        return obtainMasterPassword( DEFAULT_MASTER_PASSWORD_PROP_NAME,
                        DEFAULT_MASTER_PASSWORD_ENV_NAME );
    }

    private static String obtainMasterPassword( String masterPasswordPropName,
                    String masterPasswordEnvVarName ) {
        String masterPassword=System.getProperty( masterPasswordPropName );
        if (masterPassword == null) {
            logger.debug( "Encrypting using master password from system properties" );
            masterPassword=System.getenv( masterPasswordEnvVarName );
        }
        if (masterPassword == null) {
            logger.debug( "Encrypting using master password from environment" );
            masterPassword=System.getenv( masterPasswordPropName );
        }
        return masterPassword;
    }

    public static String encryptAndWrap( String str, String masterPassword ) {
        return ENCR_WRAP_PREFIX + encrypt( str, masterPassword ) + ENCR_WRAP_SUFFIX;
    }

    public static String encryptAndWrap( String str ) {
        
        return encryptAndWrap( str, obtainMasterPassword() );
    }

    public static String encrypt( String str, String masterPassword ) {
        StandardPBEStringEncryptor textEncryptor=new StandardPBEStringEncryptor();
        textEncryptor.setPassword( masterPassword );

        return textEncryptor.encrypt( str );
    }

}
