package com.myarch.util;

import java.util.Date;
import java.util.Map.Entry;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.time.FastDateFormat;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

public class JsonUtils {

    // ISO format as dictated by JSON convention, with offset from UTC
    public final static FastDateFormat ISO_FORMAT=FastDateFormat.getInstance("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
    public static String dateToIsoStr(Date date) {
        return ISO_FORMAT.format(date);
    }

    public static void addPropertyGuessType( JsonObject jsonObj, String name, String val ) {
        JsonPrimitive primitive=new JsonPrimitive( val );
        if (StringUtils.isNotBlank( val ) && StringUtils.isNumeric( val )) {
            primitive=new JsonPrimitive( Long.valueOf( val ) );
        }

        jsonObj.add( name, primitive );
    }

    public static void addPropertyIfNotBlankGuessType( JsonObject jsonObj, String name, String val ) {
        if (StringUtils.isNotBlank( val )) {
            addPropertyGuessType(jsonObj, name, val);
        }
    }
    
    
    public static void addPropertyIfNotBlank( JsonObject jsonObj, String name, String val ) {
        if (StringUtils.isNotBlank( val )) {
            jsonObj.addProperty( name, val );
        }
    }
 
    public static void addToArray(JsonObject jsonObj, String name, String val) {
        JsonElement jsonElt=jsonObj.get( name );
        JsonArray jsonArr=null;
        if (jsonElt!=null && jsonElt.isJsonArray()) {
            jsonArr=jsonElt.getAsJsonArray();
        }
        else {
            jsonArr=new JsonArray();
            jsonObj.add(name, jsonArr);
        }
        jsonArr.add(val);
       
    }
    
    public static JsonObject mergeJson( JsonObject fromJson, JsonObject toJson ) {
        for (Entry<String, JsonElement> jsonEltEntry : fromJson.entrySet()) {
            toJson.add( jsonEltEntry.getKey(), jsonEltEntry.getValue() );
        }

        return toJson;
    }

    public static String getString( JsonObject json, String name ) {
        String s=null;
        JsonElement member=json.get( name );
        if (member != null) {
            s=member.getAsString();
        }

        return s;
    }

}
